@@ -20,16 +20,14 @@
  */
 
 #include "ns3/core-module.h"
-#include "ns3/network-module.h"
 #include "ns3/applications-module.h"
 #include "ns3/wifi-module.h"
 #include "ns3/mobility-module.h"
-#include "ns3/ipv4-global-routing-helper.h"
 #include "ns3/internet-module.h"
 
 // This is a simple example in order to show how to configure an IEEE 802.11n Wi-Fi network.
 //
-// It ouputs the UDP or TCP goodput for every VHT bitrate value, which depends on the MCS value (0 to 7), the
+// It outputs the UDP or TCP goodput for every HT MCS value, which depends on the MCS value (0 to 7), the
 // channel width (20 or 40 MHz) and the guard interval (long or short). The PHY bitrate is constant over all
 // the simulation run. The user can also specify the distance between the access point and the station: the
 // larger the distance the smaller the goodput.
@@ -54,20 +52,40 @@ int main (int argc, char *argv[])
   double simulationTime = 10; //seconds
   double distance = 1.0; //meters
   double frequency = 5.0; //whether 2.4 or 5.0 GHz
-  
+  int mcs = -1; // -1 indicates an unset value
+  double minExpectedThroughput = 0;
+  double maxExpectedThroughput = 0;
+
   CommandLine cmd;
   cmd.AddValue ("frequency", "Whether working in the 2.4 or 5.0 GHz band (other values gets rejected)", frequency);
   cmd.AddValue ("distance", "Distance in meters between the station and the access point", distance);
   cmd.AddValue ("simulationTime", "Simulation time in seconds", simulationTime);
   cmd.AddValue ("udp", "UDP if set to 1, TCP otherwise", udp);
+  cmd.AddValue ("mcs", "if set, limit testing to a specific MCS (0-7)", mcs);
+  cmd.AddValue ("minExpectedThroughput", "if set, simulation fails if the lowest throughput is below this value", minExpectedThroughput);
+  cmd.AddValue ("maxExpectedThroughput", "if set, simulation fails if the highest throughput is above this value", maxExpectedThroughput);
   cmd.Parse (argc,argv);
 
-  std::cout << "MCS value" << "\t\t" << "Channel width" << "\t\t" << "short GI" << "\t\t" << "Throughput" << '\n';
-  for (int i = 0; i <= 7; i++) //MCS
+  double prevThroughput [8];
+  for (uint32_t l = 0; l < 8; l++)
     {
-      for (int j = 20; j <= 40; ) //channel width
+      prevThroughput[l] = 0;
+    }
+  std::cout << "MCS value" << "\t\t" << "Channel width" << "\t\t" << "short GI" << "\t\t" << "Throughput" << '\n';
+  int minMcs = 0;
+  int maxMcs = 7;
+  if (mcs >= 0 && mcs <= 7)
+    {
+      minMcs = mcs;
+      maxMcs = mcs;
+    }
+  for (int mcs = minMcs; mcs <= maxMcs; mcs++)
+    {
+      uint8_t index = 0;
+      double previous = 0;
+      for (int channelWidth = 20; channelWidth <= 40; )
         {
-          for (int k = 0; k < 2; k++) //GI: 0 and 1
+          for (int sgi = 0; sgi < 2; sgi++)
             {
               uint32_t payloadSize; //1500 byte IP packet
               if (udp)
@@ -90,7 +108,7 @@ int main (int argc, char *argv[])
               phy.SetChannel (channel.Create ());
 
               // Set guard interval
-              phy.Set ("ShortGuardEnabled", BooleanValue (k));
+              phy.Set ("ShortGuardEnabled", BooleanValue (sgi));
 
               WifiMacHelper mac;
               WifiHelper wifi;
@@ -105,15 +123,15 @@ int main (int argc, char *argv[])
                 }
               else
                 {
-                  std::cout<<"Wrong frequency value!"<<std::endl;
+                  std::cout << "Wrong frequency value!" << std::endl;
                   return 0;
                 }
 
               std::ostringstream oss;
-              oss << "HtMcs" << i;
+              oss << "HtMcs" << mcs;
               wifi.SetRemoteStationManager ("ns3::ConstantRateWifiManager","DataMode", StringValue (oss.str ()),
                                             "ControlMode", StringValue (oss.str ()));
-                
+
               Ssid ssid = Ssid ("ns3-80211n");
 
               mac.SetType ("ns3::StaWifiMac",
@@ -123,13 +141,14 @@ int main (int argc, char *argv[])
               staDevice = wifi.Install (phy, mac, wifiStaNode);
 
               mac.SetType ("ns3::ApWifiMac",
+                           "EnableBeaconJitter", BooleanValue (false),
                            "Ssid", SsidValue (ssid));
 
               NetDeviceContainer apDevice;
               apDevice = wifi.Install (phy, mac, wifiApNode);
 
               // Set channel width
-              Config::Set ("/NodeList/*/DeviceList/*/$ns3::WifiNetDevice/Phy/ChannelWidth", UintegerValue (j));
+              Config::Set ("/NodeList/*/DeviceList/*/$ns3::WifiNetDevice/Phy/ChannelWidth", UintegerValue (channelWidth));
 
               // mobility.
               MobilityHelper mobility;
@@ -150,7 +169,6 @@ int main (int argc, char *argv[])
               stack.Install (wifiStaNode);
 
               Ipv4AddressHelper address;
-
               address.SetBase ("192.168.1.0", "255.255.255.0");
               Ipv4InterfaceContainer staNodeInterface;
               Ipv4InterfaceContainer apNodeInterface;
@@ -159,21 +177,21 @@ int main (int argc, char *argv[])
               apNodeInterface = address.Assign (apDevice);
 
               /* Setting applications */
-              ApplicationContainer serverApp, sinkApp;
+              ApplicationContainer serverApp;
               if (udp)
                 {
                   //UDP flow
-                  UdpServerHelper myServer (9);
-                  serverApp = myServer.Install (wifiStaNode.Get (0));
+                  uint16_t port = 9;
+                  UdpServerHelper server (port);
+                  serverApp = server.Install (wifiStaNode.Get (0));
                   serverApp.Start (Seconds (0.0));
                   serverApp.Stop (Seconds (simulationTime + 1));
 
-                  UdpClientHelper myClient (staNodeInterface.GetAddress (0), 9);
-                  myClient.SetAttribute ("MaxPackets", UintegerValue (4294967295u));
-                  myClient.SetAttribute ("Interval", TimeValue (Time ("0.00001"))); //packets/s
-                  myClient.SetAttribute ("PacketSize", UintegerValue (payloadSize));
-
-                  ApplicationContainer clientApp = myClient.Install (wifiApNode.Get (0));
+                  UdpClientHelper client (staNodeInterface.GetAddress (0), port);
+                  client.SetAttribute ("MaxPackets", UintegerValue (4294967295u));
+                  client.SetAttribute ("Interval", TimeValue (Time ("0.00001"))); //packets/s
+                  client.SetAttribute ("PacketSize", UintegerValue (payloadSize));
+                  ApplicationContainer clientApp = client.Install (wifiApNode.Get (0));
                   clientApp.Start (Seconds (1.0));
                   clientApp.Stop (Seconds (simulationTime + 1));
                 }
@@ -181,25 +199,22 @@ int main (int argc, char *argv[])
                 {
                   //TCP flow
                   uint16_t port = 50000;
-                  Address apLocalAddress (InetSocketAddress (Ipv4Address::GetAny (), port));
-                  PacketSinkHelper packetSinkHelper ("ns3::TcpSocketFactory", apLocalAddress);
-                  sinkApp = packetSinkHelper.Install (wifiStaNode.Get (0));
+                  Address localAddress (InetSocketAddress (Ipv4Address::GetAny (), port));
+                  PacketSinkHelper packetSinkHelper ("ns3::TcpSocketFactory", localAddress);
+                  serverApp = packetSinkHelper.Install (wifiStaNode.Get (0));
+                  serverApp.Start (Seconds (0.0));
+                  serverApp.Stop (Seconds (simulationTime + 1));
 
-                  sinkApp.Start (Seconds (0.0));
-                  sinkApp.Stop (Seconds (simulationTime + 1));
-
-                  OnOffHelper onoff ("ns3::TcpSocketFactory",Ipv4Address::GetAny ());
+                  OnOffHelper onoff ("ns3::TcpSocketFactory", Ipv4Address::GetAny ());
                   onoff.SetAttribute ("OnTime",  StringValue ("ns3::ConstantRandomVariable[Constant=1]"));
                   onoff.SetAttribute ("OffTime", StringValue ("ns3::ConstantRandomVariable[Constant=0]"));
                   onoff.SetAttribute ("PacketSize", UintegerValue (payloadSize));
                   onoff.SetAttribute ("DataRate", DataRateValue (1000000000)); //bit/s
-                  ApplicationContainer apps;
-
                   AddressValue remoteAddress (InetSocketAddress (staNodeInterface.GetAddress (0), port));
                   onoff.SetAttribute ("Remote", remoteAddress);
-                  apps.Add (onoff.Install (wifiApNode.Get (0)));
-                  apps.Start (Seconds (1.0));
-                  apps.Stop (Seconds (simulationTime + 1));
+                  ApplicationContainer clientApp = onoff.Install (wifiApNode.Get (0));
+                  clientApp.Start (Seconds (1.0));
+                  clientApp.Stop (Seconds (simulationTime + 1));
                 }
 
               Ipv4GlobalRoutingHelper::PopulateRoutingTables ();
@@ -208,22 +223,59 @@ int main (int argc, char *argv[])
               Simulator::Run ();
               Simulator::Destroy ();
 
-              double throughput = 0;
+              uint64_t rxBytes = 0;
               if (udp)
                 {
-                  //UDP
-                  uint32_t totalPacketsThrough = DynamicCast<UdpServer> (serverApp.Get (0))->GetReceived ();
-                  throughput = totalPacketsThrough * payloadSize * 8 / (simulationTime * 1000000.0); //Mbit/s
+                  rxBytes = payloadSize * DynamicCast<UdpServer> (serverApp.Get (0))->GetReceived ();
                 }
               else
                 {
-                  //TCP
-                  uint32_t totalPacketsThrough = DynamicCast<PacketSink> (sinkApp.Get (0))->GetTotalRx ();
-                  throughput = totalPacketsThrough * 8 / (simulationTime * 1000000.0); //Mbit/s
+                  rxBytes = DynamicCast<PacketSink> (serverApp.Get (0))->GetTotalRx ();
                 }
-              std::cout << i << "\t\t\t" << j << " MHz\t\t\t" << k << "\t\t\t" << throughput << " Mbit/s" << std::endl;
+              double throughput = (rxBytes * 8) / (simulationTime * 1000000.0); //Mbit/s
+              std::cout << mcs << "\t\t\t" << channelWidth << " MHz\t\t\t" << sgi << "\t\t\t" << throughput << " Mbit/s" << std::endl;
+              //test first element
+              if (mcs == 0 && channelWidth == 20 && sgi == 0)
+                {
+                  if (throughput < minExpectedThroughput)
+                    {
+                      NS_LOG_ERROR ("Obtained throughput " << throughput << " is not expected!");
+                      exit (1);
+                    }
+                }
+              //test last element
+              if (mcs == 7 && channelWidth == 40 && sgi == 1)
+                {
+                  if (maxExpectedThroughput > 0 && throughput > maxExpectedThroughput)
+                    {
+                      NS_LOG_ERROR ("Obtained throughput " << throughput << " is not expected!");
+                      exit (1);
+                    }
+                }
+              //test previous throughput is smaller (for the same mcs)
+              if (throughput > previous)
+                {
+                  previous = throughput;
+                }
+              else
+                {
+                  NS_LOG_ERROR ("Obtained throughput " << throughput << " is not expected!");
+                  exit (1);
+                }
+              //test previous throughput is smaller (for the same channel width and GI)
+              if (throughput > prevThroughput [index])
+                {
+                  prevThroughput [index] = throughput;
+                }
+              else
+                {
+                  NS_LOG_ERROR ("Obtained throughput " << throughput << " is not expected!");
+                  exit (1);
+                }
+              index++;
+
             }
-          j *= 2;
+          channelWidth *= 2;
         }
     }
   return 0;
