@@ -22,7 +22,7 @@
  *
  * QUICK INSTRUCTIONS:
  *
- * To optimize build: 
+ * To optimize build:
  * ./waf -d optimized configure
  * ./waf
  *
@@ -49,20 +49,13 @@
  */
 
 #include "ns3/core-module.h"
-#include "ns3/network-module.h"
 #include "ns3/applications-module.h"
 #include "ns3/mobility-module.h"
 #include "ns3/stats-module.h"
-#include "ns3/random-variable-stream.h"
 #include "ns3/wifi-module.h"
 #include "ns3/internet-module.h"
 #include "ns3/flow-monitor-helper.h"
 #include "ns3/olsr-helper.h"
-#include "ns3/ipv4-static-routing-helper.h"
-#include "ns3/ipv4-list-routing-helper.h"
-
-#include <iostream>
-#include <fstream>
 
 using namespace ns3;
 
@@ -71,24 +64,40 @@ NS_LOG_COMPONENT_DEFINE ("multirate");
 class Experiment
 {
 public:
-
   Experiment ();
   Experiment (std::string name);
   Gnuplot2dDataset Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
                         const WifiMacHelper &wifiMac, const YansWifiChannelHelper &wifiChannel, const MobilityHelper &mobility);
 
   bool CommandSetup (int argc, char **argv);
-  bool IsRouting () { return (enableRouting == 1) ? 1 : 0; }
-  bool IsMobility () { return (enableMobility == 1) ? 1 : 0; }
+  bool IsRouting ()
+  {
+    return (enableRouting == 1) ? 1 : 0;
+  }
+  bool IsMobility ()
+  {
+    return (enableMobility == 1) ? 1 : 0;
+  }
 
-  uint32_t GetScenario () { return scenario; }
+  uint32_t GetScenario ()
+  {
+    return scenario;
+  }
 
-  std::string GetRtsThreshold () { return rtsThreshold; }
-  std::string GetOutputFileName () { return outputFileName; }
-  std::string GetRateManager () { return rateManager; }
+  std::string GetRtsThreshold ()
+  {
+    return rtsThreshold;
+  }
+  std::string GetOutputFileName ()
+  {
+    return outputFileName;
+  }
+  std::string GetRateManager ()
+  {
+    return rateManager;
+  }
 
 private:
-
   Ptr<Socket> SetupPacketReceive (Ptr<Node> node);
   NodeContainer GenerateNeighbors (NodeContainer c, uint32_t senderId);
 
@@ -101,13 +110,13 @@ private:
 
   Gnuplot2dDataset m_output;
 
-  double totalTime; 
+  double totalTime;
   double expMean;
   double samplingPeriod;
 
   uint32_t bytesTotal;
   uint32_t packetSize;
-  uint32_t gridSize; 
+  uint32_t gridSize;
   uint32_t nodeDistance;
   uint32_t port;
   uint32_t scenario;
@@ -118,7 +127,7 @@ private:
   bool enableRouting;
   bool enableMobility;
 
-  NodeContainer containerA, containerB, containerC, containerD; 
+  NodeContainer containerA, containerB, containerC, containerD;
   std::string rtsThreshold, rateManager, outputFileName;
 };
 
@@ -126,25 +135,28 @@ Experiment::Experiment ()
 {
 }
 
-Experiment::Experiment (std::string name) :
-  m_output (name),
-  totalTime (0.3),
-  expMean (0.1), //flows being exponentially distributed
-  samplingPeriod(0.1),
-  bytesTotal (0),
-  packetSize (2000),
-  gridSize (10), //10x10 grid  for a total of 100 nodes
-  nodeDistance (30),
-  port (5000),
-  scenario (4),
-  enablePcap (false),
-  enableTracing (true),
-  enableFlowMon (false),
-  enableRouting (false),
-  enableMobility (false),
-  rtsThreshold ("2200"), //0 for enabling rts/cts
-  rateManager ("ns3::MinstrelWifiManager"),
-  outputFileName ("minstrel")
+Experiment::Experiment (std::string name)
+  : m_output (name),
+    totalTime (0.3),
+    expMean (0.1),
+    //flows being exponentially distributed
+    samplingPeriod (0.1),
+    bytesTotal (0),
+    packetSize (2000),
+    gridSize (10),
+    //10x10 grid  for a total of 100 nodes
+    nodeDistance (30),
+    port (5000),
+    scenario (4),
+    enablePcap (false),
+    enableTracing (true),
+    enableFlowMon (false),
+    enableRouting (false),
+    enableMobility (false),
+    rtsThreshold ("2200"),
+    //0 for enabling rts/cts
+    rateManager ("ns3::MinstrelWifiManager"),
+    outputFileName ("minstrel")
 {
   m_output.SetStyle (Gnuplot2dDataset::LINES);
 }
@@ -174,7 +186,7 @@ Experiment::ReceivePacket (Ptr<Socket> socket)
 void
 Experiment::CheckThroughput ()
 {
-  double mbs = ((bytesTotal * 8.0) /1000000 /samplingPeriod);
+  double mbs = ((bytesTotal * 8.0) / 1000000 / samplingPeriod);
   bytesTotal = 0;
   m_output.Add ((Simulator::Now ()).GetSeconds (), mbs);
 
@@ -185,39 +197,39 @@ Experiment::CheckThroughput ()
 /**
  *
  * Take the grid map, divide it into 4 quadrants
- * Assign all nodes from each quadrant to a specific container 
- * 
+ * Assign all nodes from each quadrant to a specific container
+ *
  */
 void
 Experiment::AssignNeighbors (NodeContainer c)
 {
   uint32_t totalNodes = c.GetN ();
-  for (uint32_t i=0; i< totalNodes; i++)
+  for (uint32_t i = 0; i < totalNodes; i++)
     {
-      if ( (i % gridSize) <= (gridSize/2 - 1))
+      if ( (i % gridSize) <= (gridSize / 2 - 1))
         {
           //lower left quadrant
-          if ( i < totalNodes/2 )
+          if ( i < totalNodes / 2 )
             {
               containerA.Add (c.Get (i));
             }
 
           //upper left quadrant
-          if ( i >= (uint32_t)(4*totalNodes)/10 )
+          if ( i >= (uint32_t)(4 * totalNodes) / 10 )
             {
               containerC.Add (c.Get (i));
             }
         }
-      if ( (i % gridSize) >= (gridSize/2 - 1))
+      if ( (i % gridSize) >= (gridSize / 2 - 1))
         {
           //lower right quadrant
-          if ( i < totalNodes/2 )
+          if ( i < totalNodes / 2 )
             {
               containerB.Add (c.Get (i));
             }
 
           //upper right quadrant
-          if ( i >= (uint32_t)(4*totalNodes)/10  )
+          if ( i >= (uint32_t)(4 * totalNodes) / 10  )
             {
               containerD.Add (c.Get (i));
             }
@@ -234,7 +246,7 @@ Experiment::GenerateNeighbors (NodeContainer c, uint32_t senderId)
 {
   NodeContainer nc;
   uint32_t limit = senderId + 2;
-  for (uint32_t i= senderId - 2; i <= limit; i++)
+  for (uint32_t i = senderId - 2; i <= limit; i++)
     {
       //must ensure the boundaries for other topologies
       nc.Add (c.Get (i));
@@ -247,9 +259,9 @@ Experiment::GenerateNeighbors (NodeContainer c, uint32_t senderId)
 }
 
 /**
- * Sources and destinations are randomly selected such that a node 
- * may be the source for multiple destinations and a node maybe a destination 
- * for multiple sources. 
+ * Sources and destinations are randomly selected such that a node
+ * may be the source for multiple destinations and a node maybe a destination
+ * for multiple sources.
  */
 void
 Experiment::SelectSrcDest (NodeContainer c)
@@ -257,12 +269,12 @@ Experiment::SelectSrcDest (NodeContainer c)
   uint32_t totalNodes = c.GetN ();
   Ptr<UniformRandomVariable> uvSrc = CreateObject<UniformRandomVariable> ();
   uvSrc->SetAttribute ("Min", DoubleValue (0));
-  uvSrc->SetAttribute ("Max", DoubleValue (totalNodes/2 -1));
+  uvSrc->SetAttribute ("Max", DoubleValue (totalNodes / 2 - 1));
   Ptr<UniformRandomVariable> uvDest = CreateObject<UniformRandomVariable> ();
-  uvDest->SetAttribute ("Min", DoubleValue (totalNodes/2));
+  uvDest->SetAttribute ("Min", DoubleValue (totalNodes / 2));
   uvDest->SetAttribute ("Max", DoubleValue (totalNodes));
 
-  for (uint32_t i=0; i < totalNodes/3; i++)
+  for (uint32_t i = 0; i < totalNodes / 3; i++)
     {
       ApplicationSetup (c.Get (uvSrc->GetInteger ()), c.Get (uvDest->GetInteger ()),  0, totalTime);
     }
@@ -288,23 +300,25 @@ Experiment::SendMultiDestinations (Ptr<Node> sender, NodeContainer c)
   ev->SetAttribute ("Mean", DoubleValue (expMean));
   ev->SetAttribute ("Bound", DoubleValue (totalTime));
 
-  double start=0.0, stop=totalTime;
-  uint32_t destIndex; 
+  double start = 0.0, stop;
+  uint32_t destIndex;
 
-  for (uint32_t i=0; i < c.GetN (); i++)
+  for (uint32_t i = 0; i < c.GetN (); i++)
     {
       stop = start + ev->GetValue ();
       NS_LOG_DEBUG ("Start=" << start << " Stop=" << stop);
 
-      do {
+      do
+        {
           destIndex = (uint32_t) uv->GetValue ();
-        } while ( (c.Get (destIndex))->GetId () == sender->GetId ());
+        }
+      while ( (c.Get (destIndex))->GetId () == sender->GetId ());
 
       ApplicationSetup (sender, c.Get (destIndex),  start, stop);
 
       start = stop;
 
-      if(start > totalTime) 
+      if (start > totalTime)
         {
           break;
         }
@@ -324,8 +338,8 @@ PrintPosition (Ptr<Node> client, Ptr<Node> server)
   Vector serverPos = GetPosition (server);
   Vector clientPos = GetPosition (client);
 
-  Ptr<Ipv4> ipv4Server = server->GetObject<Ipv4>();
-  Ptr<Ipv4> ipv4Client = client->GetObject<Ipv4>();
+  Ptr<Ipv4> ipv4Server = server->GetObject<Ipv4> ();
+  Ptr<Ipv4> ipv4Client = client->GetObject<Ipv4> ();
 
   Ipv4InterfaceAddress iaddrServer = ipv4Server->GetAddress (1,0);
   Ipv4InterfaceAddress iaddrClient = ipv4Client->GetAddress (1,0);
@@ -335,13 +349,13 @@ PrintPosition (Ptr<Node> client, Ptr<Node> server)
 
   std::ostringstream oss;
   oss << "Set up Server Device " <<  (server->GetDevice (0))->GetAddress ()
-                                 << " with ip " << ipv4AddrServer
-                                 << " position (" << serverPos.x << "," << serverPos.y << "," << serverPos.z << ")";
+      << " with ip " << ipv4AddrServer
+      << " position (" << serverPos.x << "," << serverPos.y << "," << serverPos.z << ")";
 
   oss << "Set up Client Device " <<  (client->GetDevice (0))->GetAddress ()
-                                 << " with ip " << ipv4AddrClient
-                                 << " position (" << clientPos.x << "," << clientPos.y << "," << clientPos.z << ")"
-                                 << "\n";
+      << " with ip " << ipv4AddrClient
+      << " position (" << clientPos.x << "," << clientPos.y << "," << clientPos.z << ")"
+      << "\n";
   return oss.str ();
 }
 
@@ -355,7 +369,7 @@ Experiment::ApplicationSetup (Ptr<Node> client, Ptr<Node> server, double start,
 
   NS_LOG_DEBUG (PrintPosition (client, server));
 
-  // Equipping the source  node with OnOff Application used for sending 
+  // Equipping the source  node with OnOff Application used for sending
   OnOffHelper onoff ("ns3::UdpSocketFactory", Address (InetSocketAddress (Ipv4Address ("10.0.0.1"), port)));
   onoff.SetConstantRate (DataRate (60000000));
   onoff.SetAttribute ("PacketSize", UintegerValue (packetSize));
@@ -375,7 +389,7 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
 {
 
 
-  uint32_t nodeSize = gridSize*gridSize;
+  uint32_t nodeSize = gridSize * gridSize;
   NodeContainer c;
   c.Create (nodeSize);
 
@@ -412,7 +426,7 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
   Ipv4InterfaceContainer ipInterfaces;
   ipInterfaces = address.Assign (devices);
 
-  MobilityHelper mobil= mobility;
+  MobilityHelper mobil = mobility;
   mobil.SetPositionAllocator ("ns3::GridPositionAllocator",
                               "MinX", DoubleValue (0.0),
                               "MinY", DoubleValue (0.0),
@@ -444,9 +458,9 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
   else if ( scenario == 2)
     {
       //All flows begin at the same time
-      for (uint32_t i = 0; i < nodeSize - 1; i = i+2)
+      for (uint32_t i = 0; i < nodeSize - 1; i = i + 2)
         {
-          ApplicationSetup (c.Get (i), c.Get (i+1),  0, totalTime);
+          ApplicationSetup (c.Get (i), c.Get (i + 1),  0, totalTime);
         }
     }
   else if ( scenario == 3)
@@ -454,7 +468,7 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
       AssignNeighbors (c);
       //Note: these senders are hand-picked in order to ensure good coverage
       //for 10x10 grid, basically one sender for each quadrant
-      //you might have to change these values for other grids 
+      //you might have to change these values for other grids
       NS_LOG_DEBUG (">>>>>>>>>region A<<<<<<<<<");
       SendMultiDestinations (c.Get (22), containerA);
 
@@ -471,18 +485,18 @@ Experiment::Run (const WifiHelper &wifi, const YansWifiPhyHelper &wifiPhy,
     {
       //GenerateNeighbors(NodeContainer, uint32_t sender)
       //Note: these senders are hand-picked in order to ensure good coverage
-      //you might have to change these values for other grids 
+      //you might have to change these values for other grids
       NodeContainer c1, c2, c3, c4, c5, c6, c7, c8, c9;
 
       c1 = GenerateNeighbors (c, 22);
-      c2 = GenerateNeighbors (c, 24);;
-      c3 = GenerateNeighbors (c, 26);;
-      c4 = GenerateNeighbors (c, 42);;
-      c5 = GenerateNeighbors (c, 44);;
-      c6 = GenerateNeighbors (c, 46);;
-      c7 = GenerateNeighbors (c, 62);;
-      c8 = GenerateNeighbors (c, 64);;
-      c9 = GenerateNeighbors (c, 66);;
+      c2 = GenerateNeighbors (c, 24);
+      c3 = GenerateNeighbors (c, 26);
+      c4 = GenerateNeighbors (c, 42);
+      c5 = GenerateNeighbors (c, 44);
+      c6 = GenerateNeighbors (c, 46);
+      c7 = GenerateNeighbors (c, 62);
+      c8 = GenerateNeighbors (c, 64);
+      c9 = GenerateNeighbors (c, 66);
 
       SendMultiDestinations (c.Get (22), c1);
       SendMultiDestinations (c.Get (24), c2);
@@ -538,11 +552,11 @@ Experiment::CommandSetup (int argc, char **argv)
   // according to totalTime, select an appropriate samplingPeriod automatically.
   if (totalTime < 1.0)
     {
-	  samplingPeriod = 0.1;
+      samplingPeriod = 0.1;
     }
   else
     {
-	  samplingPeriod = 1.0;
+      samplingPeriod = 1.0;
     }
   // or user selects a samplingPeriod.
   cmd.AddValue ("samplingPeriod", "sampling period", samplingPeriod);
@@ -570,7 +584,7 @@ int main (int argc, char *argv[])
   Config::SetDefault ("ns3::WifiRemoteStationManager::FragmentationThreshold", StringValue ("2200"));
   Config::SetDefault ("ns3::WifiRemoteStationManager::RtsCtsThreshold", StringValue (experiment.GetRtsThreshold ()));
 
-  std::ofstream outfile ((experiment.GetOutputFileName ()+ ".plt").c_str ());
+  std::ofstream outfile ((experiment.GetOutputFileName () + ".plt").c_str ());
 
   MobilityHelper mobility;
   Gnuplot gnuplot;
