@@ -50,18 +50,12 @@
  * export NS_LOG=RateAdaptationDistance=level_info
  */
 
-#include <sstream>
-#include <fstream>
-#include <math.h>
-
 #include "ns3/core-module.h"
-#include "ns3/network-module.h"
 #include "ns3/internet-module.h"
 #include "ns3/mobility-module.h"
 #include "ns3/wifi-module.h"
 #include "ns3/applications-module.h"
 #include "ns3/stats-module.h"
-#include "ns3/flow-monitor-module.h"
 
 using namespace ns3;
 using namespace std;
@@ -193,7 +187,7 @@ int main (int argc, char *argv[])
   YansWifiChannelHelper wifiChannel = YansWifiChannelHelper::Default ();
   wifiPhy.SetChannel (wifiChannel.Create ());
 
-  wifiPhy.Set("ShortGuardEnabled", BooleanValue(shortGuardInterval));
+  wifiPhy.Set ("ShortGuardEnabled", BooleanValue (shortGuardInterval));
 
   NetDeviceContainer wifiApDevices;
   NetDeviceContainer wifiStaDevices;
@@ -214,7 +208,7 @@ int main (int argc, char *argv[])
         {
           wifi.SetStandard (WIFI_PHY_STANDARD_80211g);
         }
-      NqosWifiMacHelper wifiMac = NqosWifiMacHelper::Default ();
+      WifiMacHelper wifiMac;
 
       //Configure the STA node
       wifi.SetRemoteStationManager (staManager, "RtsCtsThreshold", UintegerValue (rtsThreshold));
@@ -243,7 +237,7 @@ int main (int argc, char *argv[])
           wifi.SetStandard (WIFI_PHY_STANDARD_80211n_5GHZ);
         }
 
-      HtWifiMacHelper wifiMac = HtWifiMacHelper::Default ();
+      WifiMacHelper wifiMac;
 
       //Configure the STA node
       wifi.SetRemoteStationManager (staManager, "RtsCtsThreshold", UintegerValue (rtsThreshold));
@@ -266,7 +260,7 @@ int main (int argc, char *argv[])
   else if (standard == "802.11ac")
     {
       wifi.SetStandard (WIFI_PHY_STANDARD_80211ac);
-      VhtWifiMacHelper wifiMac = VhtWifiMacHelper::Default ();
+      WifiMacHelper wifiMac;
 
       //Configure the STA node
       wifi.SetRemoteStationManager (staManager, "RtsCtsThreshold", UintegerValue (rtsThreshold));
