@@ -17,7 +17,6 @@
  *
  */
 
-// 
 // This script configures three nodes on an 802.11b physical layer, with
 // 802.11b NICs in adhoc mode.  There is a transmitter, receiver, and
 // interferer.  The transmitter sends one packet to the receiver and
@@ -33,7 +32,7 @@
 //
 //     |------------------------------------|
 //     |                                    |
-//     | primary received frame (time t0)   | 
+//     | primary received frame (time t0)   |
 //     |                                    |
 //     |------------------------------------|
 //
@@ -42,8 +41,8 @@
 //         |-----------------------------------|
 //         |                                   |
 //         |  interfering frame (time t1)      |
-//         |                                   | 
-//         |-----------------------------------| 
+//         |                                   |
+//         |-----------------------------------|
 //
 // The orientation is:
 //     n2  ---------> n0 <---------- n1
@@ -66,7 +65,7 @@
 //
 // This script can also be helpful to put the Wifi layer into verbose
 // logging mode; this command will turn on all wifi logging:
-// 
+//
 // ./waf --run "wifi-simple-interference --verbose=1"
 //
 // When you are done, you will notice a pcap trace file in your directory.
@@ -81,17 +80,10 @@
 // ./waf --run "wifi-simple-interference --delta=30000"
 
 #include "ns3/core-module.h"
-#include "ns3/network-module.h"
 #include "ns3/mobility-module.h"
-#include "ns3/config-store-module.h"
 #include "ns3/wifi-module.h"
 #include "ns3/internet-module.h"
 
-#include <iostream>
-#include <fstream>
-#include <vector>
-#include <string>
-
 using namespace ns3;
 
 NS_LOG_COMPONENT_DEFINE ("WifiSimpleInterference");
@@ -118,14 +110,14 @@ static void ReceivePacket (Ptr<Socket> socket)
   NS_LOG_UNCOND (PrintReceivedPacket (socket));
 }
 
-static void GenerateTraffic (Ptr<Socket> socket, uint32_t pktSize, 
+static void GenerateTraffic (Ptr<Socket> socket, uint32_t pktSize,
                              uint32_t pktCount, Time pktInterval )
 {
   if (pktCount > 0)
     {
       socket->Send (Create<Packet> (pktSize));
-      Simulator::Schedule (pktInterval, &GenerateTraffic, 
-                           socket, pktSize,pktCount-1, pktInterval);
+      Simulator::Schedule (pktInterval, &GenerateTraffic,
+                           socket, pktSize,pktCount - 1, pktInterval);
     }
   else
     {
@@ -133,11 +125,10 @@ static void GenerateTraffic (Ptr<Socket> socket, uint32_t pktSize,
     }
 }
 
-
 int main (int argc, char *argv[])
 {
 //  LogComponentEnable ("InterferenceHelper", LOG_LEVEL_ALL);
- 
+
   std::string phyMode ("DsssRate1Mbps");
   double Prss = -80;  // -dBm
   double Irss = -95;  // -dBm
@@ -152,7 +143,7 @@ int main (int argc, char *argv[])
   double startTime = 10.0; // seconds
   double distanceToRx = 100.0; // meters
 
-  double offset = 91;  // This is a magic number used to set the 
+  double offset = 91;  // This is a magic number used to set the
                        // transmit power, based on other configuration
   CommandLine cmd;
 
@@ -173,7 +164,7 @@ int main (int argc, char *argv[])
   // turn off RTS/CTS for frames below 2200 bytes
   Config::SetDefault ("ns3::WifiRemoteStationManager::RtsCtsThreshold", StringValue ("2200"));
   // Fix non-unicast data rate to be the same as that of unicast
-  Config::SetDefault ("ns3::WifiRemoteStationManager::NonUnicastMode", 
+  Config::SetDefault ("ns3::WifiRemoteStationManager::NonUnicastMode",
                       StringValue (phyMode));
 
   NodeContainer c;
@@ -189,11 +180,11 @@ int main (int argc, char *argv[])
 
   YansWifiPhyHelper wifiPhy =  YansWifiPhyHelper::Default ();
   // set it to zero; otherwise, gain will be added
-  wifiPhy.Set ("RxGain", DoubleValue (0) ); 
+  wifiPhy.Set ("RxGain", DoubleValue (0) );
   wifiPhy.Set ("CcaMode1Threshold", DoubleValue (0.0) );
 
   // ns-3 supports RadioTap and Prism tracing extensions for 802.11b
-  wifiPhy.SetPcapDataLinkType (YansWifiPhyHelper::DLT_IEEE802_11_RADIO); 
+  wifiPhy.SetPcapDataLinkType (YansWifiPhyHelper::DLT_IEEE802_11_RADIO);
 
   YansWifiChannelHelper wifiChannel;
   wifiChannel.SetPropagationDelay ("ns3::ConstantSpeedPropagationDelayModel");
@@ -208,21 +199,21 @@ int main (int argc, char *argv[])
   // Set it to adhoc mode
   wifiMac.SetType ("ns3::AdhocWifiMac");
   NetDeviceContainer devices = wifi.Install (wifiPhy, wifiMac, c.Get (0));
-  // This will disable these sending devices from detecting a signal 
+  // This will disable these sending devices from detecting a signal
   // so that they do not backoff
   wifiPhy.Set ("EnergyDetectionThreshold", DoubleValue (0.0) );
-  wifiPhy.Set ("TxGain", DoubleValue (offset + Prss) ); 
+  wifiPhy.Set ("TxGain", DoubleValue (offset + Prss) );
   devices.Add (wifi.Install (wifiPhy, wifiMac, c.Get (1)));
-  wifiPhy.Set ("TxGain", DoubleValue (offset + Irss) ); 
+  wifiPhy.Set ("TxGain", DoubleValue (offset + Irss) );
   devices.Add (wifi.Install (wifiPhy, wifiMac, c.Get (2)));
 
-  // Note that with FixedRssLossModel, the positions below are not 
-  // used for received signal strength. 
+  // Note that with FixedRssLossModel, the positions below are not
+  // used for received signal strength.
   MobilityHelper mobility;
   Ptr<ListPositionAllocator> positionAlloc = CreateObject<ListPositionAllocator> ();
   positionAlloc->Add (Vector (0.0, 0.0, 0.0));
   positionAlloc->Add (Vector (distanceToRx, 0.0, 0.0));
-  positionAlloc->Add (Vector (-1*distanceToRx, 0.0, 0.0));
+  positionAlloc->Add (Vector (-1 * distanceToRx, 0.0, 0.0));
   mobility.SetPositionAllocator (positionAlloc);
   mobility.SetMobilityModel ("ns3::ConstantPositionMobilityModel");
   mobility.Install (c);
@@ -260,11 +251,11 @@ int main (int argc, char *argv[])
   NS_LOG_UNCOND ("Primary packet RSS=" << Prss << " dBm and interferer RSS=" << Irss << " dBm at time offset=" << delta << " ms");
 
   Simulator::ScheduleWithContext (source->GetNode ()->GetId (),
-                                  Seconds (startTime), &GenerateTraffic, 
+                                  Seconds (startTime), &GenerateTraffic,
                                   source, PpacketSize, numPackets, interPacketInterval);
 
   Simulator::ScheduleWithContext (interferer->GetNode ()->GetId (),
-                                  Seconds (startTime + delta/1000000.0), &GenerateTraffic, 
+                                  Seconds (startTime + delta / 1000000.0), &GenerateTraffic,
                                   interferer, IpacketSize, numPackets, interPacketInterval);
 
   Simulator::Run ();
@@ -272,4 +263,3 @@ int main (int argc, char *argv[])
 
   return 0;
 }
-
