@@ -31,7 +31,7 @@
 /**
  * \file
  * \ingroup config
- * Imlementation of the various ns3::Config functions and classes.
+ * ns3::Config implementations.
  */
 
 namespace ns3 {
@@ -146,10 +146,11 @@ MatchContainer::DisconnectWithoutContext (std::string name, const CallbackBase &
     }
 }
 
-} // namespace Config
 
-
-/** Helper to test if an array entry matches a config path specification. */
+/**
+ * \ingroup config-impl
+ * Helper to test if an array entry matches a config path specification.
+ */
 class ArrayMatcher
 {
 public:
@@ -177,7 +178,8 @@ private:
   bool StringToUint32 (std::string str, uint32_t *value) const;
   /** The Config path element. */
   std::string m_element;
-};
+
+};  // class ArrayMatcher
 
 
 ArrayMatcher::ArrayMatcher (std::string element)
@@ -260,6 +262,7 @@ ArrayMatcher::StringToUint32 (std::string str, uint32_t *value) const
 }
 
 /**
+ * \ingroup config-impl
  * Abstract class to parse Config paths into object references.
  */
 class Resolver
@@ -325,7 +328,8 @@ private:
   std::vector<std::string> m_workStack;
   /** The Config path. */
   std::string m_path;
-};
+
+};  // class Resolver
 
 Resolver::Resolver (std::string path)
   : m_path (path)
@@ -572,7 +576,10 @@ Resolver::DoArrayResolve (std::string path, const ObjectPtrContainerValue &conta
     }
 }
 
-/** Config system implementation class. */
+/**
+ * \ingroup config-impl
+ * Config system implementation class.
+ */
 class ConfigImpl : public Singleton<ConfigImpl>
 {
 public:
@@ -587,7 +594,7 @@ public:
   /** \copydoc Config::Disconnect() */
   void Disconnect (std::string path, const CallbackBase &cb);
   /** \copydoc Config::LookupMatches() */
-  Config::MatchContainer LookupMatches (std::string path);
+  MatchContainer LookupMatches (std::string path);
 
   /** \copydoc Config::RegisterRootNamespaceObject() */
   void RegisterRootNamespaceObject (Ptr<Object> obj);
@@ -614,7 +621,8 @@ private:
 
   /** The list of Config path roots. */
   Roots m_roots;
-};
+
+};  // class ConfigImpl
 
 void 
 ConfigImpl::ParsePath (std::string path, std::string *root, std::string *leaf) const
@@ -635,7 +643,7 @@ ConfigImpl::Set (std::string path, const AttributeValue &value)
 
   std::string root, leaf;
   ParsePath (path, &root, &leaf);
-  Config::MatchContainer container = LookupMatches (root);
+  MatchContainer container = LookupMatches (root);
   container.Set (leaf, value);
 }
 void 
@@ -644,7 +652,7 @@ ConfigImpl::ConnectWithoutContext (std::string path, const CallbackBase &cb)
   NS_LOG_FUNCTION (this << path << &cb);
   std::string root, leaf;
   ParsePath (path, &root, &leaf);
-  Config::MatchContainer container = LookupMatches (root);
+  MatchContainer container = LookupMatches (root);
   container.ConnectWithoutContext (leaf, cb);
 }
 void 
@@ -653,7 +661,7 @@ ConfigImpl::DisconnectWithoutContext (std::string path, const CallbackBase &cb)
   NS_LOG_FUNCTION (this << path << &cb);
   std::string root, leaf;
   ParsePath (path, &root, &leaf);
-  Config::MatchContainer container = LookupMatches (root);
+  MatchContainer container = LookupMatches (root);
   container.DisconnectWithoutContext (leaf, cb);
 }
 void 
@@ -663,7 +671,7 @@ ConfigImpl::Connect (std::string path, const CallbackBase &cb)
 
   std::string root, leaf;
   ParsePath (path, &root, &leaf);
-  Config::MatchContainer container = LookupMatches (root);
+  MatchContainer container = LookupMatches (root);
   container.Connect (leaf, cb);
 }
 void 
@@ -673,11 +681,11 @@ ConfigImpl::Disconnect (std::string path, const CallbackBase &cb)
 
   std::string root, leaf;
   ParsePath (path, &root, &leaf);
-  Config::MatchContainer container = LookupMatches (root);
+  MatchContainer container = LookupMatches (root);
   container.Disconnect (leaf, cb);
 }
 
-Config::MatchContainer 
+MatchContainer 
 ConfigImpl::LookupMatches (std::string path)
 {
   NS_LOG_FUNCTION (this << path);
@@ -687,7 +695,8 @@ ConfigImpl::LookupMatches (std::string path)
     LookupMatchesResolver (std::string path)
       : Resolver (path)
     {}
-    virtual void DoOne (Ptr<Object> object, std::string path) {
+    virtual void DoOne (Ptr<Object> object, std::string path)
+    {
       m_objects.push_back (object);
       m_contexts.push_back (path);
     }
@@ -706,7 +715,7 @@ ConfigImpl::LookupMatches (std::string path)
   //
   resolver.Resolve (0);
 
-  return Config::MatchContainer (resolver.m_objects, resolver.m_contexts, path);
+  return MatchContainer (resolver.m_objects, resolver.m_contexts, path);
 }
 
 void 
@@ -744,7 +753,6 @@ ConfigImpl::GetRootNamespaceObject (uint32_t i) const
   return m_roots[i];
 }
 
-namespace Config {
 
 void Reset (void)
 {
@@ -843,7 +851,7 @@ Disconnect (std::string path, const CallbackBase &cb)
   NS_LOG_FUNCTION (path << &cb);
   ConfigImpl::Get ()->Disconnect (path, cb);
 }
-Config::MatchContainer LookupMatches (std::string path)
+MatchContainer LookupMatches (std::string path)
 {
   NS_LOG_FUNCTION (path);
   return ConfigImpl::Get ()->LookupMatches (path);
