@@ -34,7 +34,7 @@
 /**
  * \file
  * \ingroup randomvariable
- * Declaration of ns3::RandomVariableStream and derivatives.
+ * ns3::RandomVariableStream declaration, and related classes.
  */
 
 namespace ns3 {
@@ -81,14 +81,14 @@ class RngStream;
  *
  * ns-3 has a rich set of random number generators that allow stream
  * numbers to be set deterministically if desired.  Class
- * RandomVariableStream defines the base class functionalty required
+ * RandomVariableStream defines the base class functionality required
  * for all such random number generators.
  *
  * By default, the underlying generator is seeded all the time with
  * the same seed value and run number coming from the ns3::GlobalValue
  * \ref GlobalValueRngSeed "RngSeed" and \ref GlobalValueRngRun
- * "RngRun".  Also by default, the stream number value for this RNG
- * stream is automatically allocated.
+ * "RngRun".  Also by default, the stream number value for the
+ * underlying RngStream is automatically allocated.
  *
  * Instances can be configured to return "antithetic" values.
  * See the documentation for the specific distributions to see
@@ -112,15 +112,15 @@ public:
   virtual ~RandomVariableStream();
 
   /**
-   * \brief Specifies the stream number for this RNG stream.
-   * \param [in] stream The stream number for this RNG stream.
+   * \brief Specifies the stream number for the RngStream.
+   * \param [in] stream The stream number for the RngStream.
    * -1 means "allocate a stream number automatically".
    */
   void SetStream (int64_t stream);
 
   /**
-   * \brief Returns the stream number for this RNG stream.
-   * \return The stream number for this RNG stream.
+   * \brief Returns the stream number for the RngStream.
+   * \return The stream number for the RngStream.
    * -1 means this stream was allocated automatically.
    */
   int64_t GetStream(void) const;
@@ -151,7 +151,8 @@ public:
 
 protected:
   /**
-   * \brief Get the pointer to the underlying RNG stream.
+   * \brief Get the pointer to the underlying RngStream.
+   * \return The underlying RngStream
    */
   RngStream *Peek(void) const;
 
@@ -166,7 +167,7 @@ private:
    */
   RandomVariableStream (const RandomVariableStream &o);
   /**
-   * Assignment operator.  These objects can't be copied by assignement.
+   * Assignment operator.  These objects can't be copied by assignment.
    *
    * \param [in] o The RandomVariableStream to copy.
    * \return lvalue RandomVariableStream.
@@ -177,13 +178,13 @@ private:
    */
   RandomVariableStream &operator = (const RandomVariableStream &o);
 
-  /** Pointer to the underlying RNG stream. */
+  /** Pointer to the underlying RngStream. */
   RngStream *m_rng;
 
   /** Indicates if antithetic values should be generated by this RNG stream. */
   bool m_isAntithetic;
 
-  /** The stream number for this RNG stream. */
+  /** The stream number for the RngStream. */
   int64_t m_stream;
 
 };  // class RandomVariableStream
@@ -544,7 +545,7 @@ public:
   static TypeId GetTypeId (void);
 
   /**
-   * \brief Creates a exponential distribution RNG with the default
+   * \brief Creates an exponential distribution RNG with the default
    * values for the mean and upper bound.
    */
   ExponentialRandomVariable ();
@@ -576,7 +577,7 @@ public:
   /**
    * \brief Get the next random value, as an unsigned integer from
    * the exponential distribution with the specified mean and upper bound.
-   * \param [in] mean Mean value of the unbounded exponential distributuion.
+   * \param [in] mean Mean value of the unbounded exponential distribution.
    * \param [in] bound Upper bound on values returned.
    * \return A random unsigned integer value.
    */
@@ -607,9 +608,9 @@ private:
  * The probability density function of a Pareto variable is defined
  * over the range [\f$x_m\f$,\f$+\infty\f$) as: \f$ k \frac{x_m^k}{x^{k+1}}\f$
  * where \f$x_m > 0\f$ is called the scale parameter and \f$ k > 0\f$
- * is called the pareto index or shape.
+ * is called the Pareto index or shape.
  *
- * The parameter \f$ x_m \f$ can be infered from the mean and the parameter \f$ k \f$
+ * The parameter \f$ x_m \f$ can be inferred from the mean and the parameter \f$ k \f$
  * with the equation \f$ x_m = mean \frac{k-1}{k},  k > 1\f$.
  *
  * Since Pareto distributions can theoretically return unbounded values,
@@ -619,11 +620,11 @@ private:
  *
  * Here is an example of how to use this class:
  * \code
- *   double mean = 5.0;
+ *   double scale = 5.0;
  *   double shape = 2.0;
  * 
  *   Ptr<ParetoRandomVariable> x = CreateObject<ParetoRandomVariable> ();
- *   x->SetAttribute ("Mean", DoubleValue (mean));
+ *   x->SetAttribute ("Scale", DoubleValue (scale));
  *   x->SetAttribute ("Shape", DoubleValue (shape));
  * 
  *   // The expected value for the mean of the values returned by a
@@ -632,11 +633,7 @@ private:
  *   //                   shape * scale
  *   //     E[value]  =  ---------------  ,
  *   //                     shape - 1
- *   // 
- *   // where
- *   // 
- *   //     scale  =  mean * (shape - 1.0) / shape .
- *   //
+ *
  *   double value = x->GetValue ();
  * \endcode
  */
@@ -659,8 +656,15 @@ public:
    * \brief Returns the mean parameter for the Pareto distribution returned by this RNG stream.
    * \return The mean parameter for the Pareto distribution returned by this RNG stream.
    */
+  NS_DEPRECATED
   double GetMean (void) const;
 
+  /**
+   * \brief Returns the scale parameter for the Pareto distribution returned by this RNG stream.
+   * \return The scale parameter for the Pareto distribution returned by this RNG stream.
+   */
+  double GetScale (void) const;
+
   /**
    * \brief Returns the shape parameter for the Pareto distribution returned by this RNG stream.
    * \return The shape parameter for the Pareto distribution returned by this RNG stream.
@@ -674,8 +678,8 @@ public:
   double GetBound (void) const;
 
   /**
-   * \brief Returns a random double from a Pareto distribution with the specified mean, shape, and upper bound.
-   * \param [in] mean Mean parameter for the Pareto distribution.
+   * \brief Returns a random double from a Pareto distribution with the specified scale, shape, and upper bound.
+   * \param [in] scale Mean parameter for the Pareto distribution.
    * \param [in] shape Shape parameter for the Pareto distribution.
    * \param [in] bound Upper bound on values returned.
    * \return A floating point random value.
@@ -688,27 +692,22 @@ public:
    *         x = \frac{scale}{u^{\frac{1}{shape}}}
    *    \f]
    *
-   * is a value that would be returned normally, where
-   *     
-   *    \f[
-   *         scale  =  mean * (shape - 1.0) / shape  .
-   *    \f]
+   * is a value that would be returned normally.
    *    
-   * Then \f$(1 - u\f$) is the distance that \f$u\f$ would be from
-   * \f$1\f$.  The value returned in the antithetic case, \f$x'\f$, is
+   * The value returned in the antithetic case, \f$x'\f$, is
    * calculated as
    *
    *    \f[
    *         x' = \frac{scale}{{(1 - u)}^{\frac{1}{shape}}} ,
    *    \f]
    *
-   * which now involves the distance \f$u\f$ is from 1 in the denonator.
+   * which now involves the distance \f$u\f$ is from 1 in the denominator.
    */
-  double GetValue (double mean, double shape, double bound);
+  double GetValue (double scale, double shape, double bound);
 
   /**
    * \brief Returns a random unsigned integer from a Pareto distribution with the specified mean, shape, and upper bound.
-   * \param [in] mean Mean parameter for the Pareto distribution.
+   * \param [in] scale Scale parameter for the Pareto distribution.
    * \param [in] shape Shape parameter for the Pareto distribution.
    * \param [in] bound Upper bound on values returned.
    * \return A random unsigned integer value.
@@ -721,23 +720,18 @@ public:
    *         x = \frac{scale}{u^{\frac{1}{shape}}}
    *    \f]
    *
-   * is a value that would be returned normally, where
-   *     
-   *    \f[
-   *         scale  =  mean * (shape - 1.0) / shape  .
-   *    \f]
-   *    
-   * Then \f$(1 - u\f$) is the distance that \f$u\f$ would be from
-   * \f$1\f$.  The value returned in the antithetic case, \f$x'\f$, is
+   * is a value that would be returned normally.
+   *
+   * The value returned in the antithetic case, \f$x'\f$, is
    * calculated as
    *
    *    \f[
    *         x' = \frac{scale}{{(1 - u)}^{\frac{1}{shape}}} ,
    *    \f]
    *
-   * which now involves the distance \f$u\f$ is from 1 in the denonator.
+   * which now involves the distance \f$u\f$ is from 1 in the denominator.
    */
-  uint32_t GetInteger (uint32_t mean, uint32_t shape, uint32_t bound);
+  uint32_t GetInteger (uint32_t scale, uint32_t shape, uint32_t bound);
 
   /**
    * \brief Returns a random double from a Pareto distribution with the current mean, shape, and upper bound.
@@ -757,15 +751,14 @@ public:
    *         scale  =  mean * (shape - 1.0) / shape  .
    *    \f]
    *    
-   * Then \f$(1 - u\f$) is the distance that \f$u\f$ would be from
-   * \f$1\f$.  The value returned in the antithetic case, \f$x'\f$, is
+   * The value returned in the antithetic case, \f$x'\f$, is
    * calculated as
    *
    *    \f[
    *         x' = \frac{scale}{{(1 - u)}^{\frac{1}{shape}}} ,
    *    \f]
    *
-   * which now involves the distance \f$u\f$ is from 1 in the denonator.
+   * which now involves the distance \f$u\f$ is from 1 in the denominator.
    *
    * Note that we have to re-implement this method here because the method is
    * overloaded above for the three-argument variant and the c++ name resolution
@@ -786,21 +779,16 @@ public:
    *         x = \frac{scale}{u^{\frac{1}{shape}}}
    *    \f]
    *
-   * is a value that would be returned normally, where
-   *     
-   *    \f[
-   *         scale  =  mean * (shape - 1.0) / shape  .
-   *    \f]
-   *    
-   * Then \f$(1 - u\f$) is the distance that \f$u\f$ would be from
-   * \f$1\f$.  The value returned in the antithetic case, \f$x'\f$, is
+   * is a value that would be returned normally.
+   *
+   * The value returned in the antithetic case, \f$x'\f$, is
    * calculated as
    *
    *    \f[
    *         x' = \frac{scale}{{(1 - u)}^{\frac{1}{shape}}} ,
    *    \f]
    *
-   * which now involves the distance \f$u\f$ is from 1 in the denonator.
+   * which now involves the distance \f$u\f$ is from 1 in the denominator.
    */
   virtual uint32_t GetInteger (void);
 
@@ -808,6 +796,9 @@ private:
   /** The mean parameter for the Pareto distribution returned by this RNG stream. */
   double m_mean;
 
+  /** The scale parameter for the Pareto distribution returned by this RNG stream. */
+  double m_scale;
+
   /** The shape parameter for the Pareto distribution returned by this RNG stream. */
   double m_shape;
 
@@ -2548,9 +2539,9 @@ public:
                               double v1, double v2, double r);
   
   /** \c true once the CDF has been validated. */
-  bool validated;
+  bool m_validated;
   /** The vector of CDF points. */
-  std::vector<ValueCDF> emp; 
+  std::vector<ValueCDF> m_emp;
 
 };  // class EmpiricalRandomVariable
   
