@@ -12,8 +12,9 @@ cpp_examples = [
     ("main-callback", "True", "True"),
     ("sample-simulator", "True", "True"),
     ("main-ptr", "True", "True"),
-    ("main-random-variable", "False", "False"),
+    ("main-random-variable", "True", "False"),
     ("sample-random-variable", "True", "True"),
+    ("test-string-value-formatting", "True", "True"),
 ]
 
 # A list of Python examples to run in order to ensure that they remain
