@@ -17,12 +17,29 @@
 #include "ns3/test.h"
 #include "ns3/names.h"
 
-using namespace ns3;
 
-// ===========================================================================
-// Cook up a couple of simple object class that we can use in the object 
-// naming tests.  They do nothing but be of the right type.
-// ===========================================================================
+/**
+ * \file
+ * \ingroup core-tests
+ * \ingroup config
+ * \ingroup names-tests
+ * Object Names test suite.
+ */
+
+/**
+ * \ingroup core-tests
+ * \defgroup names-tests Object Names test suite
+ */
+
+namespace ns3 {
+
+  namespace tests {
+    
+
+/**
+ * \ingroup names-tests
+ * Simple test object to excercise the Name service.
+ */
 class TestObject : public Object
 {
 public:
@@ -39,10 +56,14 @@ public:
       .AddConstructor<TestObject> ();
     return tid;
   }
+  /** Constructor. */
   TestObject () {}
-  virtual void Dispose (void) {}
 };
 
+/**
+ * \ingroup names-tests
+ * Alternate test object for the Name service.
+ */
 class AlternateTestObject : public Object
 {
 public:
@@ -59,24 +80,28 @@ public:
       .AddConstructor<AlternateTestObject> ();
     return tid;
   }
+  /** Constructor. */
   AlternateTestObject () {}
-  virtual void Dispose (void) {}
 };
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can do its most basic
-// job and add associations between Objects using the lowest level add 
-// function, which is:
-//
-//   Add (Ptr<Object> context, std::string name, Ptr<Object> object);
-//
-// All other add functions will just translate into this form, so this is the
-// most basic Add functionality.
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can do its most basic job.
+ *
+ * Add associations between Objects using the lowest level add 
+ * function, which is:
+ *
+ *     Add (Ptr<Object> context, std::string name, Ptr<Object> object);
+ *
+ * All other add functions will just translate into this form, so this is the
+ * most basic Add functionality.
+ */
 class BasicAddTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   BasicAddTestCase ();
+  /** Destructor. */
   virtual ~BasicAddTestCase ();
 
 private:
@@ -129,19 +154,21 @@ BasicAddTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "Child", "Could not Names::Add and Names::FindName a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can correctly use a
-// string context in the most basic ways
-//
-//   Add (std::string context, std::string name, Ptr<Object> object);
-//
-// High level path-based functions will translate into this form, so this is 
-// the second most basic Add functionality.
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can correctly use a string context.
+ *
+ *     Add (std::string context, std::string name, Ptr<Object> object);
+ *
+ * High level path-based functions will translate into this form, so this is 
+ * the second most basic Add functionality.
+ */
 class StringContextAddTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   StringContextAddTestCase ();
+  /** Destructor. */
   virtual ~StringContextAddTestCase ();
 
 private:
@@ -195,16 +222,20 @@ StringContextAddTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "Child", "Could not Names::Add and Names::FindName a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can correctly use a
-// fully qualified path to add assocations
-//
-//   Add (std::string name, Ptr<Object> object);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can correctly use a 
+ * fully qualified path to add assocations.
+ *
+ *     Add (std::string name, Ptr<Object> object);
+ * 
+ */
 class FullyQualifiedAddTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   FullyQualifiedAddTestCase ();
+  /** Destructor. */
   virtual ~FullyQualifiedAddTestCase ();
 
 private:
@@ -258,19 +289,24 @@ FullyQualifiedAddTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "Child", "Could not Names::Add and Names::FindName a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can correctly use a
-// relative path to add assocations.  This functionality is provided as a 
-// convenience so clients don't always have to provide the name service 
-// namespace name in all of their strings.
-//
-//
-//   Add (std::string name, Ptr<Object> object);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can correctly use a
+ * relative path to add assocations.
+ *
+ * This functionality is provided as a convenience so clients
+ * don't always have to provide the name service namespace name
+ * in all of their strings.
+ *
+ *     Add (std::string name, Ptr<Object> object);
+ * 
+ */
 class RelativeAddTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   RelativeAddTestCase ();
+  /** Destructor. */
   virtual ~RelativeAddTestCase ();
 
 private:
@@ -324,19 +360,21 @@ RelativeAddTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "Child", "Could not Names::Add and Names::FindName a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can rename objects in
-// its most basic way, which is 
-//
-//   Rename (Ptr<Object> context, std::string oldname, std::string newname);
-//
-// All other rename functions will just translate into this form, so this is the
-// most basic rename functionality.
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can rename objects.
+ *
+ *     Rename (Ptr<Object> context, std::string oldname, std::string newname);
+ *
+ * All other rename functions will just translate into this form, so this is the
+ * most basic rename functionality.
+ */
 class BasicRenameTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   BasicRenameTestCase ();
+  /** Destructor. */
   virtual ~BasicRenameTestCase ();
 
 private:
@@ -387,16 +425,20 @@ BasicRenameTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "New Child", "Could not Names::Rename a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can rename objects 
-// using a string context
-//
-//   Rename (std::string context, std::string oldname, std::string newname);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can rename objects 
+ * using a string context.
+ *
+ *     Rename (std::string context, std::string oldname, std::string newname);
+ * 
+ */
 class StringContextRenameTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   StringContextRenameTestCase ();
+  /** Destructor. */
   virtual ~StringContextRenameTestCase ();
 
 private:
@@ -447,16 +489,20 @@ StringContextRenameTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "New Child", "Could not Names::Rename a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can rename objects 
-// using a fully qualified path name
-//
-//   Rename (std::string oldpath, std::string newname);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can rename objects 
+ * using a fully qualified path name.
+ *
+ *     Rename (std::string oldpath, std::string newname);
+ * 
+ */
 class FullyQualifiedRenameTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   FullyQualifiedRenameTestCase ();
+  /** Destructor. */
   virtual ~FullyQualifiedRenameTestCase ();
 
 private:
@@ -507,16 +553,20 @@ FullyQualifiedRenameTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "New Child", "Could not Names::Rename a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can rename objects 
-// using a relaltive path name
-//
-//   Rename (std::string oldpath, std::string newname);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can rename objects 
+ * using a relative path name.
+ *
+ *     Rename (std::string oldpath, std::string newname);
+ * 
+ */
 class RelativeRenameTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   RelativeRenameTestCase ();
+  /** Destructor. */
   virtual ~RelativeRenameTestCase ();
 
 private:
@@ -567,16 +617,20 @@ RelativeRenameTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "New Child", "Could not Names::Rename a child Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can look up an object
-// and return its fully qualified path name
-//
-//   FindPath (Ptr<Object> object);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can look up an object
+ * and return its fully qualified path name.
+ *
+ *     FindPath (Ptr<Object> object);
+ * 
+ */
 class FindPathTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   FindPathTestCase ();
+  /** Destructor. */
   virtual ~FindPathTestCase ();
 
 private:
@@ -621,16 +675,19 @@ FindPathTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, "", "Unexpectedly found a non-existent Object");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can find Objects using 
-// the lowest level find function, which is:
-//
-//   Find (Ptr<Object> context, std::string name);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can find Objects.
+ *
+ *     Find (Ptr<Object> context, std::string name);
+ * 
+ */
 class BasicFindTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   BasicFindTestCase ();
+  /** Destructor. */
   virtual ~BasicFindTestCase ();
 
 private:
@@ -683,16 +740,20 @@ BasicFindTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, childOfObjectTwo, "Could not find a previously named child Object via object context");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can find Objects using 
-// a string context-based find function, which is:
-//
-//   Find (std::string context, std::string name);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can find Objects using 
+ * a string context.
+ *
+ *     Find (std::string context, std::string name);
+ * 
+ */
 class StringContextFindTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   StringContextFindTestCase ();
+  /** Destructor. */
   virtual ~StringContextFindTestCase ();
 
 private:
@@ -745,16 +806,20 @@ StringContextFindTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, childOfObjectTwo, "Could not find a previously named child Object via string context");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can find Objects using 
-// a fully qualified path name-based find function, which is:
-//
-//   Find (std::string name);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can find Objects using 
+ * a fully qualified path name.
+ *
+ *     Find (std::string name);
+ * 
+ */
 class FullyQualifiedFindTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   FullyQualifiedFindTestCase ();
+  /** Destructor. */
   virtual ~FullyQualifiedFindTestCase ();
 
 private:
@@ -807,16 +872,20 @@ FullyQualifiedFindTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, childOfObjectTwo, "Could not find a previously named child Object via string context");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can find Objects using 
-// a relative path name-based find function, which is:
-//
-//   Find (std::string name);
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can find Objects using 
+ * a relative path name.
+ *
+ *     Find (std::string name);
+ * 
+ */
 class RelativeFindTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   RelativeFindTestCase ();
+  /** Destructor. */
   virtual ~RelativeFindTestCase ();
 
 private:
@@ -869,14 +938,17 @@ RelativeFindTestCase::DoRun (void)
   NS_TEST_ASSERT_MSG_EQ (found, childOfObjectTwo, "Could not find a previously named child Object via string context");
 }
 
-// ===========================================================================
-// Test case to make sure that the Object Name Service can find Objects using 
-// a second type.
-// ===========================================================================
+/**
+ * \ingroup names-tests
+ * Test the Object Name Service can find Objects using 
+ * a second type.
+ */
 class AlternateFindTestCase : public TestCase
 {
 public:
+  /** Constructor. */
   AlternateFindTestCase ();
+  /** Destructor. */
   virtual ~AlternateFindTestCase ();
 
 private:
@@ -929,29 +1001,43 @@ AlternateFindTestCase::DoRun (void)
                          "Unexpectedly able to GetObject<TestObject> on an AlternateTestObject");
 }
 
+/**
+ * \ingroup names-tests
+ * Names Test Suite 
+ */
 class NamesTestSuite : public TestSuite
 {
 public:
+  /** Constructor. */
   NamesTestSuite ();
 };
 
 NamesTestSuite::NamesTestSuite ()
-  : TestSuite ("object-name-service", UNIT)
+  : TestSuite ("object-name-service")
 {
-  AddTestCase (new BasicAddTestCase, TestCase::QUICK);
-  AddTestCase (new StringContextAddTestCase, TestCase::QUICK);
-  AddTestCase (new FullyQualifiedAddTestCase, TestCase::QUICK);
-  AddTestCase (new RelativeAddTestCase, TestCase::QUICK);
-  AddTestCase (new BasicRenameTestCase, TestCase::QUICK);
-  AddTestCase (new StringContextRenameTestCase, TestCase::QUICK);
-  AddTestCase (new FullyQualifiedRenameTestCase, TestCase::QUICK);
-  AddTestCase (new RelativeRenameTestCase, TestCase::QUICK);
-  AddTestCase (new FindPathTestCase, TestCase::QUICK);
-  AddTestCase (new BasicFindTestCase, TestCase::QUICK);
-  AddTestCase (new StringContextFindTestCase, TestCase::QUICK);
-  AddTestCase (new FullyQualifiedFindTestCase, TestCase::QUICK);
-  AddTestCase (new RelativeFindTestCase, TestCase::QUICK);
-  AddTestCase (new AlternateFindTestCase, TestCase::QUICK);
+  AddTestCase (new BasicAddTestCase);
+  AddTestCase (new StringContextAddTestCase);
+  AddTestCase (new FullyQualifiedAddTestCase);
+  AddTestCase (new RelativeAddTestCase);
+  AddTestCase (new BasicRenameTestCase);
+  AddTestCase (new StringContextRenameTestCase);
+  AddTestCase (new FullyQualifiedRenameTestCase);
+  AddTestCase (new RelativeRenameTestCase);
+  AddTestCase (new FindPathTestCase);
+  AddTestCase (new BasicFindTestCase);
+  AddTestCase (new StringContextFindTestCase);
+  AddTestCase (new FullyQualifiedFindTestCase);
+  AddTestCase (new RelativeFindTestCase);
+  AddTestCase (new AlternateFindTestCase);
 }
 
-static NamesTestSuite namesTestSuite;
+/**
+ * \ingroup names-tests
+ *  NamesTestSuite instance variable.
+ */
+static NamesTestSuite g_namesTestSuite;
+
+
+  }  // namespace tests
+
+}  // namespace ns3
