@@ -32,6 +32,7 @@
 #include "ns3/trace-source-accessor.h"
 #include "csma-net-device.h"
 #include "csma-channel.h"
+#include "ns3/net-device-queue-interface.h"
 
 namespace ns3 {
 
@@ -86,7 +87,7 @@ CsmaNetDevice::GetTypeId (void)
                    "A queue to use as the transmit queue in the device.",
                    PointerValue (),
                    MakePointerAccessor (&CsmaNetDevice::m_queue),
-                   MakePointerChecker<Queue> ())
+                   MakePointerChecker<Queue<Packet> > ())
 
     //
     // Trace sources at the "top" of the net device, where packets transition
@@ -189,7 +190,7 @@ CsmaNetDevice::CsmaNetDevice ()
   NS_LOG_FUNCTION (this);
   m_txMachineState = READY;
   m_tInterframeGap = Seconds (0);
-  m_channel = 0; 
+  m_channel = 0;
 
   // 
   // We would like to let the attribute system take care of initializing the 
@@ -217,9 +218,45 @@ CsmaNetDevice::DoDispose ()
   NS_LOG_FUNCTION_NOARGS ();
   m_channel = 0;
   m_node = 0;
+  m_queue = 0;
+  m_queueInterface = 0;
   NetDevice::DoDispose ();
 }
 
+void
+CsmaNetDevice::DoInitialize (void)
+{
+  if (m_queueInterface)
+    {
+      NS_ASSERT_MSG (m_queue != 0, "A Queue object has not been attached to the device");
+
+      // connect the traced callbacks of m_queue to the static methods provided by
+      // the NetDeviceQueue class to support flow control and dynamic queue limits.
+      // This could not be done in NotifyNewAggregate because at that time we are
+      // not guaranteed that a queue has been attached to the netdevice
+      m_queueInterface->ConnectQueueTraces (m_queue, 0);
+    }
+
+  NetDevice::DoInitialize ();
+}
+
+void
+CsmaNetDevice::NotifyNewAggregate (void)
+{
+  NS_LOG_FUNCTION (this);
+  if (m_queueInterface == 0)
+    {
+      Ptr<NetDeviceQueueInterface> ndqi = this->GetObject<NetDeviceQueueInterface> ();
+      //verify that it's a valid netdevice queue interface and that
+      //the netdevice queue interface was not set before
+      if (ndqi != 0)
+        {
+          m_queueInterface = ndqi;
+        }
+    }
+  NetDevice::NotifyNewAggregate ();
+}
+
 void
 CsmaNetDevice::SetEncapsulationMode (enum EncapsulationMode mode)
 {
@@ -564,9 +601,9 @@ CsmaNetDevice::TransmitAbort (void)
     }
   else
     {
-      Ptr<QueueItem> item = m_queue->Dequeue ();
-      NS_ASSERT_MSG (item != 0, "CsmaNetDevice::TransmitAbort(): IsEmpty false but no Packet on queue?");
-      m_currentPkt = item->GetPacket ();
+      Ptr<Packet> packet = m_queue->Dequeue ();
+      NS_ASSERT_MSG (packet != 0, "CsmaNetDevice::TransmitAbort(): IsEmpty false but no Packet on queue?");
+      m_currentPkt = packet;
       m_snifferTrace (m_currentPkt);
       m_promiscSnifferTrace (m_currentPkt);
       TransmitStart ();
@@ -633,9 +670,9 @@ CsmaNetDevice::TransmitReadyEvent (void)
     }
   else
     {
-      Ptr<QueueItem> item = m_queue->Dequeue ();
-      NS_ASSERT_MSG (item != 0, "CsmaNetDevice::TransmitReadyEvent(): IsEmpty false but no Packet on queue?");
-      m_currentPkt = item->GetPacket ();
+      Ptr<Packet> packet = m_queue->Dequeue ();
+      NS_ASSERT_MSG (packet != 0, "CsmaNetDevice::TransmitReadyEvent(): IsEmpty false but no Packet on queue?");
+      m_currentPkt = packet;
       m_snifferTrace (m_currentPkt);
       m_promiscSnifferTrace (m_currentPkt);
       TransmitStart ();
@@ -669,7 +706,7 @@ CsmaNetDevice::Attach (Ptr<CsmaChannel> ch)
 }
 
 void
-CsmaNetDevice::SetQueue (Ptr<Queue> q)
+CsmaNetDevice::SetQueue (Ptr<Queue<Packet> > q)
 {
   NS_LOG_FUNCTION (q);
   m_queue = q;
@@ -819,7 +856,7 @@ CsmaNetDevice::Receive (Ptr<Packet> packet, Ptr<CsmaNetDevice> senderDevice)
     }
 }
 
-Ptr<Queue>
+Ptr<Queue<Packet> >
 CsmaNetDevice::GetQueue (void) const 
 { 
   NS_LOG_FUNCTION_NOARGS ();
@@ -970,7 +1007,7 @@ CsmaNetDevice::SendFrom (Ptr<Packet> packet, const Address& src, const Address&
   // Place the packet to be sent on the send queue.  Note that the 
   // queue may fire a drop trace, but we will too.
   //
-  if (m_queue->Enqueue (Create<QueueItem> (packet)) == false)
+  if (m_queue->Enqueue (packet) == false)
     {
       m_macTxDropTrace (packet);
       return false;
@@ -985,9 +1022,9 @@ CsmaNetDevice::SendFrom (Ptr<Packet> packet, const Address& src, const Address&
     {
       if (m_queue->IsEmpty () == false)
         {
-          Ptr<QueueItem> item = m_queue->Dequeue ();
-          NS_ASSERT_MSG (item != 0, "CsmaNetDevice::SendFrom(): IsEmpty false but no Packet on queue?");
-          m_currentPkt = item->GetPacket ();
+          Ptr<Packet> packet = m_queue->Dequeue ();
+          NS_ASSERT_MSG (packet != 0, "CsmaNetDevice::SendFrom(): IsEmpty false but no Packet on queue?");
+          m_currentPkt = packet;
           m_promiscSnifferTrace (m_currentPkt);
           m_snifferTrace (m_currentPkt);
           TransmitStart ();
