@@ -800,8 +800,7 @@ TypeId DsrOptionRerrUnreachHeader::GetInstanceTypeId () const
 }
 
 DsrOptionRerrUnreachHeader::DsrOptionRerrUnreachHeader ()
-  :
-    m_salvage (0)
+  : m_salvage (0)
 {
   SetType (3);
   SetLength (18);
@@ -929,8 +928,7 @@ TypeId DsrOptionRerrUnsupportHeader::GetInstanceTypeId () const
 }
 
 DsrOptionRerrUnsupportHeader::DsrOptionRerrUnsupportHeader ()
-  :
-    m_salvage (0)
+  : m_salvage (0)
 {
   SetType (3);
   SetLength (14);
