@@ -178,7 +178,7 @@ V4Ping::Receive (Ptr<Socket> socket)
 void
 V4Ping::Write32 (uint8_t *buffer, const uint32_t data)
 {
-  NS_LOG_FUNCTION (this << buffer << data);
+  NS_LOG_FUNCTION (this << (void *) buffer << data);
   buffer[0] = (data >> 0) & 0xff;
   buffer[1] = (data >> 8) & 0xff;
   buffer[2] = (data >> 16) & 0xff;
@@ -189,7 +189,7 @@ V4Ping::Write32 (uint8_t *buffer, const uint32_t data)
 void
 V4Ping::Read32 (const uint8_t *buffer, uint32_t &data)
 {
-  NS_LOG_FUNCTION (this << buffer << data);
+  NS_LOG_FUNCTION (this << (void *) buffer << data);
   data = (buffer[3] << 24) + (buffer[2] << 16) + (buffer[1] << 8) + buffer[0];
 }
 
