@@ -30,6 +30,7 @@
 #include "ns3/ipv6-route.h"
 #include "ns3/pointer.h"
 #include "ns3/string.h"
+#include "ns3/integer.h"
 
 #include "ipv6-raw-socket-factory-impl.h"
 #include "ipv6-l3-protocol.h"
@@ -44,25 +45,21 @@ NS_OBJECT_ENSURE_REGISTERED (Icmpv6L4Protocol);
 
 const uint8_t Icmpv6L4Protocol::PROT_NUMBER = 58;
 
-const uint8_t Icmpv6L4Protocol::MAX_INITIAL_RTR_ADVERT_INTERVAL = 16;
-const uint8_t Icmpv6L4Protocol::MAX_INITIAL_RTR_ADVERTISEMENTS = 3;
-const uint8_t Icmpv6L4Protocol::MAX_FINAL_RTR_ADVERTISEMENTS = 3;
-const uint8_t Icmpv6L4Protocol::MIN_DELAY_BETWEEN_RAS = 3;
-const uint32_t Icmpv6L4Protocol::MAX_RA_DELAY_TIME = 500; /* millisecond */
+//const uint8_t Icmpv6L4Protocol::MAX_INITIAL_RTR_ADVERT_INTERVAL = 16; // max initial RA initial interval.
+//const uint8_t Icmpv6L4Protocol::MAX_INITIAL_RTR_ADVERTISEMENTS = 3;   // max initial RA transmission.
+//const uint8_t Icmpv6L4Protocol::MAX_FINAL_RTR_ADVERTISEMENTS = 3;     // max final RA transmission.
+//const uint8_t Icmpv6L4Protocol::MIN_DELAY_BETWEEN_RAS = 3;            // min delay between RA.
+//const uint32_t Icmpv6L4Protocol::MAX_RA_DELAY_TIME = 500;             // millisecond - max delay between RA.
 
-const uint8_t Icmpv6L4Protocol::MAX_RTR_SOLICITATION_DELAY = 1;
-const uint8_t Icmpv6L4Protocol::RTR_SOLICITATION_INTERVAL = 4;
-const uint8_t Icmpv6L4Protocol::MAX_RTR_SOLICITATIONS = 3;
+//const uint8_t Icmpv6L4Protocol::MAX_RTR_SOLICITATION_DELAY = 1;       // max RS delay.
+//const uint8_t Icmpv6L4Protocol::RTR_SOLICITATION_INTERVAL = 4;        // RS interval.
+//const uint8_t Icmpv6L4Protocol::MAX_RTR_SOLICITATIONS = 3;            // max RS transmission.
 
-const uint8_t Icmpv6L4Protocol::MAX_MULTICAST_SOLICIT = 3;
-const uint8_t Icmpv6L4Protocol::MAX_UNICAST_SOLICIT = 3;
-const uint8_t Icmpv6L4Protocol::MAX_ANYCAST_DELAY_TIME = 1;
-const uint8_t Icmpv6L4Protocol::MAX_NEIGHBOR_ADVERTISEMENT = 3;
-const uint32_t Icmpv6L4Protocol::REACHABLE_TIME = 30000;
-const uint32_t Icmpv6L4Protocol::RETRANS_TIMER = 1000;
-const uint8_t Icmpv6L4Protocol::DELAY_FIRST_PROBE_TIME = 5;
-const double Icmpv6L4Protocol::MIN_RANDOM_FACTOR = 0.5;
-const double Icmpv6L4Protocol::MAX_RANDOM_FACTOR = 1.5;
+//const uint8_t Icmpv6L4Protocol::MAX_ANYCAST_DELAY_TIME = 1;           // max anycast delay.
+//const uint8_t Icmpv6L4Protocol::MAX_NEIGHBOR_ADVERTISEMENT = 3;       // max NA transmission.
+
+//const double Icmpv6L4Protocol::MIN_RANDOM_FACTOR = 0.5;               // min random factor.
+//const double Icmpv6L4Protocol::MAX_RANDOM_FACTOR = 1.5;               // max random factor.
 
 TypeId Icmpv6L4Protocol::GetTypeId ()
 {
@@ -74,12 +71,31 @@ TypeId Icmpv6L4Protocol::GetTypeId ()
                    BooleanValue (true),
                    MakeBooleanAccessor (&Icmpv6L4Protocol::m_alwaysDad),
                    MakeBooleanChecker ())
-    .AddAttribute ("SolicitationJitter", "The jitter in ms a node is allowed to wait before sending any solicitation . Some jitter aims to prevent collisions. By default, the model will wait for a duration in ms defined by a uniform random-variable between 0 and SolicitationJitter",
+    .AddAttribute ("SolicitationJitter", "The jitter in ms a node is allowed to wait before sending any solicitation. Some jitter aims to prevent collisions. By default, the model will wait for a duration in ms defined by a uniform random-variable between 0 and SolicitationJitter",
                    StringValue ("ns3::UniformRandomVariable[Min=0.0|Max=10.0]"),
                    MakePointerAccessor (&Icmpv6L4Protocol::m_solicitationJitter),
                    MakePointerChecker<RandomVariableStream> ())
-
-  ;
+    .AddAttribute ("MaxMulticastSolicit", "Neighbor Discovery node constants: max multicast solicitations.",
+                   IntegerValue (3),
+                   MakeIntegerAccessor (&Icmpv6L4Protocol::m_maxMulticastSolicit),
+                   MakeIntegerChecker<uint8_t> ())
+    .AddAttribute ("MaxUnicastSolicit", "Neighbor Discovery node constants: max unicast solicitations.",
+                   IntegerValue (3),
+                   MakeIntegerAccessor (&Icmpv6L4Protocol::m_maxUnicastSolicit),
+                   MakeIntegerChecker<uint8_t> ())
+    .AddAttribute ("ReachableTime", "Neighbor Discovery node constants: reachable time.",
+                   TimeValue (Seconds (30)),
+                   MakeTimeAccessor (&Icmpv6L4Protocol::m_reachableTime),
+                   MakeTimeChecker ())
+    .AddAttribute ("RetransmissionTime", "Neighbor Discovery node constants: retransmission timer.",
+                   TimeValue (Seconds (1)),
+                   MakeTimeAccessor (&Icmpv6L4Protocol::m_retransmissionTime),
+                   MakeTimeChecker ())
+    .AddAttribute ("DelayFirstProbe", "Neighbor Discovery node constants: delay for the first probe.",
+                   TimeValue (Seconds (5)),
+                   MakeTimeAccessor (&Icmpv6L4Protocol::m_delayFirstProbe),
+                   MakeTimeChecker ())
+    ;
   return tid;
 }
 
@@ -1272,7 +1288,7 @@ Ptr<NdiscCache> Icmpv6L4Protocol::CreateCache (Ptr<NetDevice> device, Ptr<Ipv6In
 
   Ptr<NdiscCache> cache = CreateObject<NdiscCache> ();
 
-  cache->SetDevice (device, interface);
+  cache->SetDevice (device, interface, this);
   device->AddLinkChangeCallback (MakeCallback (&NdiscCache::Flush, cache));
   m_cacheList.push_back (cache);
   return cache;
@@ -1454,5 +1470,36 @@ Icmpv6L4Protocol::GetDownTarget6 (void) const
   return m_downTarget;
 }
 
+uint8_t
+Icmpv6L4Protocol::GetMaxMulticastSolicit () const
+{
+  return m_maxMulticastSolicit;
+}
+
+uint8_t
+Icmpv6L4Protocol::GetMaxUnicastSolicit () const
+{
+  return m_maxUnicastSolicit;
+}
+
+Time
+Icmpv6L4Protocol::GetReachableTime () const
+{
+  return m_reachableTime;
+}
+
+Time
+Icmpv6L4Protocol::GetRetransmissionTime () const
+{
+  return m_retransmissionTime;
+}
+
+Time
+Icmpv6L4Protocol::GetDelayFirstProbe () const
+{
+  return m_delayFirstProbe;
+}
+
+
 } /* namespace ns3 */
 
