@@ -250,7 +250,7 @@ NscTcpSocketImpl::Bind (const Address &address)
     }
   else if (ipv4 == Ipv4Address::GetAny () && port != 0)
     {
-      m_endPoint = m_tcp->Allocate (port);
+      m_endPoint = m_tcp->Allocate (GetBoundNetDevice (), port);
       NS_LOG_LOGIC ("NscTcpSocketImpl "<<this<<" got an endpoint: "<<m_endPoint);
     }
   else if (ipv4 != Ipv4Address::GetAny () && port == 0)
@@ -260,7 +260,7 @@ NscTcpSocketImpl::Bind (const Address &address)
     }
   else if (ipv4 != Ipv4Address::GetAny () && port != 0)
     {
-      m_endPoint = m_tcp->Allocate (ipv4, port);
+      m_endPoint = m_tcp->Allocate (GetBoundNetDevice (), ipv4, port);
       NS_LOG_LOGIC ("NscTcpSocketImpl "<<this<<" got an endpoint: "<<m_endPoint);
     }
 
@@ -268,6 +268,19 @@ NscTcpSocketImpl::Bind (const Address &address)
   return FinishBind ();
 }
 
+/* Inherit from Socket class: Bind this socket to the specified NetDevice */
+void
+NscTcpSocketImpl::BindToNetDevice (Ptr<NetDevice> netdevice)
+{
+  NS_LOG_FUNCTION (this << netdevice);
+  Socket::BindToNetDevice (netdevice); // Includes sanity check
+  if (m_endPoint != 0)
+    {
+      m_endPoint->BindToNetDevice (netdevice);
+    }
+  return;
+}
+
 int 
 NscTcpSocketImpl::ShutdownSend (void)
 {
@@ -411,6 +424,7 @@ NscTcpSocketImpl::Listen (void)
 void
 NscTcpSocketImpl::NSCWakeup ()
 {
+  NS_LOG_FUNCTION (this);
   switch (m_state) {
     case SYN_SENT:
       if (!m_nscTcpSocket->is_connected ())
@@ -420,7 +434,16 @@ NscTcpSocketImpl::NSCWakeup ()
     // fall through to schedule read/write events
     case ESTABLISHED:
       if (!m_txBuffer.empty ())
-        Simulator::ScheduleNow (&NscTcpSocketImpl::SendPendingData, this);
+        {
+          Simulator::ScheduleNow (&NscTcpSocketImpl::SendPendingData, this);
+        }
+      else
+        {
+          if (GetTxAvailable ())
+            {
+              NotifySend (GetTxAvailable ());
+            }
+        }
       Simulator::ScheduleNow (&NscTcpSocketImpl::ReadPendingData, this);
       break;
     case LISTEN:
@@ -461,14 +484,7 @@ NscTcpSocketImpl::RecvFrom (uint32_t maxSize, uint32_t flags,
 {
   NS_LOG_FUNCTION (this << maxSize << flags);
   Ptr<Packet> packet = Recv (maxSize, flags);
-  if (packet != 0)
-    {
-      SocketAddressTag tag;
-      bool found;
-      found = packet->PeekPacketTag (tag);
-      NS_ASSERT (found);
-      fromAddress = tag.GetAddress ();
-    }
+  GetPeerName (fromAddress);
   return packet;
 }
 
@@ -630,10 +646,6 @@ bool NscTcpSocketImpl::ReadPendingData (void)
 
   Ptr<Packet> p =  Create<Packet> (buffer, len);
 
-  SocketAddressTag tag;
-
-  tag.SetAddress (m_peerAddress);
-  p->AddPacketTag (tag);
   m_deliveryQueue.push (p);
   m_rxAvailable += p->GetSize ();
 
@@ -697,9 +709,14 @@ bool NscTcpSocketImpl::SendPendingData (void)
 
   if (written > 0)
     {
+      NS_LOG_DEBUG ("Notifying data sent, remaining txbuffer size: " << m_txBufferSize);
       Simulator::ScheduleNow (&NscTcpSocketImpl::NotifyDataSent, this, ret);
       return true;
     }
+  else
+    {
+      NS_LOG_DEBUG ("Not notifying data sent, return value " << ret);
+    }
   return false;
 }
 
