@@ -164,6 +164,9 @@ public:
   TracedValue<SequenceNumber32> m_highTxMark; //!< Highest seqno ever sent, regardless of ReTx
   TracedValue<SequenceNumber32> m_nextTxSequence; //!< Next seqnum to be sent (SND.NXT), ReTx pushes it back
 
+  uint32_t               m_rcvTimestampValue;     //!< Receiver Timestamp value 
+  uint32_t               m_rcvTimestampEchoReply; //!< Sender Timestamp echoed by the receiver
+
   /**
    * \brief Get cwnd in segments rather than bytes
    *
@@ -192,11 +195,11 @@ public:
  * \brief A base class for implementation of a stream socket using TCP.
  *
  * This class contains the essential components of TCP, as well as a sockets
- * interface for upper layers to call. This serves as a base for other TCP
- * functions where the sliding window mechanism is handled here. This class
- * provides connection orientation and sliding window flow control. Part of
- * this class is modified from the original NS-3 TCP socket implementation
- * (TcpSocketImpl) by Raj Bhattacharjea <raj.b@gatech.edu> of Georgia Tech.
+ * interface for upper layers to call. This class provides connection orientation
+ * and sliding window flow control; congestion control is delegated to subclasses
+ * of TcpCongestionOps. Part of TcpSocketBase is modified from the original
+ * NS-3 TCP socket implementation (TcpSocketImpl) by
+ * Raj Bhattacharjea <raj.b@gatech.edu> of Georgia Tech.
  *
  * For IPv4 packets, the TOS set for the socket is used. The Bind and Connect
  * operations set the TOS for the socket to the value specified in the provided
@@ -225,10 +228,15 @@ public:
  * Congestion control interface
  * ---------------------------
  *
- * Congestion control, unlike older releases of ns-3, has been splitted from
+ * Congestion control, unlike older releases of ns-3, has been split from
  * TcpSocketBase. In particular, each congestion control is now a subclass of
  * the main TcpCongestionOps class. Switching between congestion algorithm is
- * now a matter of setting a pointer into the TcpSocketBase class.
+ * now a matter of setting a pointer into the TcpSocketBase class. The idea
+ * and the interfaces are inspired by the Linux operating system, and in
+ * particular from the structure tcp_congestion_ops.
+ *
+ * Transmission Control Block (TCB)
+ * --------------------------------
  *
  * The variables needed to congestion control classes to operate correctly have
  * been moved inside the TcpSocketState class. It contains information on the
@@ -240,30 +248,55 @@ public:
  * (see for example cWnd trace source).
  *
  * Fast retransmit
- * ---------------------------
+ * ----------------
  *
  * The fast retransmit enhancement is introduced in RFC 2581 and updated in
- * RFC 5681. It basically reduces the time a sender waits before retransmitting
+ * RFC 5681. It reduces the time a sender waits before retransmitting
  * a lost segment, through the assumption that if it receives a certain number
  * of duplicate ACKs, a segment has been lost and it can be retransmitted.
- * Usually it is coupled with the Limited Transmit algorithm, defined in
+ * Usually, it is coupled with the Limited Transmit algorithm, defined in
  * RFC 3042.
  *
  * In ns-3, these algorithms are included in this class, and it is implemented inside
- * the ReceivedAck method. The attribute which manages the number of dup ACKs
+ * the ProcessAck method. The attribute which manages the number of dup ACKs
  * necessary to start the fast retransmit algorithm is named "ReTxThreshold",
  * and its default value is 3, while the Limited Transmit one can be enabled
- * by setting the attribute "LimitedTransmit" to true.
+ * by setting the attribute "LimitedTransmit" to true. Before entering the
+ * recovery phase, the method EnterRecovery is called.
  *
  * Fast recovery
- * --------------------------
+ * -------------
  *
- * The fast recovery algorithm is introduced RFC 2001, and it basically
+ * The fast recovery algorithm is introduced RFC 2001, and it
  * avoids to reset cWnd to 1 segment after sensing a loss on the channel. Instead,
  * the slow start threshold is halved, and the cWnd is set equal to such value,
  * plus segments for the cWnd inflation.
  *
- * The algorithm is implemented in the ReceivedAck method.
+ * The algorithm is implemented in the ProcessAck method.
+ *
+ * RTO expiration
+ * --------------
+ *
+ * When the Retransmission Time Out expires, the TCP faces a big performance
+ * drop. The expiration event is managed in ReTxTimeout method, that basically
+ * set the cWnd to 1 segment and start "from scratch" again.
+ *
+ * Options management
+ * ------------------
+ *
+ * SYN and SYN-ACK options, which are allowed only at the beginning of the
+ * connection, are managed in the DoForwardUp and SendEmptyPacket methods.
+ * To read all others, we have set up a cycle inside ReadOptions. For adding
+ * them, there is no a unique place, since the options (and the information
+ * available to build them) are scattered around the code. For instance,
+ * the SACK option is built in SendEmptyPacket only under certain conditions.
+ *
+ * SACK
+ * ----
+ *
+ * The SACK generation/management is delegated to the buffer classes, namely
+ * TcpTxBuffer and TcpRxBuffer. Please take a look on their documentation if
+ * you need more informations.
  *
  */
 class TcpSocketBase : public TcpSocket
@@ -282,6 +315,10 @@ public:
    */
   virtual TypeId GetInstanceTypeId () const;
 
+  /**
+   * \brief TcpGeneralTest friend class (for tests).
+   * \relates TcpGeneralTest
+   */
   friend class TcpGeneralTest;
 
   /**
@@ -447,8 +484,8 @@ public:
    * TracedCallback signature for tcp packet transmission or reception events.
    *
    * \param [in] packet The packet.
-   * \param [in] ipv4
-   * \param [in] interface
+   * \param [in] header The TcpHeader
+   * \param [in] socket This socket
    */
   typedef void (* TcpTxRxTracedCallback)(const Ptr<const Packet> packet, const TcpHeader& header,
                                          const Ptr<const TcpSocketBase> socket);
@@ -603,9 +640,9 @@ protected:
    * Note that this function did not implement the PSH flag.
    *
    * \param withAck forces an ACK to be sent
-   * \returns true if some data have been sent
+   * \returns the number of packets sent
    */
-  bool SendPendingData (bool withAck = false);
+  uint32_t SendPendingData (bool withAck = false);
 
   /**
    * \brief Extract at most maxSize bytes from the TxBuffer at sequence seq, add the
@@ -619,7 +656,7 @@ protected:
   uint32_t SendDataPacket (SequenceNumber32 seq, uint32_t maxSize, bool withAck);
 
   /**
-   * \brief Send a empty packet that carries a flag, e.g. ACK
+   * \brief Send a empty packet that carries a flag, e.g., ACK
    *
    * \param flags the packet's flags
    */
@@ -768,15 +805,21 @@ protected:
 
   /**
    * \brief Return count of number of unacked bytes
+   *
+   * The difference between SND.UNA and HighTx
+   *
    * \returns count of number of unacked bytes
    */
   virtual uint32_t UnAckDataCount (void) const;
 
   /**
    * \brief Return total bytes in flight
+   *
+   * Does not count segments lost and SACKed (or dupACKed)
+   *
    * \returns total bytes in flight
    */
-  virtual uint32_t BytesInFlight (void);
+  virtual uint32_t BytesInFlight (void) const;
 
   /**
    * \brief Return the max possible number of unacked bytes
@@ -828,6 +871,14 @@ protected:
    */
   virtual void ReceivedAck (Ptr<Packet> packet, const TcpHeader& tcpHeader);
 
+  /**
+   * \brief Process a received ack
+   * \param ackNumber ack number
+   * \param scoreboardUpdated if true indicates that the scoreboard has been
+   * updated with SACK information
+   */
+  virtual void ProcessAck (const SequenceNumber32 &ackNumber, bool scoreboardUpdated);
+
   /**
    * \brief Recv of a data, put into buffer, call L7 to get it if necessary
    * \param packet the packet
@@ -870,20 +921,15 @@ protected:
   void LimitedTransmit ();
 
   /**
-   * \brief Enter the FastRetransmit, and retransmit the head
+   * \brief Enter the CA_RECOVERY, and retransmit the head
    */
-  void FastRetransmit ();
+  void EnterRecovery ();
 
   /**
-   * \brief Call Retransmit() upon RTO event
+   * \brief An RTO event happened
    */
   virtual void ReTxTimeout (void);
 
-  /**
-   * \brief Halving cwnd and call DoRetransmit()
-   */
-  virtual void Retransmit (void);
-
   /**
    * \brief Action upon delay ACK timeout, i.e. send an ACK
    */
@@ -911,7 +957,26 @@ protected:
    *
    * \param tcpHeader TcpHeader to add options to
    */
-  virtual void AddOptions (TcpHeader& tcpHeader);
+  void AddOptions (TcpHeader& tcpHeader);
+
+  /**
+   * \brief Read TCP options begore Ack processing
+   *
+   * Timestamp and Window scale are managed in other pieces of code.
+   *
+   * \param tcpHeader Header of the segment
+   * \param scoreboardUpdated indicates if the scoreboard was updated due to a
+   * SACK option
+   */
+  void ReadOptions (const TcpHeader &tcpHeader, bool &scoreboardUpdated);
+
+  /**
+   * \brief Return true if the specified option is enabled
+   *
+   * \param kind kind of TCP option
+   * \return true if the option is enabled
+   */
+  bool IsTcpOptionEnabled (uint8_t kind) const;
 
   /**
    * \brief Read and parse the Window scale option
@@ -941,6 +1006,38 @@ protected:
    */
   uint8_t CalculateWScale () const;
 
+  /**
+   * \brief Read the SACK PERMITTED option
+   *
+   * Currently this is a placeholder, since no operations should be done
+   * on such option.
+   *
+   * \param option SACK PERMITTED option from the header
+   */
+  void ProcessOptionSackPermitted (const Ptr<const TcpOption> option);
+
+  /**
+   * \brief Read the SACK option
+   *
+   * \param option SACK option from the header
+   * \returns true in case of an update to the SACKed blocks
+   */
+  bool ProcessOptionSack (const Ptr<const TcpOption> option);
+
+  /**
+   * \brief Add the SACK PERMITTED option to the header
+   *
+   * \param header TcpHeader where the method should add the option
+   */
+  void AddOptionSackPermitted (TcpHeader &header);
+
+  /**
+   * \brief Add the SACK option to the header
+   *
+   * \param header TcpHeader where the method should add the option
+   */
+  void AddOptionSack (TcpHeader& header);
+
   /** \brief Process the timestamp option from other side
    *
    * Get the timestamp and the echo, then save timestamp (which will
@@ -1025,6 +1122,7 @@ protected:
   // Window management
   uint16_t              m_maxWinSize;  //!< Maximum window size to advertise
   TracedValue<uint32_t> m_rWnd;        //!< Receiver window (RCV.WND in RFC793)
+  TracedValue<uint32_t> m_advWnd;      //!< Advertised Window size
   TracedValue<SequenceNumber32> m_highRxMark;     //!< Highest seqno received
   SequenceNumber32 m_highTxAck;                   //!< Highest ack sent
   TracedValue<SequenceNumber32> m_highRxAckMark;  //!< Highest ack received
@@ -1032,6 +1130,7 @@ protected:
   TracedValue<uint32_t>         m_bytesInFlight; //!< Bytes in flight
 
   // Options
+  bool    m_sackEnabled;       //!< RFC SACK option enabled
   bool    m_winScalingEnabled; //!< Window Scale option enabled (RFC 7323)
   uint8_t m_rcvWindShift;      //!< Window shift to apply to outgoing segments
   uint8_t m_sndWindShift;      //!< Window shift to apply to incoming segments
@@ -1045,7 +1144,6 @@ protected:
   SequenceNumber32       m_recover;      //!< Previous highest Tx seqnum for fast recovery
   uint32_t               m_retxThresh;   //!< Fast Retransmit threshold
   bool                   m_limitedTx;    //!< perform limited transmit
-  uint32_t               m_retransOut;   //!< Number of retransmission in this window
 
   // Transmission Control Block
   Ptr<TcpSocketState>    m_tcb;               //!< Congestion control informations
