@@ -24,12 +24,13 @@
 #include "ns3/ipv4-address-helper.h"
 #include "ns3/internet-stack-helper.h"
 #include "ns3/log.h"
+#include "ns3/queue.h"
 #include "ns3/tcp-l4-protocol.h"
 #include "../model/ipv4-end-point.h"
 #include "../model/ipv6-end-point.h"
 #include "tcp-general-test.h"
 
-namespace ns3 {
+using namespace ns3;
 
 NS_LOG_COMPONENT_DEFINE ("TcpGeneralTest");
 
@@ -173,7 +174,8 @@ TcpGeneralTest::DoRun (void)
                                        MakeCallback (&TcpGeneralTest::ErrorCloseCb, this));
   m_receiverSocket->SetRcvAckCb (MakeCallback (&TcpGeneralTest::RcvAckCb, this));
   m_receiverSocket->SetProcessedAckCb (MakeCallback (&TcpGeneralTest::ProcessedAckCb, this));
-  m_receiverSocket->SetRetransmitCb (MakeCallback (&TcpGeneralTest::RtoExpiredCb, this));
+  m_receiverSocket->SetAfterRetransmitCb (MakeCallback (&TcpGeneralTest::AfterRetransmitCb, this));
+  m_receiverSocket->SetBeforeRetransmitCb (MakeCallback (&TcpGeneralTest::BeforeRetransmitCb, this));
   m_receiverSocket->SetForkCb (MakeCallback (&TcpGeneralTest::ForkCb, this));
   m_receiverSocket->SetUpdateRttHistoryCb (MakeCallback (&TcpGeneralTest::UpdateRttHistoryCb, this));
   m_receiverSocket->TraceConnectWithoutContext ("Tx",
@@ -189,7 +191,8 @@ TcpGeneralTest::DoRun (void)
                                      MakeCallback (&TcpGeneralTest::ErrorCloseCb, this));
   m_senderSocket->SetRcvAckCb (MakeCallback (&TcpGeneralTest::RcvAckCb, this));
   m_senderSocket->SetProcessedAckCb (MakeCallback (&TcpGeneralTest::ProcessedAckCb, this));
-  m_senderSocket->SetRetransmitCb (MakeCallback (&TcpGeneralTest::RtoExpiredCb, this));
+  m_senderSocket->SetAfterRetransmitCb (MakeCallback (&TcpGeneralTest::AfterRetransmitCb, this));
+  m_senderSocket->SetBeforeRetransmitCb (MakeCallback (&TcpGeneralTest::BeforeRetransmitCb, this));
   m_senderSocket->SetDataSentCallback (MakeCallback (&TcpGeneralTest::DataSentCb, this));
   m_senderSocket->SetUpdateRttHistoryCb (MakeCallback (&TcpGeneralTest::UpdateRttHistoryCb, this));
   m_senderSocket->TraceConnectWithoutContext ("CongestionWindow",
@@ -370,16 +373,34 @@ TcpGeneralTest::UpdateRttHistoryCb (Ptr<const TcpSocketBase> tcp,
 }
 
 void
-TcpGeneralTest::RtoExpiredCb (const Ptr<const TcpSocketState> tcb,
-                              const Ptr<const TcpSocketBase> tcp)
+TcpGeneralTest::AfterRetransmitCb (const Ptr<const TcpSocketState> tcb,
+                                   const Ptr<const TcpSocketBase> tcp)
 {
   if (tcp->GetNode () == m_receiverSocket->GetNode ())
     {
-      RTOExpired (tcb, RECEIVER);
+      AfterRTOExpired (tcb, RECEIVER);
     }
   else if (tcp->GetNode () == m_senderSocket->GetNode ())
     {
-      RTOExpired (tcb, SENDER);
+      AfterRTOExpired (tcb, SENDER);
+    }
+  else
+    {
+      NS_FATAL_ERROR ("Closed socket, but not recognized");
+    }
+}
+
+void
+TcpGeneralTest::BeforeRetransmitCb (const Ptr<const TcpSocketState> tcb,
+                                    const Ptr<const TcpSocketBase> tcp)
+{
+  if (tcp->GetNode () == m_receiverSocket->GetNode ())
+    {
+      BeforeRTOExpired (tcb, RECEIVER);
+    }
+  else if (tcp->GetNode () == m_senderSocket->GetNode ())
+    {
+      BeforeRTOExpired (tcb, SENDER);
     }
   else
     {
@@ -933,10 +954,17 @@ TcpSocketMsgBase::SetProcessedAckCb (AckManagementCb cb)
 }
 
 void
-TcpSocketMsgBase::SetRetransmitCb (RetrCb cb)
+TcpSocketMsgBase::SetAfterRetransmitCb (RetrCb cb)
 {
   NS_ASSERT (!cb.IsNull ());
-  m_retrCallback = cb;
+  m_afterRetrCallback = cb;
+}
+
+void
+TcpSocketMsgBase::SetBeforeRetransmitCb (RetrCb cb)
+{
+  NS_ASSERT (!cb.IsNull ());
+  m_beforeRetrCallback = cb;
 }
 
 void
@@ -951,11 +979,11 @@ TcpSocketMsgBase::ReceivedAck (Ptr<Packet> packet, const TcpHeader& tcpHeader)
 }
 
 void
-TcpSocketMsgBase::Retransmit ()
+TcpSocketMsgBase::ReTxTimeout ()
 {
-  TcpSocketBase::Retransmit ();
-
-  m_retrCallback (m_tcb, this);
+  m_beforeRetrCallback (m_tcb, this);
+  TcpSocketBase::ReTxTimeout ();
+  m_afterRetrCallback (m_tcb, this);
 }
 
 void
@@ -1008,8 +1036,8 @@ TcpSocketSmallAcks::GetTypeId (void)
   return tid;
 }
 
-/**
- * \brief Send empty packet, copied/pasted from TcpSocketBase
+/*
+ * Send empty packet, copied/pasted from TcpSocketBase
  *
  * The rationale for copying/pasting is that we need to edit a little the
  * code inside. Since there isn't a well-defined division of duties,
@@ -1176,4 +1204,3 @@ TcpSocketSmallAcks::Fork (void)
   return CopyObject<TcpSocketSmallAcks> (this);
 }
 
-} // namespace ns3
