@@ -17,6 +17,9 @@
  *
  * Author: Marco Miozzo <marco.miozzo@cttc.es>
  *         Nicola Baldo  <nbaldo@cttc.es>
+ * Modified by:
+ *          Danilo Abrignani <danilo.abrignani@unibo.it> (Carrier Aggregation - GSoC 2015)
+ *          Biljana Bojovic <biljana.bojovic@cttc.es> (Carrier Aggregation)
  */
 
 
@@ -35,6 +38,7 @@
 #include <ns3/lte-ue-phy.h>
 
 #include "ns3/lte-mac-sap.h"
+#include "ns3/lte-enb-cmac-sap.h"
 #include <ns3/lte-common.h>
 
 
@@ -51,9 +55,15 @@ NS_OBJECT_ENSURE_REGISTERED (LteEnbMac);
 // //////////////////////////////////////
 
 
+/// EnbMacMemberLteEnbCmacSapProvider class
 class EnbMacMemberLteEnbCmacSapProvider : public LteEnbCmacSapProvider
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mac the MAC
+   */
   EnbMacMemberLteEnbCmacSapProvider (LteEnbMac* mac);
 
   // inherited from LteEnbCmacSapProvider
@@ -69,7 +79,7 @@ public:
   
 
 private:
-  LteEnbMac* m_mac;
+  LteEnbMac* m_mac; ///< the MAC
 };
 
 
@@ -133,16 +143,22 @@ EnbMacMemberLteEnbCmacSapProvider::AllocateNcRaPreamble (uint16_t rnti)
 }
 
 
+/// EnbMacMemberFfMacSchedSapUser class
 class EnbMacMemberFfMacSchedSapUser : public FfMacSchedSapUser
 {
 public:
+  /**
+   * Constructor
+   * 
+   * \param mac the MAC
+   */
   EnbMacMemberFfMacSchedSapUser (LteEnbMac* mac);
 
 
   virtual void SchedDlConfigInd (const struct SchedDlConfigIndParameters& params);
   virtual void SchedUlConfigInd (const struct SchedUlConfigIndParameters& params);
 private:
-  LteEnbMac* m_mac;
+  LteEnbMac* m_mac; ///< the MAC
 };
 
 
@@ -167,10 +183,15 @@ EnbMacMemberFfMacSchedSapUser::SchedUlConfigInd (const struct SchedUlConfigIndPa
 }
 
 
-
+/// EnbMacMemberFfMacCschedSapUser class
 class EnbMacMemberFfMacCschedSapUser : public FfMacCschedSapUser
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mac the MAC
+   */
   EnbMacMemberFfMacCschedSapUser (LteEnbMac* mac);
 
   virtual void CschedCellConfigCnf (const struct CschedCellConfigCnfParameters& params);
@@ -182,7 +203,7 @@ public:
   virtual void CschedCellConfigUpdateInd (const struct CschedCellConfigUpdateIndParameters& params);
 
 private:
-  LteEnbMac* m_mac;
+  LteEnbMac* m_mac; ///< the MAC
 };
 
 
@@ -235,12 +256,15 @@ EnbMacMemberFfMacCschedSapUser::CschedCellConfigUpdateInd (const struct CschedCe
 
 
 
-// ---------- PHY-SAP
-
-
+/// ---------- PHY-SAP
 class EnbMacMemberLteEnbPhySapUser : public LteEnbPhySapUser
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mac the MAC
+   */
   EnbMacMemberLteEnbPhySapUser (LteEnbMac* mac);
 
   // inherited from LteEnbPhySapUser
@@ -253,7 +277,7 @@ public:
   virtual void DlInfoListElementHarqFeeback (DlInfoListElement_s params);
 
 private:
-  LteEnbMac* m_mac;
+  LteEnbMac* m_mac; ///< the MAC
 };
 
 EnbMacMemberLteEnbPhySapUser::EnbMacMemberLteEnbPhySapUser (LteEnbMac* mac) : m_mac (mac)
@@ -339,13 +363,19 @@ LteEnbMac::GetTypeId (void)
                      "Information regarding UL scheduling.",
                      MakeTraceSourceAccessor (&LteEnbMac::m_ulScheduling),
                      "ns3::LteEnbMac::UlSchedulingTracedCallback")
+    .AddAttribute ("ComponentCarrierId",
+                   "ComponentCarrier Id, needed to reply on the appropriate sap.",
+                   UintegerValue (0),
+                   MakeUintegerAccessor (&LteEnbMac::m_componentCarrierId),
+                   MakeUintegerChecker<uint8_t> (0,4))
   ;
 
   return tid;
 }
 
 
-LteEnbMac::LteEnbMac ()
+LteEnbMac::LteEnbMac ():
+m_ccmMacSapUser (0)
 {
   NS_LOG_FUNCTION (this);
   m_macSapProvider = new EnbMacMemberLteMacSapProvider<LteEnbMac> (this);
@@ -353,6 +383,7 @@ LteEnbMac::LteEnbMac ()
   m_schedSapUser = new EnbMacMemberFfMacSchedSapUser (this);
   m_cschedSapUser = new EnbMacMemberFfMacCschedSapUser (this);
   m_enbPhySapUser = new EnbMacMemberLteEnbPhySapUser (this);
+  m_ccmMacSapProvider = new MemberLteCcmMacSapProvider<LteEnbMac> (this);
 }
 
 
@@ -376,8 +407,14 @@ LteEnbMac::DoDispose ()
   delete m_schedSapUser;
   delete m_cschedSapUser;
   delete m_enbPhySapUser;
+  delete m_ccmMacSapProvider;
 }
 
+void
+LteEnbMac::SetComponentCarrierId (uint8_t index)
+{
+  m_componentCarrierId = index;
+}
 
 void
 LteEnbMac::SetFfMacSchedSapProvider (FfMacSchedSapProvider* s)
@@ -442,8 +479,19 @@ LteEnbMac::GetLteEnbPhySapUser ()
   return m_enbPhySapUser;
 }
 
+void
+LteEnbMac::SetLteCcmMacSapUser (LteCcmMacSapUser* s)
+{
+  m_ccmMacSapUser = s;
+}
 
 
+LteCcmMacSapProvider*
+LteEnbMac::GetLteCcmMacSapProvider ()
+{
+  return m_ccmMacSapProvider;
+}
+
 void
 LteEnbMac::DoSubframeIndication (uint32_t frameNo, uint32_t subframeNo)
 {
@@ -460,12 +508,6 @@ LteEnbMac::DoSubframeIndication (uint32_t frameNo, uint32_t subframeNo)
     {
       FfMacSchedSapProvider::SchedDlCqiInfoReqParameters dlcqiInfoReq;
       dlcqiInfoReq.m_sfnSf = ((0x3FF & frameNo) << 4) | (0xF & subframeNo);
-
-      int cqiNum = m_dlCqiReceived.size ();
-      if (cqiNum > MAX_CQI_LIST)
-        {
-          cqiNum = MAX_CQI_LIST;
-        }
       dlcqiInfoReq.m_cqiList.insert (dlcqiInfoReq.m_cqiList.begin (), m_dlCqiReceived.begin (), m_dlCqiReceived.end ());
       m_dlCqiReceived.erase (m_dlCqiReceived.begin (), m_dlCqiReceived.end ());
       m_schedSapProvider->SchedDlCqiInfoReq (dlcqiInfoReq);
@@ -663,8 +705,17 @@ void
 LteEnbMac::ReceiveBsrMessage  (MacCeListElement_s bsr)
 {
   NS_LOG_FUNCTION (this);
+  m_ccmMacSapUser->UlReceiveMacCe (bsr, m_componentCarrierId);
+}
 
-  m_ulCeReceived.push_back (bsr);
+void
+LteEnbMac::DoReportMacCeToScheduler (MacCeListElement_s bsr)
+{
+  NS_LOG_FUNCTION (this);
+  NS_LOG_DEBUG (this << " bsr Size " << (uint16_t) m_ulCeReceived.size ());
+  //send to LteCcmMacSapUser
+  m_ulCeReceived.push_back (bsr); // this to called when LteUlCcmSapProvider::ReportMacCeToScheduler is called
+  NS_LOG_DEBUG (this << " bsr Size after push_back " << (uint16_t) m_ulCeReceived.size ());
 }
 
 
@@ -718,7 +769,7 @@ LteEnbMac::DoReceivePhyPdu (Ptr<Packet> p)
   //Receive PDU only if LCID is found
   if (lcidIt != rntiIt->second.end ())
     {
-      (*lcidIt).second->ReceivePdu (p);
+      (*lcidIt).second->ReceivePdu (p, rnti, lcid);
     }
 }
 
@@ -793,7 +844,7 @@ LteEnbMac::DoRemoveUe (uint16_t rnti)
 void
 LteEnbMac::DoAddLc (LteEnbCmacSapProvider::LcInfo lcinfo, LteMacSapUser* msu)
 {
-  NS_LOG_FUNCTION (this);
+  NS_LOG_FUNCTION (this << lcinfo.rnti << (uint16_t) lcinfo.lcId);
 
   std::map <LteFlowId_t, LteMacSapUser* >::iterator it;
   
@@ -932,6 +983,7 @@ LteEnbMac::DoTransmitPdu (LteMacSapProvider::TransmitPduParameters params)
   NS_LOG_FUNCTION (this);
   LteRadioBearerTag tag (params.rnti, params.lcid, params.layer);
   params.pdu->AddPacketTag (tag);
+  params.componentCarrierId = m_componentCarrierId;
   // Store pkt in HARQ buffer
   std::map <uint16_t, DlHarqProcessesBuffer_t>::iterator it =  m_miDlHarqProcessesPackets.find (params.rnti);
   NS_ASSERT (it != m_miDlHarqProcessesPackets.end ());
@@ -1001,9 +1053,9 @@ LteEnbMac::DoSchedDlConfigInd (FfMacSchedSapUser::SchedDlConfigIndParameters ind
                   std::map <uint16_t, std::map<uint8_t, LteMacSapUser*> >::iterator rntiIt = m_rlcAttached.find (rnti);
                   NS_ASSERT_MSG (rntiIt != m_rlcAttached.end (), "could not find RNTI" << rnti);
                   std::map<uint8_t, LteMacSapUser*>::iterator lcidIt = rntiIt->second.find (lcid);
-                  NS_ASSERT_MSG (lcidIt != rntiIt->second.end (), "could not find LCID" << lcid);
+                  NS_ASSERT_MSG (lcidIt != rntiIt->second.end (), "could not find LCID" << (uint32_t)lcid<<" carrier id:"<<(uint16_t)m_componentCarrierId);
                   NS_LOG_DEBUG (this << " rnti= " << rnti << " lcid= " << (uint32_t) lcid << " layer= " << k);
-                  (*lcidIt).second->NotifyTxOpportunity (ind.m_buildDataList.at (i).m_rlcPduList.at (j).at (k).m_size, k, ind.m_buildDataList.at (i).m_dci.m_harqProcess);
+                  (*lcidIt).second->NotifyTxOpportunity (ind.m_buildDataList.at (i).m_rlcPduList.at (j).at (k).m_size, k, ind.m_buildDataList.at (i).m_dci.m_harqProcess, m_componentCarrierId, rnti, lcid);
                 }
               else
                 {
@@ -1034,22 +1086,30 @@ LteEnbMac::DoSchedDlConfigInd (FfMacSchedSapUser::SchedDlConfigIndParameters ind
       // Only one TB used
       if (ind.m_buildDataList.at (i).m_dci.m_tbsSize.size () == 1)
         {
-          m_dlScheduling (m_frameNo, m_subframeNo, ind.m_buildDataList.at (i).m_dci.m_rnti,
-                          ind.m_buildDataList.at (i).m_dci.m_mcs.at (0),
-                          ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (0),
-                          0, 0
-                          );
-
+          DlSchedulingCallbackInfo dlSchedulingCallbackInfo;
+          dlSchedulingCallbackInfo.frameNo = m_frameNo;
+          dlSchedulingCallbackInfo.subframeNo = m_subframeNo;
+          dlSchedulingCallbackInfo.rnti = ind.m_buildDataList.at (i).m_dci.m_rnti;
+          dlSchedulingCallbackInfo.mcsTb1=ind.m_buildDataList.at (i).m_dci.m_mcs.at (0);
+          dlSchedulingCallbackInfo.sizeTb1 = ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (0);
+          dlSchedulingCallbackInfo.mcsTb2 = 0;
+          dlSchedulingCallbackInfo.sizeTb2 = 0;
+          dlSchedulingCallbackInfo.componentCarrierId = m_componentCarrierId;
+          m_dlScheduling(dlSchedulingCallbackInfo);
         }
       // Two TBs used
       else if (ind.m_buildDataList.at (i).m_dci.m_tbsSize.size () == 2)
         {
-          m_dlScheduling (m_frameNo, m_subframeNo, ind.m_buildDataList.at (i).m_dci.m_rnti,
-                          ind.m_buildDataList.at (i).m_dci.m_mcs.at (0),
-                          ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (0),
-                          ind.m_buildDataList.at (i).m_dci.m_mcs.at (1),
-                          ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (1)
-                          );
+          DlSchedulingCallbackInfo dlSchedulingCallbackInfo;
+          dlSchedulingCallbackInfo.frameNo = m_frameNo;
+          dlSchedulingCallbackInfo.subframeNo = m_subframeNo;
+          dlSchedulingCallbackInfo.rnti = ind.m_buildDataList.at (i).m_dci.m_rnti;
+          dlSchedulingCallbackInfo.mcsTb1=ind.m_buildDataList.at (i).m_dci.m_mcs.at (0);
+          dlSchedulingCallbackInfo.sizeTb1 = ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (0);
+          dlSchedulingCallbackInfo.mcsTb2 = ind.m_buildDataList.at (i).m_dci.m_mcs.at (1);
+          dlSchedulingCallbackInfo.sizeTb2 = ind.m_buildDataList.at (i).m_dci.m_tbsSize.at (1);
+          dlSchedulingCallbackInfo.componentCarrierId = m_componentCarrierId;
+          m_dlScheduling(dlSchedulingCallbackInfo);
         }
       else
         {
@@ -1109,7 +1169,7 @@ LteEnbMac::DoSchedUlConfigInd (FfMacSchedSapUser::SchedUlConfigIndParameters ind
   for (  uint32_t i  = 0; i < ind.m_dciList.size (); i++ )
     {
       m_ulScheduling (m_frameNo, m_subframeNo, ind.m_dciList.at (i).m_rnti,
-                      ind.m_dciList.at (i).m_mcs, ind.m_dciList.at (i).m_tbSize);
+                      ind.m_dciList.at (i).m_mcs, ind.m_dciList.at (i).m_tbSize, m_componentCarrierId);
     }
 
 
