@@ -34,6 +34,7 @@ NS_LOG_COMPONENT_DEFINE ("LteFfrEnhancedAlgorithm");
 
 NS_OBJECT_ENSURE_REGISTERED (LteFfrEnhancedAlgorithm);
 
+/// Spectral efficency for CQI table
 static const double SpectralEfficiencyForCqi[16] = {
   0.0, // out of range
   0.15, 0.23, 0.38, 0.6, 0.88, 1.18,
@@ -41,13 +42,14 @@ static const double SpectralEfficiencyForCqi[16] = {
   2.73, 3.32, 3.9, 4.52, 5.12, 5.55
 };
 
+/// FfrEnhancedDownlinkDefaultConfiguration structure
 static const struct FfrEnhancedDownlinkDefaultConfiguration
 {
-  uint8_t cellId;
-  uint8_t dlBandwidth;
-  uint8_t dlSubBandOffset;
-  uint8_t dlReuse3SubBandwidth;
-  uint8_t dlReuse1SubBandwidth;
+  uint8_t cellId; ///< cell ID
+  uint8_t dlBandwidth; ///< DL bandwidth
+  uint8_t dlSubBandOffset; ///< DL subband offset
+  uint8_t dlReuse3SubBandwidth; ///< reuse 3 subbandwidth 
+  uint8_t dlReuse1SubBandwidth; ///< reuse 1 subbandwidth
 } g_ffrEnhancedDownlinkDefaultConfiguration[] = {
   { 1, 25, 0, 4, 4},
   { 2, 25, 8, 4, 4},
@@ -61,15 +63,16 @@ static const struct FfrEnhancedDownlinkDefaultConfiguration
   { 1, 100, 0, 16, 16},
   { 2, 100, 32, 16, 16},
   { 3, 100, 64, 16, 16}
-};
+}; ///< the enhanced downlink default configation
 
+/// FfrEnhancedUplinkDefaultConfiguration structure
 static const struct FfrEnhancedUplinkDefaultConfiguration
 {
-  uint8_t cellId;
-  uint8_t ulBandwidth;
-  uint8_t ulSubBandOffset;
-  uint8_t ulReuse3SubBandwidth;
-  uint8_t ulReuse1SubBandwidth;
+  uint8_t cellId; ///< cell ID
+  uint8_t ulBandwidth; ///< UL bandwidth
+  uint8_t ulSubBandOffset; ///< UL subband offset
+  uint8_t ulReuse3SubBandwidth; ///< UL reuse 3 subbandwidth
+  uint8_t ulReuse1SubBandwidth; ///< UL reuse 1 subbandwidth
 } g_ffrEnhancedUplinkDefaultConfiguration[] = {
   { 1, 25, 0, 4, 4},
   { 2, 25, 8, 4, 4},
@@ -83,9 +86,11 @@ static const struct FfrEnhancedUplinkDefaultConfiguration
   { 1, 100, 0, 16, 16},
   { 2, 100, 32, 16, 16},
   { 3, 100, 64, 16, 16}
-};
+}; ///< the enhanced uplink default configuration
 
+/** \returns number of downlink configurations */
 const uint16_t NUM_DOWNLINK_CONFS (sizeof (g_ffrEnhancedDownlinkDefaultConfiguration) / sizeof (FfrEnhancedDownlinkDefaultConfiguration));
+/** \returns number of uplink configurations */
 const uint16_t NUM_UPLINK_CONFS (sizeof (g_ffrEnhancedUplinkDefaultConfiguration) / sizeof (FfrEnhancedUplinkDefaultConfiguration));
 
 
@@ -771,7 +776,7 @@ LteFfrEnhancedAlgorithm::EstimateUlSinr (uint16_t rnti, uint16_t rb, std::map <u
     {
       // take the average SINR value among the available
       double sinrSum = 0;
-      int sinrNum = 0;
+      unsigned int sinrNum = 0;
       for (uint32_t i = 0; i < m_ulBandwidth; i++)
         {
           double sinr = (*itCqi).second.at (i);
