@@ -35,6 +35,12 @@ namespace ns3 {
 
 NS_LOG_COMPONENT_DEFINE ("LteRrcProtocolIdeal");
 
+/**
+ * \ingroup lte
+ *
+ */
+
+/// RRC ideal message delay
 static const Time RRC_IDEAL_MSG_DELAY = MilliSeconds (0);
 
 NS_OBJECT_ENSURE_REGISTERED (LteUeRrcProtocolIdeal);
@@ -190,7 +196,7 @@ LteUeRrcProtocolIdeal::SetEnbRrcSapProvider ()
             }
           else
             {
-              if (enbDev->GetCellId () == cellId)
+              if (enbDev->HasCellId (cellId))
                 {
                   found = true;          
                   break;
@@ -323,9 +329,9 @@ LteEnbRrcProtocolIdeal::DoRemoveUe (uint16_t rnti)
 }
 
 void 
-LteEnbRrcProtocolIdeal::DoSendSystemInformation (LteRrcSap::SystemInformation msg)
+LteEnbRrcProtocolIdeal::DoSendSystemInformation (uint16_t cellId, LteRrcSap::SystemInformation msg)
 {
-  NS_LOG_FUNCTION (this << m_cellId);
+  NS_LOG_FUNCTION (this << cellId);
   // walk list of all nodes to get UEs with this cellId
   Ptr<LteUeRrc> ueRrc;
   for (NodeList::Iterator i = NodeList::Begin (); i != NodeList::End (); ++i)
@@ -339,7 +345,7 @@ LteEnbRrcProtocolIdeal::DoSendSystemInformation (LteRrcSap::SystemInformation ms
             {
               Ptr<LteUeRrc> ueRrc = ueDev->GetRrc ();              
               NS_LOG_LOGIC ("considering UE IMSI " << ueDev->GetImsi () << " that has cellId " << ueRrc->GetCellId ());
-              if (ueRrc->GetCellId () == m_cellId)
+              if (ueRrc->GetCellId () == cellId)
                 {       
                   NS_LOG_LOGIC ("sending SI to IMSI " << ueDev->GetImsi ());
                   ueRrc->GetLteUeRrcSapProvider ()->RecvSystemInformation (msg);
@@ -419,10 +425,10 @@ LteEnbRrcProtocolIdeal::DoSendRrcConnectionReject (uint16_t rnti, LteRrcSap::Rrc
  * 
  */
 
-static std::map<uint32_t, LteRrcSap::HandoverPreparationInfo> g_handoverPreparationInfoMsgMap;
-static uint32_t g_handoverPreparationInfoMsgIdCounter = 0;
+static std::map<uint32_t, LteRrcSap::HandoverPreparationInfo> g_handoverPreparationInfoMsgMap; ///< handover preparation info message map
+static uint32_t g_handoverPreparationInfoMsgIdCounter = 0; ///< handover preparation info message ID counter
 
-/*
+/**
  * This header encodes the map key discussed above. We keep this
  * private since it should not be used outside this file.
  * 
@@ -430,8 +436,22 @@ static uint32_t g_handoverPreparationInfoMsgIdCounter = 0;
 class IdealHandoverPreparationInfoHeader : public Header
 {
 public:
+  /**
+   * Get the message ID function
+   *
+   * \returns the message ID
+   */
   uint32_t GetMsgId ();
+  /**
+   * Set the message ID function
+   *
+   * \param id the message ID 
+   */
   void SetMsgId (uint32_t id);
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   virtual TypeId GetInstanceTypeId (void) const;
   virtual void Print (std::ostream &os) const;
@@ -440,7 +460,7 @@ public:
   virtual uint32_t Deserialize (Buffer::Iterator start);
 
 private:
-  uint32_t m_msgId;
+  uint32_t m_msgId; ///< message ID
 };
 
 uint32_t 
@@ -526,10 +546,10 @@ LteEnbRrcProtocolIdeal::DoDecodeHandoverPreparationInformation (Ptr<Packet> p)
 
 
 
-static std::map<uint32_t, LteRrcSap::RrcConnectionReconfiguration> g_handoverCommandMsgMap;
-static uint32_t g_handoverCommandMsgIdCounter = 0;
+static std::map<uint32_t, LteRrcSap::RrcConnectionReconfiguration> g_handoverCommandMsgMap; ///< handover command message map
+static uint32_t g_handoverCommandMsgIdCounter = 0; ///< handover command message ID counter
 
-/*
+/**
  * This header encodes the map key discussed above. We keep this
  * private since it should not be used outside this file.
  * 
@@ -537,8 +557,22 @@ static uint32_t g_handoverCommandMsgIdCounter = 0;
 class IdealHandoverCommandHeader : public Header
 {
 public:
+  /**
+   * Get the message ID function
+   *
+   * \returns the message ID
+   */
   uint32_t GetMsgId ();
+  /**
+   * Set the message ID function
+   *
+   * \param id the message ID
+   */
   void SetMsgId (uint32_t id);
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   virtual TypeId GetInstanceTypeId (void) const;
   virtual void Print (std::ostream &os) const;
@@ -547,7 +581,7 @@ public:
   virtual uint32_t Deserialize (Buffer::Iterator start);
 
 private:
-  uint32_t m_msgId;
+  uint32_t m_msgId; ///< message ID
 };
 
 uint32_t 
