@@ -44,14 +44,15 @@ namespace ns3 {
 NS_LOG_COMPONENT_DEFINE ("LteSpectrumPhy");
 
 
-// duration of SRS portion of UL subframe  
-// = 1 symbol for SRS -1ns as margin to avoid overlapping simulator events
+/// duration of SRS portion of UL subframe  
+/// = 1 symbol for SRS -1ns as margin to avoid overlapping simulator events
 static const Time UL_SRS_DURATION = NanoSeconds (71429 -1);  
 
-// duration of the control portion of a subframe
-// = 0.001 / 14 * 3 (ctrl fixed to 3 symbols) -1ns as margin to avoid overlapping simulator events
+/// duration of the control portion of a subframe
+/// = 0.001 / 14 * 3 (ctrl fixed to 3 symbols) -1ns as margin to avoid overlapping simulator events
 static const Time DL_CTRL_DURATION = NanoSeconds (214286 -1);
 
+/// Effective coding rate
 static const double EffectiveCodingRate[29] = {
   0.08,
   0.1,
@@ -97,12 +98,26 @@ TbId_t::TbId_t (const uint16_t a, const uint8_t b)
 {
 }
 
+/**
+ * Equality operator
+ *
+ * \param a lhs
+ * \param b rhs
+ * \returns true if rnti and layer are equal
+ */
 bool
 operator == (const TbId_t &a, const TbId_t &b)
 {
   return ( (a.m_rnti == b.m_rnti) && (a.m_layer == b.m_layer) );
 }
 
+/**
+ * Less than operator
+ *
+ * \param a lhs
+ * \param b rhs
+ * \returns true if rnti less than ro rnti equal and layer less than
+ */
 bool
 operator < (const TbId_t& a, const TbId_t& b)
 {
@@ -114,8 +129,9 @@ NS_OBJECT_ENSURE_REGISTERED (LteSpectrumPhy);
 LteSpectrumPhy::LteSpectrumPhy ()
   : m_state (IDLE),
     m_cellId (0),
-  m_transmissionMode (0),
-  m_layersNum (1)
+    m_componentCarrierId (0),
+    m_transmissionMode (0),
+    m_layersNum (1)
 {
   NS_LOG_FUNCTION (this);
   m_random = CreateObject<UniformRandomVariable> ();
@@ -158,6 +174,13 @@ void LteSpectrumPhy::DoDispose ()
   SpectrumPhy::DoDispose ();
 } 
 
+/**
+ * Output stream output operator
+ *
+ * \param os output stream
+ * \param s state
+ * \returns output stream
+ */
 std::ostream& operator<< (std::ostream& os, LteSpectrumPhy::State s)
 {
   switch (s)
@@ -984,6 +1007,7 @@ LteSpectrumPhy::EndRxData ()
           params.m_rv = (*itTb).second.rv;
           params.m_ndi = (*itTb).second.ndi;
           params.m_correctness = (uint8_t)!(*itTb).second.corrupt;
+          params.m_ccId = m_componentCarrierId;
           if ((*itTb).second.downlink)
             {
               // DL
@@ -1062,7 +1086,7 @@ LteSpectrumPhy::EndRxData ()
                         if (itHarq==harqDlInfoMap.end ())
                           {
                             DlInfoListElement_s harqDlInfo;
-                            harqDlInfo.m_harqStatus.resize (m_layersNum, DlInfoListElement_s::NACK);
+                            harqDlInfo.m_harqStatus.resize (m_layersNum, DlInfoListElement_s::ACK);
                             harqDlInfo.m_rnti = tbId.m_rnti;
                             harqDlInfo.m_harqProcessId = (*itTb).second.harqProcessId;
                             if ((*itTb).second.corrupt)
@@ -1189,6 +1213,11 @@ LteSpectrumPhy::SetCellId (uint16_t cellId)
   m_cellId = cellId;
 }
 
+void
+LteSpectrumPhy::SetComponentCarrierId (uint8_t componentCarrierId)
+{
+  m_componentCarrierId = componentCarrierId;
+}
 
 void
 LteSpectrumPhy::AddRsPowerChunkProcessor (Ptr<LteChunkProcessor> p)
