@@ -70,10 +70,15 @@ static const Time UL_SRS_DELAY_FROM_SUBFRAME_START = NanoSeconds (1e6 - 71429);
 // member SAP forwarders
 ////////////////////////////////////////
 
-
+/// UeMemberLteUePhySapProvider class
 class UeMemberLteUePhySapProvider : public LteUePhySapProvider
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param phy the LTE UE Phy
+   */
   UeMemberLteUePhySapProvider (LteUePhy* phy);
 
   // inherited from LtePhySapProvider
@@ -82,7 +87,7 @@ public:
   virtual void SendRachPreamble (uint32_t prachId, uint32_t raRnti);
 
 private:
-  LteUePhy* m_phy;
+  LteUePhy* m_phy; ///< the Phy
 };
 
 UeMemberLteUePhySapProvider::UeMemberLteUePhySapProvider (LteUePhy* phy) : m_phy (phy)
@@ -141,8 +146,8 @@ LteUePhy::LteUePhy ()
 
 LteUePhy::LteUePhy (Ptr<LteSpectrumPhy> dlPhy, Ptr<LteSpectrumPhy> ulPhy)
   : LtePhy (dlPhy, ulPhy),
-    m_p10CqiPeriocity (MilliSeconds (1)),  // ideal behavior
-    m_a30CqiPeriocity (MilliSeconds (1)),  // ideal behavior
+    m_p10CqiPeriodicity (MilliSeconds (1)),  // ideal behavior
+    m_a30CqiPeriodicity (MilliSeconds (1)),  // ideal behavior
     m_uePhySapUser (0),
     m_ueCphySapUser (0),
     m_state (CELL_SEARCH),
@@ -487,7 +492,7 @@ LteUePhy::GenerateCqiRsrpRsrq (const SpectrumValue& sinr)
   if (m_dlConfigured && m_ulConfigured && (m_rnti > 0))
     {
       // check periodic wideband CQI
-      if (Simulator::Now () > m_p10CqiLast + m_p10CqiPeriocity)
+      if (Simulator::Now () > m_p10CqiLast + m_p10CqiPeriodicity)
         {
           Ptr<LteUeNetDevice> thisDevice = GetDevice ()->GetObject<LteUeNetDevice> ();
           Ptr<DlCqiLteControlMessage> msg = CreateDlCqiFeedbackMessage (sinr);
@@ -498,7 +503,7 @@ LteUePhy::GenerateCqiRsrpRsrq (const SpectrumValue& sinr)
           m_p10CqiLast = Simulator::Now ();
         }
       // check aperiodic high-layer configured subband CQI
-      if  (Simulator::Now () > m_a30CqiLast + m_a30CqiPeriocity)
+      if  (Simulator::Now () > m_a30CqiLast + m_a30CqiPeriodicity)
         {
           Ptr<LteUeNetDevice> thisDevice = GetDevice ()->GetObject<LteUeNetDevice> ();
           Ptr<DlCqiLteControlMessage> msg = CreateDlCqiFeedbackMessage (sinr);
@@ -538,9 +543,9 @@ LteUePhy::GenerateCqiRsrpRsrq (const SpectrumValue& sinr)
           rbNum++;
         }
       double avSinr = (rbNum > 0) ? (sum / rbNum) : DBL_MAX;
-      NS_LOG_INFO (this << " cellId " << m_cellId << " rnti " << m_rnti << " RSRP " << rsrp << " SINR " << avSinr);
+      NS_LOG_INFO (this << " cellId " << m_cellId << " rnti " << m_rnti << " RSRP " << rsrp << " SINR " << avSinr << " ComponentCarrierId " << (uint16_t) m_componentCarrierId);
 
-      m_reportCurrentCellRsrpSinrTrace (m_cellId, m_rnti, rsrp, avSinr);
+      m_reportCurrentCellRsrpSinrTrace (m_cellId, m_rnti, rsrp, avSinr, (uint16_t) m_componentCarrierId);
       m_rsrpSinrSampleCounter = 0;
     }
 
@@ -612,7 +617,7 @@ LteUePhy::GenerateMixedCqiReport (const SpectrumValue& sinr)
 
   NS_ASSERT (m_state != CELL_SEARCH);
   NS_ASSERT (m_cellId > 0);
-  
+
   SpectrumValue mixedSinr = (m_rsReceivedPower * m_paLinear);
   if (m_dataInterferencePowerUpdated)
     {
@@ -710,7 +715,7 @@ LteUePhy::CreateDlCqiFeedbackMessage (const SpectrumValue& sinr)
   Ptr<DlCqiLteControlMessage> msg = Create<DlCqiLteControlMessage> ();
   CqiListElement_s dlcqi;
   std::vector<int> cqi;
-  if (Simulator::Now () > m_p10CqiLast + m_p10CqiPeriocity)
+  if (Simulator::Now () > m_p10CqiLast + m_p10CqiPeriodicity)
     {
       cqi = m_amc->CreateCqiFeedbacks (newSinr, m_dlBandwidth);
 
@@ -749,7 +754,7 @@ LteUePhy::CreateDlCqiFeedbackMessage (const SpectrumValue& sinr)
       dlcqi.m_wbPmi = 0; // not yet used
       // dl.cqi.m_sbMeasResult others CQI report modes: not yet implemented
     }
-  else if (Simulator::Now () > m_a30CqiLast + m_a30CqiPeriocity)
+  else if (Simulator::Now () > m_a30CqiLast + m_a30CqiPeriodicity)
     {
       cqi = m_amc->CreateCqiFeedbacks (newSinr, GetRbgSize ());
       int nLayer = TransmissionModesLayers::TxMode2LayerNum (m_transmissionMode);
@@ -817,16 +822,18 @@ LteUePhy::ReportUeMeasurements ()
                          << " RSRP " << avg_rsrp
                          << " (nSamples " << (uint16_t)(*it).second.rsrpNum << ")"
                          << " RSRQ " << avg_rsrq
-                         << " (nSamples " << (uint16_t)(*it).second.rsrqNum << ")");
+                         << " (nSamples " << (uint16_t)(*it).second.rsrqNum << ")"
+                         << " ComponentCarrierID " << (uint16_t)m_componentCarrierId);
 
       LteUeCphySapUser::UeMeasurementsElement newEl;
       newEl.m_cellId = (*it).first;
       newEl.m_rsrp = avg_rsrp;
       newEl.m_rsrq = avg_rsrq;
       ret.m_ueMeasurementsList.push_back (newEl);
+      ret.m_componentCarrierId = m_componentCarrierId;
 
       // report to UE measurements trace
-      m_reportUeMeasurements (m_rnti, (*it).first, avg_rsrp, avg_rsrq, ((*it).first == m_cellId ? 1 : 0));
+      m_reportUeMeasurements (m_rnti, (*it).first, avg_rsrp, avg_rsrq, ((*it).first == m_cellId ? 1 : 0), m_componentCarrierId);
     }
 
   // report to RRC
@@ -864,6 +871,7 @@ LteUePhy::ReceiveLteControlMessageList (std::list<Ptr<LteControlMessage> > msgLi
   NS_LOG_FUNCTION (this);
 
   std::list<Ptr<LteControlMessage> >::iterator it;
+  NS_LOG_DEBUG (this << " I am rnti = " << m_rnti << " and I received msgs " << (uint16_t) msgList.size ());
   for (it = msgList.begin (); it != msgList.end (); it++)
     {
       Ptr<LteControlMessage> msg = (*it);
@@ -948,6 +956,7 @@ LteUePhy::ReceiveLteControlMessageList (std::list<Ptr<LteControlMessage> > msgLi
           params.m_size = dci.m_tbSize;
           params.m_rv = harqInfoList.size ();
           params.m_ndi = dci.m_ndi;
+          params.m_ccId = m_componentCarrierId;
           m_ulPhyTransmission (params);
           // pass the info to the MAC
           m_uePhySapUser->ReceiveLteControlMessage (msg);
@@ -1222,7 +1231,7 @@ LteUePhy::DoReset ()
 } // end of void LteUePhy::DoReset ()
 
 void
-LteUePhy::DoStartCellSearch (uint16_t dlEarfcn)
+LteUePhy::DoStartCellSearch (uint32_t dlEarfcn)
 {
   NS_LOG_FUNCTION (this << dlEarfcn);
   m_dlEarfcn = dlEarfcn;
@@ -1231,7 +1240,7 @@ LteUePhy::DoStartCellSearch (uint16_t dlEarfcn)
 }
 
 void
-LteUePhy::DoSynchronizeWithEnb (uint16_t cellId, uint16_t dlEarfcn)
+LteUePhy::DoSynchronizeWithEnb (uint16_t cellId, uint32_t dlEarfcn)
 {
   NS_LOG_FUNCTION (this << cellId << dlEarfcn);
   m_dlEarfcn = dlEarfcn;
@@ -1293,7 +1302,7 @@ LteUePhy::DoSetDlBandwidth (uint8_t dlBandwidth)
 
 
 void 
-LteUePhy::DoConfigureUplink (uint16_t ulEarfcn, uint8_t ulBandwidth)
+LteUePhy::DoConfigureUplink (uint32_t ulEarfcn, uint8_t ulBandwidth)
 {
   m_ulEarfcn = ulEarfcn;
   m_ulBandwidth = ulBandwidth;
