@@ -51,12 +51,17 @@ class PeerManagementProtocol : public Object
 public:
   PeerManagementProtocol ();
   ~PeerManagementProtocol ();
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId ();
   void DoDispose ();
   /**
    * \brief Install PMP on given mesh point.
+   * \return true if successful
    *
-   * Installing protocol cause installing its interface MAC plugins.
+   * Installing protocol causes installation of its interface MAC plugins.
    *
    * Also MP aggregates all installed protocols, PMP protocol can be accessed
    * via MeshPointDevice::GetObject<PeerManagementProtocol>();
@@ -98,8 +103,7 @@ public:
    * to peer address when only one interface)
    * \param aid is association ID, which peer has assigned to us
    * \param peerManagementElement is peer link management element
-   * \param meshConfig is mesh configuration element taken from the peer
-   * management frame
+   * \param meshConfig is mesh configuration element taken from the peer management frame
    */
   void ReceivePeerLinkFrame (
     uint32_t interface,
@@ -137,21 +141,51 @@ public:
   std::vector < Ptr<PeerLink> > GetPeerLinks () const;
   /// Get list of active peers of my given interface
   std::vector<Mac48Address> GetPeers (uint32_t interface) const;
-  /// Get mesh point address.
-  /// \todo this used by plugins only. Now MAC plugins can ask MP addrress directly from main MAC
+  /**
+   * Get mesh point address. \todo this used by plugins only. Now MAC plugins can ask MP address directly from main MAC
+   *
+   * \return Mac48Address
+   */
   Mac48Address GetAddress ();
+  /**
+   * Get number of links
+   * \returns the number of links
+   */
   uint8_t GetNumberOfLinks ();
+  /**
+   * Set mesh ID to a string value 
+   * \param s the mesh ID string value
+   */
   void SetMeshId (std::string s);
+  /**
+   * Get mesh ID information element
+   * \returns the mesh ID information element
+   */
   Ptr<IeMeshId> GetMeshId () const;
-  /// Enable or disable beacon collision avoidance
+  /**
+   * Enable or disable beacon collision avoidance
+   * \param enable true to enable beacon collision avoidance
+   */
   void SetBeaconCollisionAvoidance (bool enable);
+  /**
+   * Get beacon collision avoidance
+   * \returns the beacon collision avoidance flag
+   */
   bool GetBeaconCollisionAvoidance () const;
-  /// Notify about beacon send event, needed to schedule BCA
+  /**
+   * Notify about beacon send event, needed to schedule BCA
+   * \param interface the interface to use
+   * \param beaconInterval the beacon interval
+   */
   void NotifyBeaconSent (uint32_t interface, Time beaconInterval);
   // \}
   
-  ///\brief: Report statistics
-  void Report (std::ostream &) const;
+  /**
+   * \brief: Report statistics
+   * \param os the output stream
+   */
+  void Report (std::ostream & os) const;
+  /// Reset statistics function
   void ResetStats ();
   /**
    * Assign a fixed random variable stream number to the random variables
@@ -180,9 +214,9 @@ private:
   /// Keeps information about beacon of peer station: beacon interval, association ID, last time we have received a beacon
   struct BeaconInfo
   {
-    uint16_t aid; //Assoc ID
-    Time referenceTbtt; //When one of my station's beacons was put into a beacon queue;
-    Time beaconInterval; //Beacon interval of my station;
+    uint16_t aid; ///< Assoc ID
+    Time referenceTbtt; ///< When one of my station's beacons was put into a beacon queue;
+    Time beaconInterval; ///< Beacon interval of my station;
   };
   /// We keep a vector of pointers to PeerLink class. This vector
   /// keeps all peer links at a given interface.
@@ -197,9 +231,24 @@ private:
   typedef std::map<uint32_t, Ptr<PeerManagementProtocolMac> > PeerManagementProtocolMacMap;
 
 private:
-  PeerManagementProtocol& operator= (const PeerManagementProtocol &);
+  /**
+   * assignment operator
+   *
+   * \param peer the value to assign
+   * \returns the assigned value
+   */
+  PeerManagementProtocol& operator= (const PeerManagementProtocol & peer);
+  /// type conversion operator
   PeerManagementProtocol (const PeerManagementProtocol &);
 
+  /**
+   * Initiate link functon
+   *
+   * \param interface the interface to use
+   * \param peerAddress the peer address
+   * \param peerMeshPointAddress the peer mesh point address
+   * \returns the peer link
+   */
   Ptr<PeerLink> InitiateLink (
     uint32_t interface,
     Mac48Address peerAddress,
@@ -207,38 +256,44 @@ private:
     );
   /**
    * \name External peer-chooser
-   * \{
+   * \param interface the interface to use
+   * \param peerAddress the peer address
+   * \returns true is should send an open
    */
   bool ShouldSendOpen (uint32_t interface, Mac48Address peerAddress);
+  /**
+   * \name External peer-chooser
+   */
   bool ShouldAcceptOpen (uint32_t interface, Mac48Address peerAddress, PmpReasonCode & reasonCode);
   /**
-   * \}
    * \brief Indicates changes in peer links
    */
   void PeerLinkStatus (uint32_t interface, Mac48Address peerAddress, Mac48Address peerMeshPointAddres, PeerLink::PeerState ostate, PeerLink::PeerState nstate);
   ///\brief BCA
   void CheckBeaconCollisions (uint32_t interface);
+  /// Shift own beacon function
   void ShiftOwnBeacon (uint32_t interface);
   /**
    * \name Time<-->TU converters:
-   * \{
    */
   Time TuToTime (int x);
+  /**
+   * \name Time<-->TU converters:
+   */
   int TimeToTu (Time x);
-  // \}
 
   /// Aux. method to register open links
   void NotifyLinkOpen (Mac48Address peerMp, Mac48Address peerIface, Mac48Address myIface, uint32_t interface);
   /// Aux. method to register closed links
   void NotifyLinkClose (Mac48Address peerMp, Mac48Address peerIface, Mac48Address myIface, uint32_t interface);
 private:
-  PeerManagementProtocolMacMap m_plugins;
-  Mac48Address m_address;
-  Ptr<IeMeshId> m_meshId;
+  PeerManagementProtocolMacMap m_plugins; ///< plugins
+  Mac48Address m_address; ///< addresss
+  Ptr<IeMeshId> m_meshId; ///< mesh ID
 
-  uint16_t m_lastAssocId;
-  uint16_t m_lastLocalLinkId;
-  uint8_t m_maxNumberOfPeerLinks;
+  uint16_t m_lastAssocId; ///< last associated ID
+  uint16_t m_lastLocalLinkId; ///< last local link ID
+  uint8_t m_maxNumberOfPeerLinks; ///< maimum number of peer links
   /// Flag which enables BCA
   bool m_enableBca;
   /// Beacon can be shifted at [-m_maxBeaconShift; +m_maxBeaconShift] TUs
@@ -250,12 +305,8 @@ private:
 
   /**
    * \name Peer Links
-   * \{
    */
   PeerLinksMap m_peerLinks;
-  /**
-   * \}
-   */
   /**
    * \brief Callback to notify about peer link changes:
    * Mac48Address is peer address of mesh point,
@@ -272,18 +323,26 @@ private:
   /// LinkClose trace source
   LinkEventCallback m_linkCloseTraceSrc;
 
-  ///\name Statistics:
-  // \{
+  /// Statistics structure
   struct Statistics {
-    uint16_t linksTotal;
-    uint16_t linksOpened;
-    uint16_t linksClosed;
+    uint16_t linksTotal; ///< total links
+    uint16_t linksOpened; ///< opened links
+    uint16_t linksClosed; ///< links closed
 
+    /**
+     * Constructor
+     *
+     * \param t 
+     */
     Statistics (uint16_t t = 0);
+    /**
+     * Print function
+     * \param os the output stream to print to
+     */
     void Print (std::ostream & os) const;
   };
-  struct Statistics m_stats;
-  // \}
+  struct Statistics m_stats; ///< statistics
+
   /// Add randomness to beacon shift
   Ptr<UniformRandomVariable> m_beaconShift;
 };
