@@ -115,7 +115,7 @@ DistributedSimulatorImpl::DistributedSimulatorImpl ()
   // before ::Run is entered, the m_currentUid will be zero
   m_currentUid = 0;
   m_currentTs = 0;
-  m_currentContext = 0xffffffff;
+  m_currentContext = Simulator::NO_CONTEXT;
   m_unscheduledEvents = 0;
   m_events = 0;
 }
