@@ -43,8 +43,7 @@ Socket::GetTypeId (void)
 }
 
 Socket::Socket (void)
-  : m_manualIpTos (false),
-    m_manualIpTtl (false),
+  : m_manualIpTtl (false),
     m_ipRecvTos (false),
     m_ipRecvTtl (false),
     m_manualIpv6Tclass (false),
@@ -56,6 +55,7 @@ Socket::Socket (void)
   m_boundnetdevice = 0;
   m_recvPktInfo = false;
 
+  m_priority = 0;
   m_ipTos = 0;
   m_ipTtl = 0;
   m_ipv6Tclass = 0;
@@ -367,12 +367,6 @@ bool Socket::IsRecvPktInfo () const
   return m_recvPktInfo;
 }
 
-bool
-Socket::IsManualIpTos (void) const
-{
-  return m_manualIpTos;
-}
-
 bool
 Socket::IsManualIpv6Tclass (void) const
 {
@@ -391,13 +385,74 @@ Socket::IsManualIpv6HopLimit (void) const
   return m_manualIpv6HopLimit;
 }
 
+void
+Socket::SetPriority (uint8_t priority)
+{
+  if (priority <= 6)
+    {
+      m_priority = priority;
+    }
+  else
+    {
+      NS_LOG_ERROR ("Cannot set a priority higher than 6");
+    }
+}
+
+uint8_t
+Socket::GetPriority (void) const
+{
+  return m_priority;
+}
+
+uint8_t
+Socket::IpTos2Priority (uint8_t ipTos)
+{
+  uint8_t prio = NS3_PRIO_BESTEFFORT;
+  ipTos &= 0x1e;
+  switch (ipTos >> 1)
+    {
+    case 0:
+    case 1:
+    case 2:
+    case 3:
+      prio = NS3_PRIO_BESTEFFORT;
+      break;
+    case 4:
+    case 5:
+    case 6:
+    case 7:
+      prio = NS3_PRIO_BULK;
+      break;
+    case 8:
+    case 9:
+    case 10:
+    case 11:
+      prio = NS3_PRIO_INTERACTIVE;
+      break;
+    case 12:
+    case 13:
+    case 14:
+    case 15:
+      prio = NS3_PRIO_INTERACTIVE_BULK;
+      break;
+    }
+  return prio;
+}
+
 void
 Socket::SetIpTos (uint8_t tos)
 {
   Address address;
   GetSockName (address);
-  m_manualIpTos = true;
+  if (GetSocketType () == NS3_SOCK_STREAM)
+    {
+      // preserve the least two significant bits of the current TOS
+      // value, which are used for ECN
+      tos &= 0xfc;
+      tos |= m_ipTos & 0x3;
+    }
   m_ipTos = tos;
+  m_priority = IpTos2Priority (tos);
 }
 
 uint8_t
@@ -546,67 +601,6 @@ Socket::Ipv6LeaveGroup (void)
  *           Socket Tags
  ***************************************************************/
 
-SocketAddressTag::SocketAddressTag ()
-{
-  NS_LOG_FUNCTION (this);
-}
-
-void 
-SocketAddressTag::SetAddress (Address addr)
-{
-  NS_LOG_FUNCTION (this << addr);
-  m_address = addr;
-}
-
-Address 
-SocketAddressTag::GetAddress (void) const
-{
-  NS_LOG_FUNCTION (this);
-  return m_address;
-}
-
-NS_OBJECT_ENSURE_REGISTERED (SocketAddressTag);
-
-TypeId
-SocketAddressTag::GetTypeId (void)
-{
-  static TypeId tid = TypeId ("ns3::SocketAddressTag")
-    .SetParent<Tag> ()
-    .SetGroupName("Network")
-    .AddConstructor<SocketAddressTag> ()
-  ;
-  return tid;
-}
-TypeId
-SocketAddressTag::GetInstanceTypeId (void) const
-{
-  return GetTypeId ();
-}
-uint32_t
-SocketAddressTag::GetSerializedSize (void) const
-{
-  NS_LOG_FUNCTION (this);
-  return m_address.GetSerializedSize ();
-}
-void
-SocketAddressTag::Serialize (TagBuffer i) const
-{
-  NS_LOG_FUNCTION (this << &i);
-  m_address.Serialize (i);
-}
-void
-SocketAddressTag::Deserialize (TagBuffer i)
-{
-  NS_LOG_FUNCTION (this << &i);
-  m_address.Deserialize (i);
-}
-void
-SocketAddressTag::Print (std::ostream &os) const
-{
-  NS_LOG_FUNCTION (this << &os);
-  os << "address=" << m_address;
-}
-
 SocketIpTtlTag::SocketIpTtlTag ()
 {
   NS_LOG_FUNCTION (this);
@@ -846,6 +840,64 @@ SocketIpTosTag::Print (std::ostream &os) const
 }
 
 
+SocketPriorityTag::SocketPriorityTag ()
+{
+}
+
+void
+SocketPriorityTag::SetPriority (uint8_t priority)
+{
+  m_priority = priority;
+}
+
+uint8_t
+SocketPriorityTag::GetPriority (void) const
+{
+  return m_priority;
+}
+
+TypeId
+SocketPriorityTag::GetTypeId (void)
+{
+  static TypeId tid = TypeId ("ns3::SocketPriorityTag")
+    .SetParent<Tag> ()
+    .SetGroupName("Network")
+    .AddConstructor<SocketPriorityTag> ()
+    ;
+  return tid;
+}
+
+TypeId
+SocketPriorityTag::GetInstanceTypeId (void) const
+{
+  return GetTypeId ();
+}
+
+uint32_t
+SocketPriorityTag::GetSerializedSize (void) const
+{
+  return sizeof (uint8_t);
+}
+
+void
+SocketPriorityTag::Serialize (TagBuffer i) const
+{
+  i.WriteU8 (m_priority);
+}
+
+void
+SocketPriorityTag::Deserialize (TagBuffer i)
+{
+  m_priority = i.ReadU8();
+}
+
+void
+SocketPriorityTag::Print (std::ostream &os) const
+{
+  os << "SO_PRIORITY = " << m_priority;
+}
+
+
 SocketIpv6TclassTag::SocketIpv6TclassTag ()
 {
 }
