@@ -643,7 +643,6 @@ uint32_t
 DeviceNameTag::GetSerializedSize (void) const
 {
   uint32_t s = 1 + m_deviceName.size();  // +1 for name length field
-  s = std::min (s, (uint32_t)PacketTagList::TagData::MAX_SIZE);
   return s;
 }
 void
@@ -652,8 +651,6 @@ DeviceNameTag::Serialize (TagBuffer i) const
   const char *n = m_deviceName.c_str();
   uint8_t l = (uint8_t) m_deviceName.size ();
 
-  l = std::min ((uint32_t)l, (uint32_t)PacketTagList::TagData::MAX_SIZE - 1);
-
   i.WriteU8 (l);
   i.Write ( (uint8_t*) n , (uint32_t) l);
 }
