@@ -25,6 +25,7 @@
 #include "ns3/trace-source-accessor.h"
 #include "ns3/uinteger.h"
 #include "ns3/pointer.h"
+#include "ns3/net-device-queue-interface.h"
 #include "point-to-point-net-device.h"
 #include "point-to-point-channel.h"
 #include "ppp-header.h"
@@ -76,7 +77,7 @@ PointToPointNetDevice::GetTypeId (void)
                    "A queue to use as the transmit queue in the device.",
                    PointerValue (),
                    MakePointerAccessor (&PointToPointNetDevice::m_queue),
-                   MakePointerChecker<Queue> ())
+                   MakePointerChecker<Queue<Packet> > ())
 
     //
     // Trace sources at the "top" of the net device, where packets transition
@@ -206,6 +207,23 @@ PointToPointNetDevice::ProcessHeader (Ptr<Packet> p, uint16_t& param)
   return true;
 }
 
+void
+PointToPointNetDevice::DoInitialize (void)
+{
+  if (m_queueInterface)
+    {
+      NS_ASSERT_MSG (m_queue != 0, "A Queue object has not been attached to the device");
+
+      // connect the traced callbacks of m_queue to the static methods provided by
+      // the NetDeviceQueue class to support flow control and dynamic queue limits.
+      // This could not be done in NotifyNewAggregate because at that time we are
+      // not guaranteed that a queue has been attached to the netdevice
+      m_queueInterface->ConnectQueueTraces (m_queue, 0);
+    }
+
+  NetDevice::DoInitialize ();
+}
+
 void
 PointToPointNetDevice::NotifyNewAggregate (void)
 {
@@ -299,53 +317,19 @@ PointToPointNetDevice::TransmitComplete (void)
   m_phyTxEndTrace (m_currentPkt);
   m_currentPkt = 0;
 
-  Ptr<NetDeviceQueue> txq;
-  if (m_queueInterface)
-  {
-    txq = m_queueInterface->GetTxQueue (0);
-  }
-
-  Ptr<QueueItem> item = m_queue->Dequeue ();
-  if (item == 0)
+  Ptr<Packet> p = m_queue->Dequeue ();
+  if (p == 0)
     {
       NS_LOG_LOGIC ("No pending packets in device queue after tx complete");
-      if (txq)
-      {
-        NS_LOG_DEBUG ("The device queue is being woken up (" << m_queue->GetNPackets () <<
-                      " packets and " << m_queue->GetNBytes () << " bytes inside)");
-        txq->Wake ();
-      }
       return;
     }
 
   //
   // Got another packet off of the queue, so start the transmit process again.
-  // If the queue was stopped, start it again if there is room for another packet.
-  // Note that we cannot wake the upper layers because otherwise a packet is sent
-  // to the device while the machine state is busy, thus causing the assert in
-  // TransmitStart to fail.
   //
-  if (txq && txq->IsStopped ())
-    {
-      if ((m_queue->GetMode () == Queue::QUEUE_MODE_PACKETS &&
-           m_queue->GetNPackets () < m_queue->GetMaxPackets ()) ||
-          (m_queue->GetMode () == Queue::QUEUE_MODE_BYTES &&
-           m_queue->GetNBytes () + m_mtu <= m_queue->GetMaxBytes ()))
-        {
-          NS_LOG_DEBUG ("The device queue is being started (" << m_queue->GetNPackets () <<
-                        " packets and " << m_queue->GetNBytes () << " bytes inside)");
-          txq->Start ();
-        }
-    }
-  Ptr<Packet> p = item->GetPacket ();
   m_snifferTrace (p);
   m_promiscSnifferTrace (p);
   TransmitStart (p);
-  if (txq)
-    {
-      // Inform BQL
-      txq->NotifyTransmittedBytes (m_currentPkt->GetSize ());
-    }
 }
 
 bool
@@ -367,7 +351,7 @@ PointToPointNetDevice::Attach (Ptr<PointToPointChannel> ch)
 }
 
 void
-PointToPointNetDevice::SetQueue (Ptr<Queue> q)
+PointToPointNetDevice::SetQueue (Ptr<Queue<Packet> > q)
 {
   NS_LOG_FUNCTION (this << q);
   m_queue = q;
@@ -430,7 +414,7 @@ PointToPointNetDevice::Receive (Ptr<Packet> packet)
     }
 }
 
-Ptr<Queue>
+Ptr<Queue<Packet> >
 PointToPointNetDevice::GetQueue (void) const
 { 
   NS_LOG_FUNCTION (this);
@@ -561,14 +545,6 @@ PointToPointNetDevice::Send (
   const Address &dest, 
   uint16_t protocolNumber)
 {
-  Ptr<NetDeviceQueue> txq;
-  if (m_queueInterface)
-  {
-    txq = m_queueInterface->GetTxQueue (0);
-  }
-
-  NS_ASSERT_MSG (!txq || !txq->IsStopped (), "Send should not be called when the device is stopped");
-
   NS_LOG_FUNCTION (this << packet << dest << protocolNumber);
   NS_LOG_LOGIC ("p=" << packet << ", dest=" << &dest);
   NS_LOG_LOGIC ("UID is " << packet->GetUid ());
@@ -594,71 +570,25 @@ PointToPointNetDevice::Send (
   //
   // We should enqueue and dequeue the packet to hit the tracing hooks.
   //
-  if (m_queue->Enqueue (Create<QueueItem> (packet)))
+  if (m_queue->Enqueue (packet))
     {
-      // Inform BQL
-      if (txq)
-        {
-          txq->NotifyQueuedBytes (packet->GetSize ());
-        }
       //
       // If the channel is ready for transition we send the packet right now
       // 
       if (m_txMachineState == READY)
         {
-          packet = m_queue->Dequeue ()->GetPacket ();
-          // We have enqueued a packet and dequeued a (possibly different) packet. We
-          // need to check if there is still room for another packet only if the queue
-          // is in byte mode (the enqueued packet might be larger than the dequeued
-          // packet, thus leaving no room for another packet)
-          if (txq)
-            {
-              if (m_queue->GetMode () == Queue::QUEUE_MODE_BYTES &&
-                  m_queue->GetNBytes () + m_mtu > m_queue->GetMaxBytes ())
-                {
-                  NS_LOG_DEBUG ("The device queue is being stopped (" << m_queue->GetNPackets () <<
-                                " packets and " << m_queue->GetNBytes () << " bytes inside)");
-                  txq->Stop ();
-                }
-            }
+          packet = m_queue->Dequeue ();
           m_snifferTrace (packet);
           m_promiscSnifferTrace (packet);
           bool ret = TransmitStart (packet);
-          if (txq)
-            {
-              // Inform BQL
-              txq->NotifyTransmittedBytes (m_currentPkt->GetSize ());
-            }
           return ret;
         }
-      // We have enqueued a packet but we have not dequeued any packet. Thus, we
-      // need to check whether the queue is able to store another packet. If not,
-      // we stop the queue
-      if (txq)
-        {
-          if ((m_queue->GetMode () == Queue::QUEUE_MODE_PACKETS &&
-               m_queue->GetNPackets () >= m_queue->GetMaxPackets ()) ||
-              (m_queue->GetMode () == Queue::QUEUE_MODE_BYTES &&
-               m_queue->GetNBytes () + m_mtu > m_queue->GetMaxBytes ()))
-            {
-              NS_LOG_DEBUG ("The device queue is being stopped (" << m_queue->GetNPackets () <<
-                            " packets and " << m_queue->GetNBytes () << " bytes inside)");
-              txq->Stop ();
-            }
-        }
       return true;
     }
 
-  // Enqueue may fail (overflow). This should not happen if the traffic control
-  // module has been installed. Anyway, stop the tx queue, so that the upper layers
-  // do not send packets until there is room in the queue again.
+  // Enqueue may fail (overflow)
+
   m_macTxDropTrace (packet);
-  if (txq)
-  {
-    NS_LOG_ERROR ("BUG! Device queue full when the queue is not stopped! (" << m_queue->GetNPackets () <<
-                  " packets and " << m_queue->GetNBytes () << " bytes inside)");
-    txq->Stop ();
-  }
   return false;
 }
 
