@@ -240,7 +240,7 @@ main (int argc, char *argv[])
 
   // RED params
   NS_LOG_INFO ("Set RED params");
-  Config::SetDefault ("ns3::RedQueueDisc::Mode", StringValue ("QUEUE_MODE_PACKETS"));
+  Config::SetDefault ("ns3::RedQueueDisc::Mode", StringValue ("QUEUE_DISC_MODE_PACKETS"));
   Config::SetDefault ("ns3::RedQueueDisc::MeanPktSize", UintegerValue (meanPktSize));
   Config::SetDefault ("ns3::RedQueueDisc::Wait", BooleanValue (true));
   Config::SetDefault ("ns3::RedQueueDisc::Gentle", BooleanValue (true));
@@ -479,48 +479,28 @@ main (int argc, char *argv[])
   Simulator::Stop (Seconds (sink_stop_time));
   Simulator::Run ();
 
-  RedQueueDisc::Stats st = StaticCast<RedQueueDisc> (queueDiscs.Get (0))->GetStats ();
+  QueueDisc::Stats st = queueDiscs.Get (0)->GetStats ();
+
+  if (st.GetNDroppedPackets (RedQueueDisc::UNFORCED_DROP) == 0)
+    {
+      std::cout << "There should be some unforced drops" << std::endl;
+      exit (1);
+    }
 
   if (aredTest == 1 || aredTest == 2 || aredTest == 13)
     {
-      if (st.unforcedDrop > st.forcedDrop)
-        {
-          std::cout << "Drops due to prob mark should be less than the drops due to hard mark" << std::endl;
-          exit (-1);
-        }
-
-      if (st.qLimDrop == 0)
+      if (st.GetNDroppedPackets (QueueDisc::INTERNAL_QUEUE_DROP) == 0)
         {
           std::cout << "There should be some drops due to queue full" << std::endl;
-          exit (-1);
+          exit (1);
         }
     }
-  else if (aredTest == 6 || aredTest == 7 || aredTest == 8 || aredTest == 9 || aredTest == 10 || aredTest == 14 || aredTest ==15)
+  else
     {
-      if (st.unforcedDrop > st.forcedDrop)
-        {
-          std::cout << "Drops due to prob mark should be less than the drops due to hard mark" << std::endl;
-          exit (-1);
-        }
-
-      if (st.qLimDrop != 0)
+      if (st.GetNDroppedPackets (QueueDisc::INTERNAL_QUEUE_DROP) != 0)
         {
           std::cout << "There should be zero drops due to queue full" << std::endl;
-          exit (-1);
-        }
-    }
-  else if (aredTest == 12)
-    {
-      if (st.unforcedDrop < st.forcedDrop)
-        {
-          std::cout << "Drops due to prob mark should be more than the drops due to hard mark" << std::endl;
-          exit (-1);
-        }
-
-      if (st.qLimDrop != 0)
-        {
-          std::cout << "There should be zero drops due to queue full" << std::endl;
-          exit (-1);
+          exit (1);
         }
     }
 
@@ -535,9 +515,7 @@ main (int argc, char *argv[])
   if (printAredStats)
     {
       std::cout << "*** ARED stats from Node 2 queue ***" << std::endl;
-      std::cout << "\t " << st.unforcedDrop << " drops due to prob mark" << std::endl;
-      std::cout << "\t " << st.forcedDrop << " drops due to hard mark" << std::endl;
-      std::cout << "\t " << st.qLimDrop << " drops due to queue full" << std::endl;
+      std::cout << st << std::endl;
     }
 
   Simulator::Destroy ();
