@@ -21,9 +21,9 @@
  */
 
 #include "ns3/log.h"
-#include "ns3/pointer.h"
 #include "ns3/object-factory.h"
-#include "ns3/drop-tail-queue.h"
+#include "ns3/queue.h"
+#include "ns3/net-device-queue-interface.h"
 #include "ns3/socket.h"
 #include "pfifo-fast-queue-disc.h"
 
@@ -65,10 +65,10 @@ PfifoFastQueueDisc::DoEnqueue (Ptr<QueueDiscItem> item)
 {
   NS_LOG_FUNCTION (this << item);
 
-  if (GetNPackets () > m_limit)
+  if (GetNPackets () >= m_limit)
     {
       NS_LOG_LOGIC ("Queue disc limit exceeded -- dropping packet");
-      Drop (item);
+      DropBeforeEnqueue (item, LIMIT_EXCEEDED_DROP);
       return false;
     }
 
@@ -83,8 +83,8 @@ PfifoFastQueueDisc::DoEnqueue (Ptr<QueueDiscItem> item)
 
   bool retval = GetInternalQueue (band)->Enqueue (item);
 
-  // If Queue::Enqueue fails, QueueDisc::Drop is called by the internal queue
-  // because QueueDisc::AddInternalQueue sets the drop callback
+  // If Queue::Enqueue fails, QueueDisc::DropBeforeEnqueue is called by the
+  // internal queue because QueueDisc::AddInternalQueue sets the trace callback
 
   NS_LOG_LOGIC ("Number packets band " << band << ": " << GetInternalQueue (band)->GetNPackets ());
 
@@ -100,7 +100,7 @@ PfifoFastQueueDisc::DoDequeue (void)
 
   for (uint32_t i = 0; i < GetNInternalQueues (); i++)
     {
-      if ((item = StaticCast<QueueDiscItem> (GetInternalQueue (i)->Dequeue ())) != 0)
+      if ((item = GetInternalQueue (i)->Dequeue ()) != 0)
         {
           NS_LOG_LOGIC ("Popped from band " << i << ": " << item);
           NS_LOG_LOGIC ("Number packets band " << i << ": " << GetInternalQueue (i)->GetNPackets ());
@@ -121,10 +121,12 @@ PfifoFastQueueDisc::DoPeek (void) const
 
   for (uint32_t i = 0; i < GetNInternalQueues (); i++)
     {
-      item = StaticCast<const QueueDiscItem> (GetInternalQueue (i)->Peek ());
-      NS_LOG_LOGIC ("Peeked from band " << i << ": " << item);
-      NS_LOG_LOGIC ("Number packets band " << i << ": " << GetInternalQueue (i)->GetNPackets ());
-      return item;
+      if ((item = GetInternalQueue (i)->Peek ()) != 0)
+        {
+          NS_LOG_LOGIC ("Peeked from band " << i << ": " << item);
+          NS_LOG_LOGIC ("Number packets band " << i << ": " << GetInternalQueue (i)->GetNPackets ());
+          return item;
+        }
     }
 
   NS_LOG_LOGIC ("Queue empty");
@@ -151,12 +153,12 @@ PfifoFastQueueDisc::CheckConfig (void)
     {
       // create 3 DropTail queues with m_limit packets each
       ObjectFactory factory;
-      factory.SetTypeId ("ns3::DropTailQueue");
-      factory.Set ("Mode", EnumValue (Queue::QUEUE_MODE_PACKETS));
+      factory.SetTypeId ("ns3::DropTailQueue<QueueDiscItem>");
+      factory.Set ("Mode", EnumValue (QueueBase::QUEUE_MODE_PACKETS));
       factory.Set ("MaxPackets", UintegerValue (m_limit));
-      AddInternalQueue (factory.Create<Queue> ());
-      AddInternalQueue (factory.Create<Queue> ());
-      AddInternalQueue (factory.Create<Queue> ());
+      AddInternalQueue (factory.Create<InternalQueue> ());
+      AddInternalQueue (factory.Create<InternalQueue> ());
+      AddInternalQueue (factory.Create<InternalQueue> ());
     }
 
   if (GetNInternalQueues () != 3)
@@ -165,9 +167,9 @@ PfifoFastQueueDisc::CheckConfig (void)
       return false;
     }
 
-  if (GetInternalQueue (0)-> GetMode () != Queue::QUEUE_MODE_PACKETS ||
-      GetInternalQueue (1)-> GetMode () != Queue::QUEUE_MODE_PACKETS ||
-      GetInternalQueue (2)-> GetMode () != Queue::QUEUE_MODE_PACKETS)
+  if (GetInternalQueue (0)-> GetMode () != QueueBase::QUEUE_MODE_PACKETS ||
+      GetInternalQueue (1)-> GetMode () != QueueBase::QUEUE_MODE_PACKETS ||
+      GetInternalQueue (2)-> GetMode () != QueueBase::QUEUE_MODE_PACKETS)
     {
       NS_LOG_ERROR ("PfifoFastQueueDisc needs 3 internal queues operating in packet mode");
       return false;
