@@ -22,6 +22,7 @@
 
 #include "ns3/test.h"
 #include "ns3/codel-queue-disc.h"
+#include "ns3/packet.h"
 #include "ns3/uinteger.h"
 #include "ns3/string.h"
 #include "ns3/double.h"
@@ -53,16 +54,38 @@ static uint32_t _reciprocal_scale (uint32_t val, uint32_t ep_ro)
 }
 // End Linux borrow
 
-
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Codel Queue Disc Test Item
+ */
 class CodelQueueDiscTestItem : public QueueDiscItem {
 public:
+  /**
+   * Constructor
+   *
+   * \param p packet
+   * \param addr address
+   * \param protocol
+   */
   CodelQueueDiscTestItem (Ptr<Packet> p, const Address & addr, uint16_t protocol);
   virtual ~CodelQueueDiscTestItem ();
   virtual void AddHeader (void);
+  virtual bool Mark(void);
 
 private:
   CodelQueueDiscTestItem ();
+  /**
+   * \brief Copy constructor
+   * Disable default implementation to avoid misuse
+   */
   CodelQueueDiscTestItem (const CodelQueueDiscTestItem &);
+  /**
+   * \brief Assignment operator
+   * \return this object
+   * Disable default implementation to avoid misuse
+   */
   CodelQueueDiscTestItem &operator = (const CodelQueueDiscTestItem &);
 };
 
@@ -80,20 +103,42 @@ CodelQueueDiscTestItem::AddHeader (void)
 {
 }
 
-// Test 1: simple enqueue/dequeue with no drops
+bool
+CodelQueueDiscTestItem::Mark (void)
+{
+  return false;
+}
+
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Test 1: simple enqueue/dequeue with no drops
+ */
 class CoDelQueueDiscBasicEnqueueDequeue : public TestCase
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mode the mode
+   */
   CoDelQueueDiscBasicEnqueueDequeue (std::string mode);
   virtual void DoRun (void);
 
+  /**
+   * Queue test size function
+   * \param queue the queue disc
+   * \param size the size
+   * \param error the error string
+   */
   void QueueTestSize (Ptr<CoDelQueueDisc> queue, uint32_t size, std::string error)
   {
-    if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+    if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNBytes (), size, error);
       }
-    else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+    else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNPackets (), size, error);
       }
@@ -102,7 +147,7 @@ public:
   }
 
 private:
-  StringValue m_mode;
+  StringValue m_mode; ///< mode
 };
 
 CoDelQueueDiscBasicEnqueueDequeue::CoDelQueueDiscBasicEnqueueDequeue (std::string mode)
@@ -134,11 +179,11 @@ CoDelQueueDiscBasicEnqueueDequeue::DoRun (void)
   NS_TEST_EXPECT_MSG_EQ (queue->SetAttributeFailSafe ("Target", StringValue ("4ms")), true,
                          "Verify that we can actually set the attribute Target");
 
-  if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+  if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
     {
       modeSize = pktSize;
     }
-  else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+  else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
     {
       modeSize = 1;
     }
@@ -166,7 +211,8 @@ CoDelQueueDiscBasicEnqueueDequeue::DoRun (void)
   queue->Enqueue (Create<CodelQueueDiscTestItem> (p6, dest, 0));
   QueueTestSize (queue, 6 * modeSize, "There should be six packets in queue");
 
-  NS_TEST_EXPECT_MSG_EQ (queue->GetDropOverLimit (), 0, "There should be no packets being dropped due to full queue");
+  NS_TEST_EXPECT_MSG_EQ (queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::OVERLIMIT_DROP),
+                         0, "There should be no packets being dropped due to full queue");
 
   Ptr<QueueDiscItem> item;
 
@@ -203,23 +249,40 @@ CoDelQueueDiscBasicEnqueueDequeue::DoRun (void)
   item = queue->Dequeue ();
   NS_TEST_EXPECT_MSG_EQ ((item == 0), true, "There are really no packets in queue");
 
-  NS_TEST_EXPECT_MSG_EQ (queue->GetDropCount (), 0, "There should be no packet drops according to CoDel algorithm");
+  NS_TEST_EXPECT_MSG_EQ (queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP), 0,
+                         "There should be no packet drops according to CoDel algorithm");
 }
 
-// Test 2: enqueue with drops due to queue overflow
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Test 2: enqueue with drops due to queue overflow
+ */
 class CoDelQueueDiscBasicOverflow : public TestCase
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mode the mode
+   */
   CoDelQueueDiscBasicOverflow (std::string mode);
   virtual void DoRun (void);
 
+  /**
+   * Queue test size function
+   * \param queue the queue disc
+   * \param size the size
+   * \param error the error string
+   */
   void QueueTestSize (Ptr<CoDelQueueDisc> queue, uint32_t size, std::string error)
   {
-    if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+    if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNBytes (), size, error);
       }
-    else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+    else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNPackets (), size, error);
       }
@@ -228,8 +291,14 @@ public:
   }
 
 private:
+  /**
+   * Enqueue function
+   * \param queue the queue disc
+   * \param size the size
+   * \param nPkt the number of packets
+   */
   void Enqueue (Ptr<CoDelQueueDisc> queue, uint32_t size, uint32_t nPkt);
-  StringValue m_mode;
+  StringValue m_mode; ///< mode
 };
 
 CoDelQueueDiscBasicOverflow::CoDelQueueDiscBasicOverflow (std::string mode)
@@ -250,11 +319,11 @@ CoDelQueueDiscBasicOverflow::DoRun (void)
 
   Address dest;
 
-  if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+  if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
     {
       modeSize = pktSize;
     }
-  else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+  else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
     {
       modeSize = 1;
     }
@@ -279,7 +348,8 @@ CoDelQueueDiscBasicOverflow::DoRun (void)
   queue->Enqueue (Create<CodelQueueDiscTestItem> (p3, dest, 0));
 
   QueueTestSize (queue, 500 * modeSize, "There should be 500 packets in queue");
-  NS_TEST_EXPECT_MSG_EQ (queue->GetDropOverLimit (), 3, "There should be three packets being dropped due to full queue");
+  NS_TEST_EXPECT_MSG_EQ (queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::OVERLIMIT_DROP),
+                         3, "There should be three packets being dropped due to full queue");
 }
 
 void
@@ -292,8 +362,12 @@ CoDelQueueDiscBasicOverflow::Enqueue (Ptr<CoDelQueueDisc> queue, uint32_t size,
     }
 }
 
-// Test 3: NewtonStep unit test
-// test against explicit port of Linux implementation
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Test 3: NewtonStep unit test - test against explicit port of Linux implementation
+ */
 class CoDelQueueDiscNewtonStepTest : public TestCase
 {
 public:
@@ -332,13 +406,23 @@ CoDelQueueDiscNewtonStepTest::DoRun (void)
                          "ns-3 NewtonStep() fails to match Linux equivalent");
 }
 
-// Test 4: ControlLaw unit test
-// test against explicit port of Linux implementation
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Test 4: ControlLaw unit test - test against explicit port of Linux implementation
+ */
 class CoDelQueueDiscControlLawTest : public TestCase
 {
 public:
   CoDelQueueDiscControlLawTest ();
   virtual void DoRun (void);
+  /**
+   * Codel control law function
+   * \param queue the queue disc
+   * \param t
+   * \returns the codel control law
+   */
   uint32_t _codel_control_law (Ptr<CoDelQueueDisc> queue, uint32_t t);
 };
 
@@ -378,20 +462,36 @@ CoDelQueueDiscControlLawTest::DoRun (void)
     }
 }
 
-// Test 5: enqueue/dequeue with drops according to CoDel algorithm
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief Test 5: enqueue/dequeue with drops according to CoDel algorithm
+ */
 class CoDelQueueDiscBasicDrop : public TestCase
 {
 public:
+  /**
+   * Constructor
+   *
+   * \param mode the mode
+   */
   CoDelQueueDiscBasicDrop (std::string mode);
   virtual void DoRun (void);
 
+  /**
+   * Queue test size function
+   * \param queue the queue disc
+   * \param size the size
+   * \param error the error string
+   */
   void QueueTestSize (Ptr<CoDelQueueDisc> queue, uint32_t size, std::string error)
   {
-    if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+    if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNBytes (), size, error);
       }
-    else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+    else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
       {
         NS_TEST_EXPECT_MSG_EQ (queue->GetNPackets (), size, error);
       }
@@ -400,11 +500,26 @@ public:
   }
 
 private:
+  /**
+   * Enqueue function
+   * \param queue the queue disc
+   * \param size the size
+   * \param nPkt the number of packets
+   */
   void Enqueue (Ptr<CoDelQueueDisc> queue, uint32_t size, uint32_t nPkt);
+  /** Dequeue function
+   * \param queue the queue disc
+   * \param modeSize the mode size
+   */
   void Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize);
+  /**
+   * Drop next tracer function
+   * \param oldVal the old value
+   * \param newVal the new value
+   */
   void DropNextTracer (uint32_t oldVal, uint32_t newVal);
-  StringValue m_mode;
-  uint32_t m_dropNextCount;    //count the number of times m_dropNext is recalculated
+  StringValue m_mode; ///< mode
+  uint32_t m_dropNextCount;    ///< count the number of times m_dropNext is recalculated
 };
 
 CoDelQueueDiscBasicDrop::CoDelQueueDiscBasicDrop (std::string mode)
@@ -430,11 +545,11 @@ CoDelQueueDiscBasicDrop::DoRun (void)
   NS_TEST_EXPECT_MSG_EQ (queue->SetAttributeFailSafe ("Mode", m_mode), true,
                          "Verify that we can actually set the attribute Mode");
   
-  if (queue->GetMode () == Queue::QUEUE_MODE_BYTES)
+  if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_BYTES)
     {
       modeSize = pktSize;
     }
-  else if (queue->GetMode () == Queue::QUEUE_MODE_PACKETS)
+  else if (queue->GetMode () == CoDelQueueDisc::QUEUE_DISC_MODE_PACKETS)
     {
       modeSize = 1;
     }
@@ -478,7 +593,7 @@ CoDelQueueDiscBasicDrop::Enqueue (Ptr<CoDelQueueDisc> queue, uint32_t size, uint
 void
 CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
 {
-  uint32_t initialDropCount = queue->GetDropCount ();
+  uint32_t initialDropCount = queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP);
   uint32_t initialQSize = queue->GetQueueSize ();
   uint32_t initialDropNext = queue->GetDropNext ();
   Time currentTime = Simulator::Now ();
@@ -496,7 +611,7 @@ CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
         {
           if (currentTime < queue->GetInterval ())
             {
-              currentDropCount = queue->GetDropCount ();
+              currentDropCount = queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP);
               NS_TEST_EXPECT_MSG_EQ (currentDropCount, 0, "We are not in dropping state."
                                      "Sojourn time has just gone above target from below."
                                      "Hence, there should be no packet drops");
@@ -505,7 +620,7 @@ CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
             }
           else if (currentTime >= queue->GetInterval ())
             {
-              currentDropCount = queue->GetDropCount ();
+              currentDropCount = queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP);
               QueueTestSize (queue, initialQSize - 2 * modeSize, "Sojourn time has been above target for at least interval."
                                      "We enter the dropping state, perform initial packet drop, and dequeue the next."
                                      "So there should be 2 more packets dequeued.");
@@ -516,7 +631,7 @@ CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
         { // In dropping state
           if (currentTime.GetMicroSeconds () < initialDropNext)
             {
-              currentDropCount = queue->GetDropCount ();
+              currentDropCount = queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP);
               QueueTestSize (queue, initialQSize - modeSize, "We are in dropping state."
                                      "Sojourn is still above target."
                                      "However, it's not time for next drop."
@@ -526,7 +641,7 @@ CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
             }
           else if (currentTime.GetMicroSeconds () >= initialDropNext)
             {
-              currentDropCount = queue->GetDropCount ();
+              currentDropCount = queue->GetStats ().GetNDroppedPackets (CoDelQueueDisc::TARGET_EXCEEDED_DROP);
               QueueTestSize (queue, initialQSize - (m_dropNextCount + 1) * modeSize, "We are in dropping state."
                                      "It's time for next drop."
                                      "The number of packets dequeued equals to the number of times m_dropNext is updated plus initial dequeue");
@@ -536,6 +651,12 @@ CoDelQueueDiscBasicDrop::Dequeue (Ptr<CoDelQueueDisc> queue, uint32_t modeSize)
     }
 }
 
+/**
+ * \ingroup traffic-control-test
+ * \ingroup tests
+ *
+ * \brief CoDel Queue Disc Test Suite
+ */
 static class CoDelQueueDiscTestSuite : public TestSuite
 {
 public:
@@ -543,17 +664,17 @@ public:
     : TestSuite ("codel-queue-disc", UNIT)
   {
     // Test 1: simple enqueue/dequeue with no drops
-    AddTestCase (new CoDelQueueDiscBasicEnqueueDequeue ("QUEUE_MODE_PACKETS"), TestCase::QUICK);
-    AddTestCase (new CoDelQueueDiscBasicEnqueueDequeue ("QUEUE_MODE_BYTES"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicEnqueueDequeue ("QUEUE_DISC_MODE_PACKETS"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicEnqueueDequeue ("QUEUE_DISC_MODE_BYTES"), TestCase::QUICK);
     // Test 2: enqueue with drops due to queue overflow
-    AddTestCase (new CoDelQueueDiscBasicOverflow ("QUEUE_MODE_PACKETS"), TestCase::QUICK);
-    AddTestCase (new CoDelQueueDiscBasicOverflow ("QUEUE_MODE_BYTES"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicOverflow ("QUEUE_DISC_MODE_PACKETS"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicOverflow ("QUEUE_DISC_MODE_BYTES"), TestCase::QUICK);
     // Test 3: test NewtonStep() against explicit port of Linux implementation
     AddTestCase (new CoDelQueueDiscNewtonStepTest (), TestCase::QUICK);
     // Test 4: test ControlLaw() against explicit port of Linux implementation
     AddTestCase (new CoDelQueueDiscControlLawTest (), TestCase::QUICK);
     // Test 5: enqueue/dequeue with drops according to CoDel algorithm
-    AddTestCase (new CoDelQueueDiscBasicDrop ("QUEUE_MODE_PACKETS"), TestCase::QUICK);
-    AddTestCase (new CoDelQueueDiscBasicDrop ("QUEUE_MODE_BYTES"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicDrop ("QUEUE_DISC_MODE_PACKETS"), TestCase::QUICK);
+    AddTestCase (new CoDelQueueDiscBasicDrop ("QUEUE_DISC_MODE_BYTES"), TestCase::QUICK);
   }
-} g_coDelQueueTestSuite;
+} g_coDelQueueTestSuite; ///< the test suite
