@@ -267,12 +267,6 @@ public:
    */
   virtual void SetReceiveErrorCallback (RxErrCallback cb) = 0;
 
-  /**
-   * Set the receiver gain.
-   *
-   * \param gain Gain added at receiver, in dB.
-   */
-  virtual void SetRxGainDb (double gain) = 0;
 
   /**
    * Set the transmit power.
@@ -297,12 +291,6 @@ public:
    */
   virtual void SetCcaThresholdDb (double thresh) = 0;
 
-  /**
-   * Get the receiver gain added to signal at receiver in dB.
-   *
-   * \return The gain.
-   */
-  virtual double GetRxGainDb (void) = 0;
 
   /**
    * Get the current transmit power, in dB.
