@@ -204,7 +204,8 @@ UanPdp::SumTapsFromMaxC (Time delay, Time duration) const
       NS_ASSERT_MSG (GetNTaps () == 1, "Attempted to sum taps over time interval in "
                      "UanPdp with resolution 0 and multiple taps");
 
-      return m_taps[0].GetAmp ();
+      if (delay.IsZero ()) return m_taps[0].GetAmp ();
+      return std::complex<double> (0.0, 0.0);
     }
 
   uint32_t numTaps =  static_cast<uint32_t> (duration.GetSeconds () / m_resolution.GetSeconds () + 0.5);
@@ -236,7 +237,8 @@ UanPdp::SumTapsFromMaxNc (Time delay, Time duration) const
       NS_ASSERT_MSG (GetNTaps () == 1, "Attempted to sum taps over time interval in "
                      "UanPdp with resolution 0 and multiple taps");
 
-      return std::abs (m_taps[0].GetAmp ());
+      if (delay.IsZero ()) return std::abs (m_taps[0].GetAmp ());
+      return 0;
     }
 
   uint32_t numTaps =  static_cast<uint32_t> (duration.GetSeconds () / m_resolution.GetSeconds () + 0.5);
