@@ -21,6 +21,7 @@
 #ifndef AARFCD_WIFI_MANAGER_H
 #define AARFCD_WIFI_MANAGER_H
 
+#include "ns3/traced-value.h"
 #include "wifi-remote-station-manager.h"
 
 namespace ns3 {
@@ -36,28 +37,33 @@ struct AarfcdWifiRemoteStation;
  * version of ns-3. Federico died before merging this work in ns-3 itself so his code was ported
  * to ns-3 later without his supervision.
  *
- * This RAA does not support HT or VHT modes and will error exit
- * if the user tries to configure this RAA with a Wi-Fi MAC that
- * has VhtSupported or HtSupported set.
+ * This RAA does not support HT, VHT nor HE modes and will error
+ * exit if the user tries to configure this RAA with a Wi-Fi MAC
+ * that has VhtSupported, HtSupported or HeSupported set.
  */
 class AarfcdWifiManager : public WifiRemoteStationManager
 {
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   AarfcdWifiManager ();
   virtual ~AarfcdWifiManager ();
 
   // Inherited from WifiRemoteStationManager
-  virtual void SetHtSupported (bool enable);
-  virtual void SetVhtSupported (bool enable);
+  void SetHtSupported (bool enable);
+  void SetVhtSupported (bool enable);
+  void SetHeSupported (bool enable);
 
 private:
   // overriden from base class
-  virtual WifiRemoteStation * DoCreateStation (void) const;
-  virtual void DoReportRxOk (WifiRemoteStation *station,
-                             double rxSnr, WifiMode txMode);
+  WifiRemoteStation * DoCreateStation (void) const;
+  void DoReportRxOk (WifiRemoteStation *station,
+                     double rxSnr, WifiMode txMode);
 
-  virtual void DoReportRtsFailed (WifiRemoteStation *station);
+  void DoReportRtsFailed (WifiRemoteStation *station);
   /**
    * It is important to realize that "recovery" mode starts after failure of
    * the first transmission after a rate increase and ends at the first successful
@@ -69,18 +75,18 @@ private:
    *
    * \param station the station that we failed to send DATA
    */
-  virtual void DoReportDataFailed (WifiRemoteStation *station);
-  virtual void DoReportRtsOk (WifiRemoteStation *station,
-                              double ctsSnr, WifiMode ctsMode, double rtsSnr);
-  virtual void DoReportDataOk (WifiRemoteStation *station,
-                               double ackSnr, WifiMode ackMode, double dataSnr);
-  virtual void DoReportFinalRtsFailed (WifiRemoteStation *station);
-  virtual void DoReportFinalDataFailed (WifiRemoteStation *station);
-  virtual WifiTxVector DoGetDataTxVector (WifiRemoteStation *station);
-  virtual WifiTxVector DoGetRtsTxVector (WifiRemoteStation *station);
-  virtual bool DoNeedRts (WifiRemoteStation *station,
-                          Ptr<const Packet> packet, bool normally);
-  virtual bool IsLowLatency (void) const;
+  void DoReportDataFailed (WifiRemoteStation *station);
+  void DoReportRtsOk (WifiRemoteStation *station,
+                      double ctsSnr, WifiMode ctsMode, double rtsSnr);
+  void DoReportDataOk (WifiRemoteStation *station,
+                       double ackSnr, WifiMode ackMode, double dataSnr);
+  void DoReportFinalRtsFailed (WifiRemoteStation *station);
+  void DoReportFinalDataFailed (WifiRemoteStation *station);
+  WifiTxVector DoGetDataTxVector (WifiRemoteStation *station);
+  WifiTxVector DoGetRtsTxVector (WifiRemoteStation *station);
+  bool DoNeedRts (WifiRemoteStation *station,
+                  Ptr<const Packet> packet, bool normally);
+  bool IsLowLatency (void) const;
 
   /**
    * Check if the use of RTS for the given station can be turned off.
@@ -114,17 +120,19 @@ private:
   void TurnOnRts (AarfcdWifiRemoteStation *station);
 
   //aarf fields below
-  uint32_t m_minTimerThreshold;
-  uint32_t m_minSuccessThreshold;
-  double m_successK;
-  uint32_t m_maxSuccessThreshold;
-  double m_timerK;
+  uint32_t m_minTimerThreshold; ///< minimum timer threshold
+  uint32_t m_minSuccessThreshold; ///< minimum success threshold
+  double m_successK; ///< Multiplication factor for the success threshold
+  uint32_t m_maxSuccessThreshold; ///< maximum success threshold
+  double m_timerK; ///< Multiplication factor for the timer threshold
 
   //aarf-cd fields below
-  uint32_t m_minRtsWnd;
-  uint32_t m_maxRtsWnd;
-  bool m_turnOffRtsAfterRateDecrease;
-  bool m_turnOnRtsAfterRateIncrease;
+  uint32_t m_minRtsWnd; ///< minimum RTS window
+  uint32_t m_maxRtsWnd; ///< maximum RTS window
+  bool m_turnOffRtsAfterRateDecrease; ///< turn off RTS after rate decrease
+  bool m_turnOnRtsAfterRateIncrease; ///< turn on RTS after rate increase
+
+  TracedValue<uint64_t> m_currentRate; //!< Trace rate changes
 };
 
 } //namespace ns3
