@@ -22,14 +22,11 @@
 #define AMPDU_SUBFRAME_HEADER_H
 
 #include "ns3/header.h"
-#include "ns3/mac48-address.h"
 
 namespace ns3 {
 
 /**
  * \ingroup wifi
- *
- *
  */
 class AmpduSubframeHeader : public Header
 {
@@ -37,12 +34,17 @@ public:
   AmpduSubframeHeader ();
   virtual ~AmpduSubframeHeader ();
 
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
   /**
    * Set the CRC field.
@@ -52,8 +54,6 @@ public:
   void SetCrc (uint8_t crc);
   /**
    * Set the SIG field.
-   *
-   * \param crc
    */
   void SetSig ();
   /**
@@ -93,7 +93,7 @@ public:
    */
   bool GetEof (void) const;
 
-protected:
+private:
   uint8_t m_crc;     //!< CRC field
   uint8_t m_sig;     //!< SIG field
   uint16_t m_length; //!< length field
