@@ -21,20 +21,12 @@
  */
 
 #include "ap-wifi-mac.h"
-#include "ns3/assert.h"
 #include "ns3/log.h"
 #include "ns3/simulator.h"
 #include "ns3/string.h"
 #include "ns3/pointer.h"
-#include "ns3/boolean.h"
-#include "wifi-phy.h"
-#include "dcf-manager.h"
-#include "mac-rx-middle.h"
-#include "mac-tx-middle.h"
-#include "mgt-headers.h"
 #include "mac-low.h"
-#include "amsdu-subframe-header.h"
-#include "msdu-aggregator.h"
+#include "mac-tx-middle.h"
 
 namespace ns3 {
 
@@ -63,7 +55,7 @@ ApWifiMac::GetTypeId (void)
                    MakePointerChecker<UniformRandomVariable> ())
     .AddAttribute ("EnableBeaconJitter",
                    "If beacons are enabled, whether to jitter the initial send event.",
-                   BooleanValue (false),
+                   BooleanValue (true),
                    MakeBooleanAccessor (&ApWifiMac::m_enableBeaconJitter),
                    MakeBooleanChecker ())
     .AddAttribute ("BeaconGeneration",
@@ -77,11 +69,17 @@ ApWifiMac::GetTypeId (void)
                    BooleanValue (true),
                    MakeBooleanAccessor (&ApWifiMac::m_enableNonErpProtection),
                    MakeBooleanChecker ())
+    .AddAttribute ("RifsMode", "If non-HT STAs are detected, whether to force RIFS to be disabled within the BSS."
+                   "This parameter is only used when HT is supported by the AP.",
+                   BooleanValue (true),
+                   MakeBooleanAccessor (&ApWifiMac::m_disableRifs),
+                   MakeBooleanChecker ())
   ;
   return tid;
 }
 
 ApWifiMac::ApWifiMac ()
+  : m_enableBeaconGeneration (false)
 {
   NS_LOG_FUNCTION (this);
   m_beaconDca = CreateObject<DcaTxop> ();
@@ -94,14 +92,12 @@ ApWifiMac::ApWifiMac ()
 
   //Let the lower layers know that we are acting as an AP.
   SetTypeOfStation (AP);
-
-  m_enableBeaconGeneration = false;
 }
 
 ApWifiMac::~ApWifiMac ()
 {
   NS_LOG_FUNCTION (this);
-  m_staList.clear();
+  m_staList.clear ();
   m_nonErpStations.clear ();
   m_nonHtStations.clear ();
 }
@@ -110,6 +106,7 @@ void
 ApWifiMac::DoDispose ()
 {
   NS_LOG_FUNCTION (this);
+  m_beaconDca->Dispose ();
   m_beaconDca = 0;
   m_enableBeaconGeneration = false;
   m_beaconEvent.Cancel ();
@@ -156,7 +153,7 @@ ApWifiMac::GetBeaconInterval (void) const
 }
 
 void
-ApWifiMac::SetWifiRemoteStationManager (Ptr<WifiRemoteStationManager> stationManager)
+ApWifiMac::SetWifiRemoteStationManager (const Ptr<WifiRemoteStationManager> stationManager)
 {
   NS_LOG_FUNCTION (this << stationManager);
   m_beaconDca->SetWifiRemoteStationManager (stationManager);
@@ -211,12 +208,12 @@ ApWifiMac::GetShortSlotTimeEnabled (void) const
   if (m_erpSupported == true && GetShortSlotTimeSupported () == true)
     {
       for (std::list<Mac48Address>::const_iterator i = m_staList.begin (); i != m_staList.end (); i++)
-      {
-        if (m_stationManager->GetShortSlotTimeSupported (*i) == false)
-          {
-            return false;
-          }
-      }
+        {
+          if (m_stationManager->GetShortSlotTimeSupported (*i) == false)
+            {
+              return false;
+            }
+        }
       return true;
     }
   return false;
@@ -228,17 +225,50 @@ ApWifiMac::GetShortPreambleEnabled (void) const
   if (m_erpSupported || m_phy->GetShortPlcpPreambleSupported ())
     {
       for (std::list<Mac48Address>::const_iterator i = m_nonErpStations.begin (); i != m_nonErpStations.end (); i++)
-      {
-        if (m_stationManager->GetShortPreambleSupported (*i) == false)
-          {
-            return false;
-          }
-      }
+        {
+          if (m_stationManager->GetShortPreambleSupported (*i) == false)
+            {
+              return false;
+            }
+        }
       return true;
     }
   return false;
 }
 
+bool
+ApWifiMac::IsNonGfHtStasPresent (void) const
+{
+  bool isNonGfHtStasPresent = false;
+  for (std::list<Mac48Address>::const_iterator i = m_staList.begin (); i != m_staList.end (); i++)
+    {
+      if (m_stationManager->GetGreenfieldSupported (*i) == false)
+        {
+          isNonGfHtStasPresent = true;
+          break;
+        }
+    }
+  m_stationManager->SetUseGreenfieldProtection (isNonGfHtStasPresent);
+  return isNonGfHtStasPresent;
+}
+
+uint8_t
+ApWifiMac::GetVhtOperationalChannelWidth (void) const
+{
+  uint8_t channelWidth = m_phy->GetChannelWidth ();
+  for (std::list<Mac48Address>::const_iterator i = m_staList.begin (); i != m_staList.end (); i++)
+    {
+      if (m_stationManager->GetVhtSupported (*i))
+        {
+          if (m_stationManager->GetChannelWidthSupported (*i) < channelWidth)
+            {
+              channelWidth = m_stationManager->GetChannelWidthSupported (*i);
+            }
+        }
+    }
+  return channelWidth;
+}
+
 void
 ApWifiMac::ForwardDown (Ptr<const Packet> packet, Mac48Address from,
                         Mac48Address to)
@@ -293,7 +323,7 @@ ApWifiMac::ForwardDown (Ptr<const Packet> packet, Mac48Address from,
       hdr.SetTypeData ();
     }
 
-  if (m_htSupported || m_vhtSupported)
+  if (m_htSupported || m_vhtSupported || m_heSupported)
     {
       hdr.SetNoOrder ();
     }
@@ -323,6 +353,10 @@ ApWifiMac::Enqueue (Ptr<const Packet> packet, Mac48Address to, Mac48Address from
     {
       ForwardDown (packet, from, to);
     }
+  else
+    {
+      NotifyTxDrop (packet);
+    }
 }
 
 void
@@ -347,25 +381,24 @@ ApWifiMac::GetSupportedRates (void) const
 {
   NS_LOG_FUNCTION (this);
   SupportedRates rates;
-  //If it is an HT-AP or VHT-AP, then add the BSSMembershipSelectorSet
+  //If it is an HT-AP or VHT-AP or HE-AP, then add the BSSMembershipSelectorSet
   //The standard says that the BSSMembershipSelectorSet
   //must have its MSB set to 1 (must be treated as a Basic Rate)
   //Also the standard mentioned that at least 1 element should be included in the SupportedRates the rest can be in the ExtendedSupportedRates
-  if (m_htSupported || m_vhtSupported)
+  if (m_htSupported || m_vhtSupported || m_heSupported)
     {
       for (uint32_t i = 0; i < m_phy->GetNBssMembershipSelectors (); i++)
         {
           rates.AddBssMembershipSelectorRate (m_phy->GetBssMembershipSelector (i));
         }
     }
-  // 
-  uint8_t nss = 1;  // Number of spatial streams is 1 for non-MIMO modes
+  //
   //Send the set of supported rates and make sure that we indicate
   //the Basic Rate set in this set of supported rates.
   for (uint32_t i = 0; i < m_phy->GetNModes (); i++)
     {
       WifiMode mode = m_phy->GetMode (i);
-      uint64_t modeDataRate = mode.GetDataRate (m_phy->GetChannelWidth (), false, nss);
+      uint64_t modeDataRate = mode.GetDataRate (m_phy->GetChannelWidth ());
       NS_LOG_DEBUG ("Adding supported rate of " << modeDataRate);
       rates.AddSupportedRate (modeDataRate);
       //Add rates that are part of the BSSBasicRateSet (manufacturer dependent!)
@@ -381,7 +414,7 @@ ApWifiMac::GetSupportedRates (void) const
   for (uint32_t j = 0; j < m_stationManager->GetNBasicModes (); j++)
     {
       WifiMode mode = m_stationManager->GetBasicMode (j);
-      uint64_t modeDataRate = mode.GetDataRate (m_phy->GetChannelWidth (), false, nss);
+      uint64_t modeDataRate = mode.GetDataRate (m_phy->GetChannelWidth ());
       NS_LOG_DEBUG ("Setting basic rate " << mode.GetUniqueName ());
       rates.SetBasicRate (modeDataRate);
     }
@@ -392,6 +425,7 @@ ApWifiMac::GetSupportedRates (void) const
 DsssParameterSet
 ApWifiMac::GetDsssParameterSet (void) const
 {
+  NS_LOG_FUNCTION (this);
   DsssParameterSet dsssParameters;
   if (m_dsssSupported)
     {
@@ -404,6 +438,7 @@ ApWifiMac::GetDsssParameterSet (void) const
 CapabilityInformation
 ApWifiMac::GetCapabilities (void) const
 {
+  NS_LOG_FUNCTION (this);
   CapabilityInformation capabilities;
   capabilities.SetShortPreamble (GetShortPreambleEnabled ());
   capabilities.SetShortSlotTime (GetShortSlotTimeEnabled ());
@@ -413,6 +448,7 @@ ApWifiMac::GetCapabilities (void) const
 ErpInformation
 ApWifiMac::GetErpInformation (void) const
 {
+  NS_LOG_FUNCTION (this);
   ErpInformation information;
   information.SetErpSupported (1);
   if (m_erpSupported)
@@ -434,65 +470,108 @@ ApWifiMac::GetErpInformation (void) const
 EdcaParameterSet
 ApWifiMac::GetEdcaParameterSet (void) const
 {
+  NS_LOG_FUNCTION (this);
   EdcaParameterSet edcaParameters;
-  edcaParameters.SetQosSupported (1);
   if (m_qosSupported)
     {
+      edcaParameters.SetQosSupported (1);
       Ptr<EdcaTxopN> edca;
       Time txopLimit;
 
       edca = m_edca.find (AC_BE)->second;
       txopLimit = edca->GetTxopLimit ();
-      edcaParameters.SetBeAci(0);
-      edcaParameters.SetBeCWmin(edca->GetMinCw ());
-      edcaParameters.SetBeCWmax(edca->GetMaxCw ());
-      edcaParameters.SetBeAifsn(edca->GetAifsn ());
-      edcaParameters.SetBeTXOPLimit(txopLimit.GetMicroSeconds () / 32);
-      
+      edcaParameters.SetBeAci (0);
+      edcaParameters.SetBeCWmin (edca->GetMinCw ());
+      edcaParameters.SetBeCWmax (edca->GetMaxCw ());
+      edcaParameters.SetBeAifsn (edca->GetAifsn ());
+      edcaParameters.SetBeTXOPLimit (txopLimit.GetMicroSeconds () / 32);
+
       edca = m_edca.find (AC_BK)->second;
       txopLimit = edca->GetTxopLimit ();
-      edcaParameters.SetBkAci(1);
-      edcaParameters.SetBkCWmin(edca->GetMinCw ());
-      edcaParameters.SetBkCWmax(edca->GetMaxCw ());
-      edcaParameters.SetBkAifsn(edca->GetAifsn ());
-      edcaParameters.SetBkTXOPLimit(txopLimit.GetMicroSeconds () / 32);
-      
+      edcaParameters.SetBkAci (1);
+      edcaParameters.SetBkCWmin (edca->GetMinCw ());
+      edcaParameters.SetBkCWmax (edca->GetMaxCw ());
+      edcaParameters.SetBkAifsn (edca->GetAifsn ());
+      edcaParameters.SetBkTXOPLimit (txopLimit.GetMicroSeconds () / 32);
+
       edca = m_edca.find (AC_VI)->second;
       txopLimit = edca->GetTxopLimit ();
-      edcaParameters.SetViAci(2);
-      edcaParameters.SetViCWmin(edca->GetMinCw ());
-      edcaParameters.SetViCWmax(edca->GetMaxCw ());
-      edcaParameters.SetViAifsn(edca->GetAifsn ());
-      edcaParameters.SetViTXOPLimit(txopLimit.GetMicroSeconds () / 32);
-      
+      edcaParameters.SetViAci (2);
+      edcaParameters.SetViCWmin (edca->GetMinCw ());
+      edcaParameters.SetViCWmax (edca->GetMaxCw ());
+      edcaParameters.SetViAifsn (edca->GetAifsn ());
+      edcaParameters.SetViTXOPLimit (txopLimit.GetMicroSeconds () / 32);
+
       edca = m_edca.find (AC_VO)->second;
       txopLimit = edca->GetTxopLimit ();
-      edcaParameters.SetVoAci(3);
-      edcaParameters.SetVoCWmin(edca->GetMinCw ());
-      edcaParameters.SetVoCWmax(edca->GetMaxCw ());
-      edcaParameters.SetVoAifsn(edca->GetAifsn ());
-      edcaParameters.SetVoTXOPLimit(txopLimit.GetMicroSeconds () / 32);
+      edcaParameters.SetVoAci (3);
+      edcaParameters.SetVoCWmin (edca->GetMinCw ());
+      edcaParameters.SetVoCWmax (edca->GetMaxCw ());
+      edcaParameters.SetVoAifsn (edca->GetAifsn ());
+      edcaParameters.SetVoTXOPLimit (txopLimit.GetMicroSeconds () / 32);
     }
   return edcaParameters;
 }
 
-HtOperations
-ApWifiMac::GetHtOperations (void) const
+HtOperation
+ApWifiMac::GetHtOperation (void) const
 {
-  HtOperations operations;
-  operations.SetHtSupported (1);
+  NS_LOG_FUNCTION (this);
+  HtOperation operation;
   if (m_htSupported)
     {
-      if (!m_nonHtStations.empty ())
+      operation.SetHtSupported (1);
+      operation.SetRifsMode (GetRifsMode ());
+      operation.SetNonGfHtStasPresent (IsNonGfHtStasPresent ());
+      if (m_nonHtStations.empty ())
         {
-          operations.SetHtProtection (MIXED_MODE_PROTECTION);
+          operation.SetHtProtection (NO_PROTECTION);
         }
       else
         {
-          operations.SetHtProtection (NO_PROTECTION);
+          operation.SetHtProtection (MIXED_MODE_PROTECTION);
         }
     }
-  return operations;
+  return operation;
+}
+
+VhtOperation
+ApWifiMac::GetVhtOperation (void) const
+{
+  NS_LOG_FUNCTION (this);
+  VhtOperation operation;
+  if (m_vhtSupported)
+    {
+      operation.SetVhtSupported (1);
+      uint8_t channelWidth = GetVhtOperationalChannelWidth ();
+      if (channelWidth == 160)
+        {
+          operation.SetChannelWidth (2);
+        }
+      else if (channelWidth == 80)
+        {
+          operation.SetChannelWidth (1);
+        }
+      else
+        {
+          operation.SetChannelWidth (0);
+        }
+      for (uint8_t nss = 1; nss <= 8; nss++)
+        {
+          uint8_t maxMcs;
+          if (nss <= m_phy->GetMaxSupportedRxSpatialStreams ())
+            {
+              maxMcs = 9; //TBD: hardcode to 9 for now since we assume all MCS values are supported
+            }
+          else
+            {
+              maxMcs = 0;
+            }
+          operation.SetMaxVhtMcsPerNss (nss, maxMcs);
+        }
+
+    }
+  return operation;
 }
 
 void
@@ -506,6 +585,7 @@ ApWifiMac::SendProbeResp (Mac48Address to)
   hdr.SetAddr3 (GetAddress ());
   hdr.SetDsNotFrom ();
   hdr.SetDsNotTo ();
+  hdr.SetNoOrder ();
   Ptr<Packet> packet = Create<Packet> ();
   MgtProbeResponseHeader probe;
   probe.SetSsid (GetSsid ());
@@ -526,15 +606,19 @@ ApWifiMac::SendProbeResp (Mac48Address to)
     {
       probe.SetEdcaParameterSet (GetEdcaParameterSet ());
     }
-  if (m_htSupported || m_vhtSupported)
+  if (m_htSupported || m_vhtSupported || m_heSupported)
     {
       probe.SetHtCapabilities (GetHtCapabilities ());
-      probe.SetHtOperations (GetHtOperations ());
-      hdr.SetNoOrder ();
+      probe.SetHtOperation (GetHtOperation ());
     }
-  if (m_vhtSupported)
+  if (m_vhtSupported || m_heSupported)
     {
       probe.SetVhtCapabilities (GetVhtCapabilities ());
+      probe.SetVhtOperation (GetVhtOperation ());
+    }
+  if (m_heSupported)
+    {
+      probe.SetHeCapabilities (GetHeCapabilities ());
     }
   packet->AddHeader (probe);
 
@@ -556,6 +640,7 @@ ApWifiMac::SendAssocResp (Mac48Address to, bool success)
   hdr.SetAddr3 (GetAddress ());
   hdr.SetDsNotFrom ();
   hdr.SetDsNotTo ();
+  hdr.SetNoOrder ();
   Ptr<Packet> packet = Create<Packet> ();
   MgtAssocResponseHeader assoc;
   StatusCode code;
@@ -579,15 +664,19 @@ ApWifiMac::SendAssocResp (Mac48Address to, bool success)
     {
       assoc.SetEdcaParameterSet (GetEdcaParameterSet ());
     }
-  if (m_htSupported || m_vhtSupported)
+  if (m_htSupported || m_vhtSupported || m_heSupported)
     {
       assoc.SetHtCapabilities (GetHtCapabilities ());
-      assoc.SetHtOperations (GetHtOperations ());
-      hdr.SetNoOrder ();
+      assoc.SetHtOperation (GetHtOperation ());
     }
-  if (m_vhtSupported)
+  if (m_vhtSupported || m_heSupported)
     {
       assoc.SetVhtCapabilities (GetVhtCapabilities ());
+      assoc.SetVhtOperation (GetVhtOperation ());
+    }
+  if (m_heSupported)
+    {
+      assoc.SetHeCapabilities (GetHeCapabilities ());
     }
   packet->AddHeader (assoc);
 
@@ -609,6 +698,7 @@ ApWifiMac::SendOneBeacon (void)
   hdr.SetAddr3 (GetAddress ());
   hdr.SetDsNotFrom ();
   hdr.SetDsNotTo ();
+  hdr.SetNoOrder ();
   Ptr<Packet> packet = Create<Packet> ();
   MgtBeaconHeader beacon;
   beacon.SetSsid (GetSsid ());
@@ -632,34 +722,38 @@ ApWifiMac::SendOneBeacon (void)
   if (m_htSupported || m_vhtSupported)
     {
       beacon.SetHtCapabilities (GetHtCapabilities ());
-      beacon.SetHtOperations (GetHtOperations ());
-      hdr.SetNoOrder ();
+      beacon.SetHtOperation (GetHtOperation ());
     }
-  if (m_vhtSupported)
+  if (m_vhtSupported || m_heSupported)
     {
       beacon.SetVhtCapabilities (GetVhtCapabilities ());
+      beacon.SetVhtOperation (GetVhtOperation ());
+    }
+  if (m_heSupported)
+    {
+      beacon.SetHeCapabilities (GetHeCapabilities ());
     }
   packet->AddHeader (beacon);
 
   //The beacon has it's own special queue, so we load it in there
   m_beaconDca->Queue (packet, hdr);
   m_beaconEvent = Simulator::Schedule (m_beaconInterval, &ApWifiMac::SendOneBeacon, this);
-  
+
   //If a STA that does not support Short Slot Time associates,
   //the AP shall use long slot time beginning at the first Beacon
   //subsequent to the association of the long slot time STA.
   if (m_erpSupported)
     {
-    if (GetShortSlotTimeEnabled () == true)
-      {
-        //Enable short slot time
-        SetSlot (MicroSeconds (9));
-      }
-    else
-      {
-        //Disable short slot time
-        SetSlot (MicroSeconds (20));
-      }
+      if (GetShortSlotTimeEnabled () == true)
+        {
+          //Enable short slot time
+          SetSlot (MicroSeconds (9));
+        }
+      else
+        {
+          //Disable short slot time
+          SetSlot (MicroSeconds (20));
+        }
     }
 }
 
@@ -794,8 +888,7 @@ ApWifiMac::Receive (Ptr<Packet> packet, const WifiMacHeader *hdr)
               for (uint32_t i = 0; i < m_stationManager->GetNBasicModes (); i++)
                 {
                   WifiMode mode = m_stationManager->GetBasicMode (i);
-                  uint8_t nss = 1; // Assume 1 spatial stream in basic mode
-                  if (!rates.IsSupportedRate (mode.GetDataRate (m_phy->GetChannelWidth (), false, nss)))
+                  if (!rates.IsSupportedRate (mode.GetDataRate (m_phy->GetChannelWidth ())))
                     {
                       if ((mode.GetModulationClass () == WIFI_MOD_CLASS_DSSS) || (mode.GetModulationClass () == WIFI_MOD_CLASS_HR_DSSS))
                         {
@@ -836,7 +929,7 @@ ApWifiMac::Receive (Ptr<Packet> packet, const WifiMacHeader *hdr)
                 {
                   //check whether the HT STA supports all MCSs in Basic MCS Set
                   HtCapabilities htcapabilities = assocReq.GetHtCapabilities ();
-                  if (htcapabilities.GetHtCapabilitiesInfo () != 0)
+                  if (htcapabilities.IsSupportedMcs (0))
                     {
                       isHtStation = true;
                       for (uint32_t i = 0; i < m_stationManager->GetNBasicMcs (); i++)
@@ -867,6 +960,23 @@ ApWifiMac::Receive (Ptr<Packet> packet, const WifiMacHeader *hdr)
                         }
                     }
                 }
+              if (m_heSupported)
+                {
+                  //check whether the HE STA supports all MCSs in Basic MCS Set
+                  HeCapabilities hecapabilities = assocReq.GetHeCapabilities ();
+                  if (hecapabilities.GetSupportedMcsAndNss () != 0)
+                    {
+                      for (uint32_t i = 0; i < m_stationManager->GetNBasicMcs (); i++)
+                        {
+                          WifiMode mcs = m_stationManager->GetBasicMcs (i);
+                          if (!hecapabilities.IsSupportedTxMcs (mcs.GetMcsValue ()))
+                            {
+                              problem = true;
+                              break;
+                            }
+                        }
+                    }
+                }
               if (problem)
                 {
                   //One of the Basic Rate set mode is not
@@ -881,33 +991,54 @@ ApWifiMac::Receive (Ptr<Packet> packet, const WifiMacHeader *hdr)
                   for (uint32_t j = 0; j < m_phy->GetNModes (); j++)
                     {
                       WifiMode mode = m_phy->GetMode (j);
-                      uint8_t nss = 1; // Assume 1 spatial stream in basic mode
-                      if (rates.IsSupportedRate (mode.GetDataRate (m_phy->GetChannelWidth (), false, nss)))
+                      if (rates.IsSupportedRate (mode.GetDataRate (m_phy->GetChannelWidth ())))
                         {
                           m_stationManager->AddSupportedMode (from, mode);
                         }
                     }
                   if (m_htSupported)
                     {
-                      HtCapabilities htcapabilities = assocReq.GetHtCapabilities ();
-                      m_stationManager->AddStationHtCapabilities (from, htcapabilities);
-                      for (uint32_t j = 0; j < m_phy->GetNMcs (); j++)
+                      HtCapabilities htCapabilities = assocReq.GetHtCapabilities ();
+                      if (htCapabilities.IsSupportedMcs (0))
                         {
-                          WifiMode mcs = m_phy->GetMcs (j);
-                          if (mcs.GetModulationClass () == WIFI_MOD_CLASS_HT && htcapabilities.IsSupportedMcs (mcs.GetMcsValue ()))
+                          m_stationManager->AddStationHtCapabilities (from, htCapabilities);
+                          for (uint32_t j = 0; j < m_phy->GetNMcs (); j++)
                             {
-                              m_stationManager->AddSupportedMcs (from, mcs);
+                              WifiMode mcs = m_phy->GetMcs (j);
+                              if (mcs.GetModulationClass () == WIFI_MOD_CLASS_HT && htCapabilities.IsSupportedMcs (mcs.GetMcsValue ()))
+                                {
+                                  m_stationManager->AddSupportedMcs (from, mcs);
+                                }
                             }
                         }
                     }
                   if (m_vhtSupported)
                     {
                       VhtCapabilities vhtCapabilities = assocReq.GetVhtCapabilities ();
-                      m_stationManager->AddStationVhtCapabilities (from, vhtCapabilities);
+                      //we will always fill in RxHighestSupportedLgiDataRate field at TX, so this can be used to check whether it supports VHT
+                      if (vhtCapabilities.GetRxHighestSupportedLgiDataRate () > 0)
+                        {
+                          m_stationManager->AddStationVhtCapabilities (from, vhtCapabilities);
+                          for (uint32_t i = 0; i < m_phy->GetNMcs (); i++)
+                            {
+                              WifiMode mcs = m_phy->GetMcs (i);
+                              if (mcs.GetModulationClass () == WIFI_MOD_CLASS_VHT && vhtCapabilities.IsSupportedTxMcs (mcs.GetMcsValue ()))
+                                {
+                                  m_stationManager->AddSupportedMcs (hdr->GetAddr2 (), mcs);
+                                  //here should add a control to add basic MCS when it is implemented
+                                }
+                            }
+                        }
+                    }
+                  if (m_heSupported)
+                    {
+                      HeCapabilities heCapabilities = assocReq.GetHeCapabilities ();
+                      //todo: once we support non constant rate managers, we should add checks here whether HE is supported by the peer
+                      m_stationManager->AddStationHeCapabilities (from, heCapabilities);
                       for (uint32_t i = 0; i < m_phy->GetNMcs (); i++)
                         {
                           WifiMode mcs = m_phy->GetMcs (i);
-                          if (mcs.GetModulationClass () == WIFI_MOD_CLASS_VHT && vhtCapabilities.IsSupportedTxMcs (mcs.GetMcsValue ()))
+                          if (mcs.GetModulationClass () == WIFI_MOD_CLASS_HE && heCapabilities.IsSupportedTxMcs (mcs.GetMcsValue ()))
                             {
                               m_stationManager->AddSupportedMcs (hdr->GetAddr2 (), mcs);
                               //here should add a control to add basic MCS when it is implemented
@@ -931,30 +1062,30 @@ ApWifiMac::Receive (Ptr<Packet> packet, const WifiMacHeader *hdr)
           else if (hdr->IsDisassociation ())
             {
               m_stationManager->RecordDisassociated (from);
-              for (std::list<Mac48Address>::iterator i = m_staList.begin (); i != m_staList.end (); i++)
-              {
-                if ((*i) == from)
-                  {
-                    m_staList.erase (i);
-                    break;
-                  }
-              }
-              for (std::list<Mac48Address>::iterator j = m_nonErpStations.begin (); j != m_nonErpStations.end (); j++)
-              {
-                if ((*j) == from)
-                  {
-                    m_nonErpStations.erase (j);
-                    break;
-                  }
-              }
-              for (std::list<Mac48Address>::iterator j = m_nonHtStations.begin (); j != m_nonHtStations.end (); j++)
-              {
-                if ((*j) == from)
-                  {
-                    m_nonHtStations.erase (j);
-                    break;
-                  }
-              }
+              for (std::list<Mac48Address>::const_iterator i = m_staList.begin (); i != m_staList.end (); i++)
+                {
+                  if ((*i) == from)
+                    {
+                      m_staList.erase (i);
+                      break;
+                    }
+                }
+              for (std::list<Mac48Address>::const_iterator j = m_nonErpStations.begin (); j != m_nonErpStations.end (); j++)
+                {
+                  if ((*j) == from)
+                    {
+                      m_nonErpStations.erase (j);
+                      break;
+                    }
+                }
+              for (std::list<Mac48Address>::const_iterator j = m_nonHtStations.begin (); j != m_nonHtStations.end (); j++)
+                {
+                  if ((*j) == from)
+                    {
+                      m_nonHtStations.erase (j);
+                      break;
+                    }
+                }
               return;
             }
         }
@@ -1023,4 +1154,26 @@ ApWifiMac::GetUseNonErpProtection (void) const
   return useProtection;
 }
 
+bool
+ApWifiMac::GetRifsMode (void) const
+{
+  bool rifsMode = false;
+  if (m_htSupported && !m_vhtSupported) //RIFS mode is forbidden for VHT
+    {
+      if (m_nonHtStations.empty () || !m_disableRifs)
+        {
+          rifsMode = true;
+        }
+    }
+  if (GetRifsSupported () && rifsMode)
+    {
+      m_stationManager->SetRifsPermitted (true);
+    }
+  else
+    {
+      m_stationManager->SetRifsPermitted (false);
+    }
+  return rifsMode;
+}
+
 } //namespace ns3
