@@ -20,17 +20,15 @@
 
 #include "aparf-wifi-manager.h"
 #include "wifi-phy.h"
-#include "ns3/assert.h"
 #include "ns3/log.h"
 #include "ns3/uinteger.h"
-#include "ns3/trace-source-accessor.h"
 
 #define Min(a,b) ((a < b) ? a : b)
 
-NS_LOG_COMPONENT_DEFINE ("ns3::AparfWifiManager");
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("AparfWifiManager");
+
 /**
  * Hold per-remote-station state for APARF Wifi manager.
  *
@@ -45,9 +43,11 @@ AparfWifiRemoteStation : public WifiRemoteStation
   uint32_t m_pCount;                    //!< Number of power changes.
   uint32_t m_successThreshold;          //!< The minimum number of successful transmissions to try a new power or rate.
   uint32_t m_failThreshold;             //!< The minimum number of failed transmissions to try a new power or rate.
-  uint32_t m_rate;                      //!< Current rate.
-  uint32_t m_rateCrit;                  //!< Critical rate.
-  uint8_t m_power;                      //!< Current power.
+  uint32_t m_prevRateIndex;             //!< Rate index of the previous transmission.
+  uint32_t m_rateIndex;                 //!< Current rate index.
+  uint32_t m_critRateIndex;             //!< Critical rate.
+  uint8_t m_prevPowerLevel;             //!< Power level of the previous transmission.
+  uint8_t m_powerLevel;                 //!< Current power level.
   uint32_t m_nSupported;                //!< Number of supported rates by the remote station.
   bool m_initialized;                   //!< For initializing variables.
   AparfWifiManager::State m_aparfState; //!< The estimated state of the channel.
@@ -125,7 +125,7 @@ AparfWifiManager::~AparfWifiManager ()
 }
 
 void
-AparfWifiManager::SetupPhy (Ptr<WifiPhy> phy)
+AparfWifiManager::SetupPhy (const Ptr<WifiPhy> phy)
 {
   m_minPower = phy->GetTxPowerStart ();
   m_maxPower = phy->GetTxPowerEnd ();
@@ -146,8 +146,8 @@ AparfWifiManager::DoCreateStation (void) const
   station->m_aparfState = AparfWifiManager::High;
   station->m_initialized = false;
 
-  NS_LOG_DEBUG ("create station=" << station << ", rate=" << station->m_rate
-                                  << ", power=" << (int)station->m_power);
+  NS_LOG_DEBUG ("create station=" << station << ", rate=" << station->m_rateIndex
+                                  << ", power=" << (int)station->m_powerLevel);
 
   return station;
 }
@@ -158,11 +158,17 @@ AparfWifiManager::CheckInit (AparfWifiRemoteStation *station)
   if (!station->m_initialized)
     {
       station->m_nSupported = GetNSupported (station);
-      station->m_rate = station->m_nSupported - 1;
-      station->m_power = m_maxPower;
-      station->m_rateCrit = 0;
-      m_powerChange (station->m_power, station->m_state->m_address);
-      m_rateChange (station->m_rate, station->m_state->m_address);
+      station->m_rateIndex = station->m_nSupported - 1;
+      station->m_prevRateIndex = station->m_nSupported - 1;
+      station->m_powerLevel = m_maxPower;
+      station->m_prevPowerLevel = m_maxPower;
+      station->m_critRateIndex = 0;
+      WifiMode mode = GetSupported (station, station->m_rateIndex);
+      uint8_t channelWidth = GetChannelWidth (station);
+      DataRate rate = DataRate (mode.GetDataRate (channelWidth));
+      double power = GetPhy ()->GetPowerDbm (m_maxPower);
+      m_powerChange (power, power, station->m_state->m_address);
+      m_rateChange (rate, rate, station->m_state->m_address);
       station->m_initialized = true;
     }
 }
@@ -179,8 +185,8 @@ void AparfWifiManager::DoReportDataFailed (WifiRemoteStation *st)
   CheckInit (station);
   station->m_nFailed++;
   station->m_nSuccess = 0;
-  NS_LOG_DEBUG ("station=" << station << ", rate=" << station->m_rate
-                           << ", power=" << (int)station->m_power);
+  NS_LOG_DEBUG ("station=" << station << ", rate=" << station->m_rateIndex
+                           << ", power=" << (int)station->m_powerLevel);
 
   if (station->m_aparfState == AparfWifiManager::Low)
     {
@@ -198,21 +204,19 @@ void AparfWifiManager::DoReportDataFailed (WifiRemoteStation *st)
       station->m_nFailed = 0;
       station->m_nSuccess = 0;
       station->m_pCount = 0;
-      if (station->m_power == m_maxPower)
+      if (station->m_powerLevel == m_maxPower)
         {
-          station->m_rateCrit = station->m_rate;
-          if (station->m_rate != 0)
+          station->m_critRateIndex = station->m_rateIndex;
+          if (station->m_rateIndex != 0)
             {
               NS_LOG_DEBUG ("station=" << station << " dec rate");
-              station->m_rate -= m_rateDec;
-              m_rateChange (station->m_rate, station->m_state->m_address);
+              station->m_rateIndex -= m_rateDec;
             }
         }
       else
         {
           NS_LOG_DEBUG ("station=" << station << " inc power");
-          station->m_power += m_powerInc;
-          m_powerChange (station->m_power, station->m_state->m_address);
+          station->m_powerLevel += m_powerInc;
         }
     }
 }
@@ -240,7 +244,7 @@ AparfWifiManager::DoReportDataOk (WifiRemoteStation *st, double ackSnr,
   CheckInit (station);
   station->m_nSuccess++;
   station->m_nFailed = 0;
-  NS_LOG_DEBUG ("station=" << station << " data ok success=" << station->m_nSuccess << ", rate=" << station->m_rate << ", power=" << (int)station->m_power);
+  NS_LOG_DEBUG ("station=" << station << " data ok success=" << station->m_nSuccess << ", rate=" << station->m_rateIndex << ", power=" << (int)station->m_powerLevel);
 
   if ((station->m_aparfState == AparfWifiManager::High) && (station->m_nSuccess >= station->m_successThreshold))
     {
@@ -260,43 +264,38 @@ AparfWifiManager::DoReportDataOk (WifiRemoteStation *st, double ackSnr,
     {
       station->m_nSuccess = 0;
       station->m_nFailed = 0;
-      if (station->m_rate == (station->m_state->m_operationalRateSet.size () - 1))
+      if (station->m_rateIndex == (station->m_state->m_operationalRateSet.size () - 1))
         {
-          if (station->m_power != m_minPower)
+          if (station->m_powerLevel != m_minPower)
             {
               NS_LOG_DEBUG ("station=" << station << " dec power");
-              station->m_power -= m_powerDec;
-              m_powerChange (station->m_power, station->m_state->m_address);
+              station->m_powerLevel -= m_powerDec;
             }
         }
       else
         {
-          if (station->m_rateCrit == 0)
+          if (station->m_critRateIndex == 0)
             {
-              if (station->m_rate != (station->m_state->m_operationalRateSet.size () - 1))
+              if (station->m_rateIndex != (station->m_state->m_operationalRateSet.size () - 1))
                 {
                   NS_LOG_DEBUG ("station=" << station << " inc rate");
-                  station->m_rate += m_rateInc;
-                  m_rateChange (station->m_rate, station->m_state->m_address);
+                  station->m_rateIndex += m_rateInc;
                 }
             }
           else
             {
               if (station->m_pCount == m_powerMax)
                 {
-                  station->m_power = m_maxPower;
-                  m_powerChange (station->m_power, station->m_state->m_address);
-                  station->m_rate = station->m_rateCrit;
-                  m_rateChange (station->m_rate, station->m_state->m_address);
+                  station->m_powerLevel = m_maxPower;
+                  station->m_rateIndex = station->m_critRateIndex;
                   station->m_pCount = 0;
-                  station->m_rateCrit = 0;
+                  station->m_critRateIndex = 0;
                 }
               else
                 {
-                  if (station->m_power != m_minPower)
+                  if (station->m_powerLevel != m_minPower)
                     {
-                      station->m_power -= m_powerDec;
-                      m_powerChange (station->m_power, station->m_state->m_address);
+                      station->m_powerLevel -= m_powerDec;
                       station->m_pCount++;
                     }
                 }
@@ -329,7 +328,22 @@ AparfWifiManager::DoGetDataTxVector (WifiRemoteStation *st)
       channelWidth = 20;
     }
   CheckInit (station);
-  return WifiTxVector (GetSupported (station, station->m_rate), station->m_power, GetLongRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+  WifiMode mode = GetSupported (station, station->m_rateIndex);
+  DataRate rate = DataRate (mode.GetDataRate (channelWidth));
+  DataRate prevRate = DataRate (GetSupported (station, station->m_prevRateIndex).GetDataRate (channelWidth));
+  double power = GetPhy ()->GetPowerDbm (station->m_powerLevel);
+  double prevPower = GetPhy ()->GetPowerDbm (station->m_prevPowerLevel);
+  if (station->m_prevPowerLevel != station->m_powerLevel)
+    {
+      m_powerChange (prevPower, power, station->m_state->m_address);
+      station->m_prevPowerLevel = station->m_powerLevel;
+    }
+  if (station->m_prevRateIndex != station->m_rateIndex)
+    {
+      m_rateChange (prevRate, rate, station->m_state->m_address);
+      station->m_prevRateIndex = station->m_rateIndex;
+    }
+  return WifiTxVector (mode, station->m_powerLevel, GetLongRetryCount (station), GetPreambleForTransmission (mode, GetAddress (st)), 800, 1, 1, 0, channelWidth, GetAggregation (station), false);
 }
 
 WifiTxVector
@@ -346,14 +360,16 @@ AparfWifiManager::DoGetRtsTxVector (WifiRemoteStation *st)
       channelWidth = 20;
     }
   WifiTxVector rtsTxVector;
+  WifiMode mode;
   if (GetUseNonErpProtection () == false)
     {
-      rtsTxVector = WifiTxVector (GetSupported (station, 0), GetDefaultTxPowerLevel (), GetShortRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+      mode = GetSupported (station, 0);
     }
   else
     {
-      rtsTxVector = WifiTxVector (GetNonErpSupported (station, 0), GetDefaultTxPowerLevel (), GetShortRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+      mode = GetNonErpSupported (station, 0);
     }
+  rtsTxVector = WifiTxVector (mode, GetDefaultTxPowerLevel (), GetShortRetryCount (station), GetPreambleForTransmission (mode, GetAddress (st)), 800, 1, 1, 0, channelWidth, GetAggregation (station), false);
   return rtsTxVector;
 }
 
@@ -384,4 +400,14 @@ AparfWifiManager::SetVhtSupported (bool enable)
     }
 }
 
+void
+AparfWifiManager::SetHeSupported (bool enable)
+{
+  //HE is not supported by this algorithm.
+  if (enable)
+    {
+      NS_FATAL_ERROR ("WifiRemoteStationManager selected does not support HE rates");
+    }
+}
+
 } //namespace ns3
