@@ -21,22 +21,17 @@
 #ifndef BLOCK_ACK_MANAGER_H
 #define BLOCK_ACK_MANAGER_H
 
-#include <map>
-#include <list>
-#include <deque>
-#include "ns3/packet.h"
 #include "wifi-mac-header.h"
 #include "originator-block-ack-agreement.h"
 #include "ctrl-headers.h"
 #include "qos-utils.h"
-#include "wifi-mode.h"
 #include "wifi-remote-station-manager.h"
+#include <map>
 
 namespace ns3 {
 
 class MgtAddBaResponseHeader;
 class MgtAddBaRequestHeader;
-class MgtDelBaHeader;
 class MacTxMiddle;
 class WifiMacQueue;
 
@@ -61,10 +56,10 @@ struct Bar
        Mac48Address recipient,
        uint8_t tid,
        bool immediate);
-  Ptr<const Packet> bar;
-  Mac48Address recipient;
-  uint8_t tid;
-  bool immediate;
+  Ptr<const Packet> bar; ///< block ack request
+  Mac48Address recipient; ///< recipient
+  uint8_t tid; ///< TID
+  bool immediate; ///< immediate
 };
 
 
@@ -72,14 +67,26 @@ struct Bar
  * \brief Manages all block ack agreements for an originator station.
  * \ingroup wifi
  */
-class BlockAckManager
+class BlockAckManager : public Object
 {
 private:
+  /// type conversion operator
   BlockAckManager (const BlockAckManager&);
-  BlockAckManager& operator= (const BlockAckManager&);
+  /**
+   * assignment operator
+   * \param block BlockAckManager to assign
+   * \returns the assigned BlockAckManager
+   */
+  BlockAckManager& operator= (const BlockAckManager& block);
 
 
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
+  static TypeId GetTypeId (void);
+
   BlockAckManager ();
   ~BlockAckManager ();
 
@@ -88,7 +95,7 @@ public:
    *
    * \param manager WifiRemoteStationManager associated with this BlockAckManager
    */
-  void SetWifiRemoteStationManager (Ptr<WifiRemoteStationManager> manager);
+  void SetWifiRemoteStationManager (const Ptr<WifiRemoteStationManager> manager);
   /**
    * \param recipient Address of peer station involved in block ack mechanism.
    * \param tid Traffic ID.
@@ -110,7 +117,7 @@ public:
    * station addressed by <i>recipient</i> for tid <i>tid</i>.
    */
   bool ExistsAgreementInState (Mac48Address recipient, uint8_t tid,
-                               enum OriginatorBlockAckAgreement::State state) const;
+                               OriginatorBlockAckAgreement::State state) const;
   /**
    * \param reqHdr Relative Add block ack request (action frame).
    * \param recipient Address of peer station involved in block ack mechanism.
@@ -161,7 +168,14 @@ public:
    * corresponding block ack bitmap. This method doesn't remove the packet from this queue.
    */
   Ptr<const Packet> PeekNextPacket (WifiMacHeader &hdr);
-  bool HasBar (struct Bar &bar);
+  /**
+   * Returns true if the BAR is scheduled. Returns false otherwise.
+   *
+   * \param bar
+   *
+   * \return true if a BAR is scheduled, false otherwise
+   */
+  bool HasBar (Bar &bar);
   /**
    * Returns true if there are packets that need of retransmission or at least a
    * BAR is scheduled. Returns false otherwise.
@@ -250,15 +264,19 @@ public:
   /**
    * \param queue The WifiMacQueue object.
    */
-  void SetQueue (Ptr<WifiMacQueue> queue);
-  void SetTxMiddle (MacTxMiddle* txMiddle);
+  void SetQueue (const Ptr<WifiMacQueue> queue);
+  /**
+   * Set the MacTxMiddle
+   * \param txMiddle the MacTxMiddle
+   */
+  void SetTxMiddle (const Ptr<MacTxMiddle> txMiddle);
 
   /**
    * \param bAckType Type of block ack
    *
    * See ctrl-headers.h for more details.
    */
-  void SetBlockAckType (enum BlockAckType bAckType);
+  void SetBlockAckType (BlockAckType bAckType);
   /**
    * \param recipient Address of station involved in block ack mechanism.
    * \param tid Traffic ID.
@@ -291,13 +309,25 @@ public:
    */
   void SetMaxPacketDelay (Time maxDelay);
 
+  /**
+   * Set block ack inactivity callback
+   * \param callback the block ack inactivity callback function
+   */
   void SetBlockAckInactivityCallback (Callback<void, Mac48Address, uint8_t, bool> callback);
+  /**
+   * Set block destination callback
+   * \param callback the block destination callback
+   */
   void SetBlockDestinationCallback (Callback<void, Mac48Address, uint8_t> callback);
+  /**
+   * Set unblock destination callback
+   * \param callback the unblock destination callback
+   */
   void SetUnblockDestinationCallback (Callback<void, Mac48Address, uint8_t> callback);
 
   /**
-   * \param recipient
-   * \param tid
+   * \param recipient the destination address
+   * \param tid the Traffic ID
    * \param startingSeq
    *
    * \return true if there are packets in the queue that could be sent under block ACK,
@@ -310,8 +340,8 @@ public:
    */
   bool SwitchToBlockAckIfNeeded (Mac48Address recipient, uint8_t tid, uint16_t startingSeq);
   /**
-   * \param recipient
-   * \param tid
+   * \param recipient the destination address
+   * \param tid the Traffic ID
    *
    * \return the sequence number of the next retry packet for a specific agreement
    *
@@ -322,21 +352,43 @@ public:
   uint16_t GetSeqNumOfNextRetryPacket (Mac48Address recipient, uint8_t tid) const;
   /**
    * Checks if the packet already exists in the retransmit queue or not if it does then it doesn't add it again
+   *
+   * \param currentSeq the current sequence
+   * \param recipient the destination address
+   * \param tid the Traffic ID
+   * \returns true if the packet already exists
    */
-  bool AlreadyExists (uint16_t currentSeq, Mac48Address recipient, uint8_t tid);
+  bool AlreadyExists (uint16_t currentSeq, Mac48Address recipient, uint8_t tid) const;
   /**
    * Remove a packet after you peek in the queue and get it
+   * \param tid the Traffic ID
+   * \param recipient the destination address
+   * \param seqnumber sequence number
+   * \returns true if a packet was removed
    */
   bool RemovePacket (uint8_t tid, Mac48Address recipient, uint16_t seqnumber);
-  /*
+  /**
    * Peek in retransmit queue and get the next packet having address indicated
    * by <i>type</i> equals to <i>addr</i>, and tid equals to <i>tid</i>.
    * This method doesn't remove the packet from this queue.
+   *
+   * \param hdr wifi mac header
+   * \param recipient mac address
+   * \param tid Traffic ID
+   * \param timestamp timestamp
+   *
+   * \returns Ptr<const Packet>
    */
   Ptr<const Packet> PeekNextPacketByTidAndAddress (WifiMacHeader &hdr, Mac48Address recipient, uint8_t tid, Time *timestamp);
   /**
    * This function returns true if the lifetime of the packets a BAR refers to didn't expire yet else it returns false.
    * If it return false then the BAR will be discarded (i.e. will not be re-transmitted)
+   *
+   * \param tid Traffic ID
+   * \param seqNumber sequence number
+   * \param recipient MAC address
+   *
+   * \returns true if BAR retransmission needed
    */
   bool NeedBarRetransmission (uint8_t tid, uint16_t seqNumber, Mac48Address recipient);
 
@@ -380,7 +432,12 @@ private:
    * This method removes packets whose lifetime was exceeded.
    */
   void CleanupBuffers (void);
-  void InactivityTimeout (Mac48Address, uint8_t);
+  /**
+   * Inactivity timeout function
+   * \param recipient the recipient MAC address
+   * \param tid Traffic ID
+   */
+  void InactivityTimeout (Mac48Address recipient, uint8_t tid);
 
   struct Item;
   /**
@@ -418,12 +475,19 @@ private:
   struct Item
   {
     Item ();
+    /**
+     * Constructor
+     *
+     * \param packet packet
+     * \param hdr packet header
+     * \param tStamp timestamp
+     */
     Item (Ptr<const Packet> packet,
           const WifiMacHeader &hdr,
           Time tStamp);
-    Ptr<const Packet> packet;
-    WifiMacHeader hdr;
-    Time timestamp;
+    Ptr<const Packet> packet; ///< packet
+    WifiMacHeader hdr; ///< header
+    Time timestamp; ///< timestamp
   };
   /**
    * \param item
@@ -433,6 +497,16 @@ private:
    */
   void InsertInRetryQueue (PacketQueueI item);
 
+  /**
+   * Remove items from retransmission queue.
+   * This method should be called when packets are acknowledged.
+   *
+   * \param address recipient mac address of the packet to be removed
+   * \param tid Traffic ID of the packet to be removed
+   * \param seq sequence number of the packet to be removed
+   */
+  void RemoveFromRetryQueue (Mac48Address address, uint8_t tid, uint16_t seq);
+
   /**
    * This data structure contains, for each block ack agreement (recipient, tid), a set of packets
    * for which an ack by block ack is requested.
@@ -447,20 +521,20 @@ private:
    * frame.
    */
   std::list<PacketQueueI> m_retryPackets;
-  std::list<Bar> m_bars;
+  std::list<Bar> m_bars; ///< list of BARs
 
-  uint8_t m_blockAckThreshold;
-  enum BlockAckType m_blockAckType;
-  Time m_maxDelay;
-  MacTxMiddle* m_txMiddle;
-  Mac48Address m_address;
-  Ptr<WifiMacQueue> m_queue;
-  Callback<void, Mac48Address, uint8_t, bool> m_blockAckInactivityTimeout;
-  Callback<void, Mac48Address, uint8_t> m_blockPackets;
-  Callback<void, Mac48Address, uint8_t> m_unblockPackets;
-  TxOk m_txOkCallback;
-  TxFailed m_txFailedCallback;
-  Ptr<WifiRemoteStationManager> m_stationManager;
+  uint8_t m_blockAckThreshold; ///< bock ack threshold
+  BlockAckType m_blockAckType; ///< bock ack type
+  Time m_maxDelay; ///< maximum delay
+  Ptr<MacTxMiddle> m_txMiddle; ///< the MacTxMiddle
+  Mac48Address m_address; ///< address
+  Ptr<WifiMacQueue> m_queue; ///< queue
+  Callback<void, Mac48Address, uint8_t, bool> m_blockAckInactivityTimeout; ///< block ack inactivity timeout callback
+  Callback<void, Mac48Address, uint8_t> m_blockPackets; ///< block packets callback
+  Callback<void, Mac48Address, uint8_t> m_unblockPackets; ///< unblock packets callback
+  TxOk m_txOkCallback; ///< transmit ok callback
+  TxFailed m_txFailedCallback; ///< transmit failed callback
+  Ptr<WifiRemoteStationManager> m_stationManager; ///< the station manager
 };
 
 } //namespace ns3
