@@ -18,21 +18,13 @@
  * Author: Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
  */
 
-#include "ns3/assert.h"
-#include "ns3/packet.h"
 #include "ns3/log.h"
-#include "ns3/simulator.h"
-#include "ns3/node.h"
-#include "ns3/uinteger.h"
 #include "ns3/pointer.h"
 #include "dca-txop.h"
 #include "dcf-manager.h"
-#include "mac-low.h"
+#include "dcf-state.h"
 #include "wifi-mac-queue.h"
 #include "mac-tx-middle.h"
-#include "wifi-mac-trailer.h"
-#include "wifi-mac.h"
-#include "random-stream.h"
 
 #undef NS_LOG_APPEND_CONTEXT
 #define NS_LOG_APPEND_CONTEXT if (m_low != 0) { std::clog << "[mac=" << m_low->GetAddress () << "] "; }
@@ -41,113 +33,35 @@ namespace ns3 {
 
 NS_LOG_COMPONENT_DEFINE ("DcaTxop");
 
-class DcaTxop::Dcf : public DcfState
-{
-public:
-  Dcf (DcaTxop * txop)
-    : m_txop (txop)
-  {
-  }
-  virtual bool IsEdca (void) const
-  {
-    return false;
-  }
-private:
-  virtual void DoNotifyAccessGranted (void)
-  {
-    m_txop->NotifyAccessGranted ();
-  }
-  virtual void DoNotifyInternalCollision (void)
-  {
-    m_txop->NotifyInternalCollision ();
-  }
-  virtual void DoNotifyCollision (void)
-  {
-    m_txop->NotifyCollision ();
-  }
-  virtual void DoNotifyChannelSwitching (void)
-  {
-    m_txop->NotifyChannelSwitching ();
-  }
-  virtual void DoNotifySleep (void)
-  {
-    m_txop->NotifySleep ();
-  }
-  virtual void DoNotifyWakeUp (void)
-  {
-    m_txop->NotifyWakeUp ();
-  }
-
-  DcaTxop *m_txop;
-};
-
-
-/**
- * Listener for MacLow events. Forwards to DcaTxop.
- */
-class DcaTxop::TransmissionListener : public MacLowTransmissionListener
-{
-public:
-  /**
-   * Create a TransmissionListener for the given DcaTxop.
-   *
-   * \param txop
-   */
-  TransmissionListener (DcaTxop * txop)
-    : MacLowTransmissionListener (),
-      m_txop (txop)
-  {
-  }
-
-  virtual ~TransmissionListener ()
-  {
-  }
-
-  virtual void GotCts (double snr, WifiMode txMode)
-  {
-    m_txop->GotCts (snr, txMode);
-  }
-  virtual void MissedCts (void)
-  {
-    m_txop->MissedCts ();
-  }
-  virtual void GotAck (double snr, WifiMode txMode)
-  {
-    m_txop->GotAck (snr, txMode);
-  }
-  virtual void MissedAck (void)
-  {
-    m_txop->MissedAck ();
-  }
-  virtual void StartNextFragment (void)
-  {
-    m_txop->StartNextFragment ();
-  }
-  virtual void StartNext (void)
-  {
-  }
-  virtual void Cancel (void)
-  {
-    m_txop->Cancel ();
-  }
-  virtual void EndTxNoAck (void)
-  {
-    m_txop->EndTxNoAck ();
-  }
-
-private:
-  DcaTxop *m_txop;
-};
-
 NS_OBJECT_ENSURE_REGISTERED (DcaTxop);
 
 TypeId
 DcaTxop::GetTypeId (void)
 {
   static TypeId tid = TypeId ("ns3::DcaTxop")
-    .SetParent<ns3::Dcf> ()
+    .SetParent<ns3::Object> ()
     .SetGroupName ("Wifi")
     .AddConstructor<DcaTxop> ()
+    .AddAttribute ("MinCw", "The minimum value of the contention window.",
+                   UintegerValue (15),
+                   MakeUintegerAccessor (&DcaTxop::SetMinCw,
+                                         &DcaTxop::GetMinCw),
+                   MakeUintegerChecker<uint32_t> ())
+    .AddAttribute ("MaxCw", "The maximum value of the contention window.",
+                   UintegerValue (1023),
+                   MakeUintegerAccessor (&DcaTxop::SetMaxCw,
+                                         &DcaTxop::GetMaxCw),
+                   MakeUintegerChecker<uint32_t> ())
+    .AddAttribute ("Aifsn", "The AIFSN: the default value conforms to simple DCA.",
+                   UintegerValue (2),
+                   MakeUintegerAccessor (&DcaTxop::SetAifsn,
+                                         &DcaTxop::GetAifsn),
+                   MakeUintegerChecker<uint32_t> ())
+    .AddAttribute ("TxopLimit", "The TXOP limit: the default value conforms to simple DCA.",
+                   TimeValue (MilliSeconds (0)),
+                   MakeTimeAccessor (&DcaTxop::SetTxopLimit,
+                                     &DcaTxop::GetTxopLimit),
+                   MakeTimeChecker ())
     .AddAttribute ("Queue", "The WifiMacQueue object",
                    PointerValue (),
                    MakePointerAccessor (&DcaTxop::GetQueue),
@@ -161,10 +75,9 @@ DcaTxop::DcaTxop ()
     m_currentPacket (0)
 {
   NS_LOG_FUNCTION (this);
-  m_transmissionListener = new DcaTxop::TransmissionListener (this);
-  m_dcf = new DcaTxop::Dcf (this);
+  m_dcf = CreateObject<DcfState> (this);
   m_queue = CreateObject<WifiMacQueue> ();
-  m_rng = new RealRandomStream ();
+  m_rng = CreateObject<UniformRandomVariable> ();
 }
 
 DcaTxop::~DcaTxop ()
@@ -179,37 +92,34 @@ DcaTxop::DoDispose (void)
   m_queue = 0;
   m_low = 0;
   m_stationManager = 0;
-  delete m_transmissionListener;
-  delete m_dcf;
-  delete m_rng;
-  m_transmissionListener = 0;
   m_dcf = 0;
   m_rng = 0;
   m_txMiddle = 0;
 }
 
 void
-DcaTxop::SetManager (DcfManager *manager)
+DcaTxop::SetManager (const Ptr<DcfManager> manager)
 {
   NS_LOG_FUNCTION (this << manager);
   m_manager = manager;
   m_manager->Add (m_dcf);
 }
 
-void DcaTxop::SetTxMiddle (MacTxMiddle *txMiddle)
+void DcaTxop::SetTxMiddle (const Ptr<MacTxMiddle> txMiddle)
 {
+  NS_LOG_FUNCTION (this);
   m_txMiddle = txMiddle;
 }
 
 void
-DcaTxop::SetLow (Ptr<MacLow> low)
+DcaTxop::SetLow (const Ptr<MacLow> low)
 {
   NS_LOG_FUNCTION (this << low);
   m_low = low;
 }
 
 void
-DcaTxop::SetWifiRemoteStationManager (Ptr<WifiRemoteStationManager> remoteManager)
+DcaTxop::SetWifiRemoteStationManager (const Ptr<WifiRemoteStationManager> remoteManager)
 {
   NS_LOG_FUNCTION (this << remoteManager);
   m_stationManager = remoteManager;
@@ -229,6 +139,23 @@ DcaTxop::SetTxFailedCallback (TxFailed callback)
   m_txFailedCallback = callback;
 }
 
+void
+DcaTxop::SetTxDroppedCallback (TxDropped callback)
+{
+  NS_LOG_FUNCTION (this << &callback);
+  m_txDroppedCallback = callback;
+  m_queue->TraceConnectWithoutContext ("Drop", MakeCallback (&DcaTxop::TxDroppedPacket, this));
+}
+
+void
+DcaTxop::TxDroppedPacket (Ptr<const WifiMacQueueItem> item)
+{
+  if (!m_txDroppedCallback.IsNull ())
+    {
+      m_txDroppedCallback (item->GetPacket ());
+    }
+}
+
 Ptr<WifiMacQueue >
 DcaTxop::GetQueue () const
 {
@@ -267,28 +194,24 @@ DcaTxop::SetTxopLimit (Time txopLimit)
 uint32_t
 DcaTxop::GetMinCw (void) const
 {
-  NS_LOG_FUNCTION (this);
   return m_dcf->GetCwMin ();
 }
 
 uint32_t
 DcaTxop::GetMaxCw (void) const
 {
-  NS_LOG_FUNCTION (this);
   return m_dcf->GetCwMax ();
 }
 
 uint32_t
 DcaTxop::GetAifsn (void) const
 {
-  NS_LOG_FUNCTION (this);
   return m_dcf->GetAifsn ();
 }
 
 Time
 DcaTxop::GetTxopLimit (void) const
 {
-  NS_LOG_FUNCTION (this);
   return m_dcf->GetTxopLimit ();
 }
 
@@ -296,9 +219,8 @@ void
 DcaTxop::Queue (Ptr<const Packet> packet, const WifiMacHeader &hdr)
 {
   NS_LOG_FUNCTION (this << packet << &hdr);
-  WifiMacTrailer fcs;
   m_stationManager->PrepareForQueue (hdr.GetAddr1 (), &hdr, packet);
-  m_queue->Enqueue (packet, hdr);
+  m_queue->Enqueue (Create<WifiMacQueueItem> (packet, hdr));
   StartAccessIfNeeded ();
 }
 
@@ -306,7 +228,7 @@ int64_t
 DcaTxop::AssignStreams (int64_t stream)
 {
   NS_LOG_FUNCTION (this << stream);
-  m_rng->AssignStreams (stream);
+  m_rng->SetStream (stream);
   return 1;
 }
 
@@ -335,7 +257,7 @@ DcaTxop::StartAccessIfNeeded (void)
 }
 
 Ptr<MacLow>
-DcaTxop::Low (void)
+DcaTxop::GetLow (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_low;
@@ -346,28 +268,25 @@ DcaTxop::DoInitialize ()
 {
   NS_LOG_FUNCTION (this);
   m_dcf->ResetCw ();
-  m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
-  ns3::Dcf::DoInitialize ();
+  m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
 }
 
 bool
-DcaTxop::NeedRtsRetransmission (void)
+DcaTxop::NeedRtsRetransmission (Ptr<const Packet> packet, const WifiMacHeader &hdr)
 {
   NS_LOG_FUNCTION (this);
-  return m_stationManager->NeedRtsRetransmission (m_currentHdr.GetAddr1 (), &m_currentHdr,
-                                                  m_currentPacket);
+  return m_stationManager->NeedRtsRetransmission (hdr.GetAddr1 (), &hdr, packet);
 }
 
 bool
-DcaTxop::NeedDataRetransmission (void)
+DcaTxop::NeedDataRetransmission (Ptr<const Packet> packet, const WifiMacHeader &hdr)
 {
   NS_LOG_FUNCTION (this);
-  return m_stationManager->NeedDataRetransmission (m_currentHdr.GetAddr1 (), &m_currentHdr,
-                                                   m_currentPacket);
+  return m_stationManager->NeedDataRetransmission (hdr.GetAddr1 (), &hdr, packet);
 }
 
 bool
-DcaTxop::NeedFragmentation (void)
+DcaTxop::NeedFragmentation (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_stationManager->NeedFragmentation (m_currentHdr.GetAddr1 (), &m_currentHdr,
@@ -382,7 +301,7 @@ DcaTxop::NextFragment (void)
 }
 
 uint32_t
-DcaTxop::GetFragmentSize (void)
+DcaTxop::GetFragmentSize (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_stationManager->GetFragmentSize (m_currentHdr.GetAddr1 (), &m_currentHdr,
@@ -390,7 +309,7 @@ DcaTxop::GetFragmentSize (void)
 }
 
 bool
-DcaTxop::IsLastFragment (void)
+DcaTxop::IsLastFragment (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_stationManager->IsLastFragment (m_currentHdr.GetAddr1 (), &m_currentHdr,
@@ -398,7 +317,7 @@ DcaTxop::IsLastFragment (void)
 }
 
 uint32_t
-DcaTxop::GetNextFragmentSize (void)
+DcaTxop::GetNextFragmentSize (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_stationManager->GetFragmentSize (m_currentHdr.GetAddr1 (), &m_currentHdr,
@@ -406,7 +325,7 @@ DcaTxop::GetNextFragmentSize (void)
 }
 
 uint32_t
-DcaTxop::GetFragmentOffset (void)
+DcaTxop::GetFragmentOffset (void) const
 {
   NS_LOG_FUNCTION (this);
   return m_stationManager->GetFragmentOffset (m_currentHdr.GetAddr1 (), &m_currentHdr,
@@ -440,6 +359,7 @@ DcaTxop::NeedsAccess (void) const
   NS_LOG_FUNCTION (this);
   return !m_queue->IsEmpty () || m_currentPacket != 0;
 }
+
 void
 DcaTxop::NotifyAccessGranted (void)
 {
@@ -451,9 +371,12 @@ DcaTxop::NotifyAccessGranted (void)
           NS_LOG_DEBUG ("queue empty");
           return;
         }
-      m_currentPacket = m_queue->Dequeue (&m_currentHdr);
+      Ptr<WifiMacQueueItem> item = m_queue->Dequeue ();
+      NS_ASSERT (item != 0);
+      m_currentPacket = item->GetPacket ();
+      m_currentHdr = item->GetHeader ();
       NS_ASSERT (m_currentPacket != 0);
-      uint16_t sequence = m_txMiddle->GetNextSequenceNumberfor (&m_currentHdr);
+      uint16_t sequence = m_txMiddle->GetNextSequenceNumberFor (&m_currentHdr);
       m_currentHdr.SetSequenceNumber (sequence);
       m_stationManager->UpdateFragmentationThreshold ();
       m_currentHdr.SetFragmentNumber (0);
@@ -464,22 +387,18 @@ DcaTxop::NotifyAccessGranted (void)
                     ", to=" << m_currentHdr.GetAddr1 () <<
                     ", seq=" << m_currentHdr.GetSequenceControl ());
     }
-  MacLowTransmissionParameters params;
-  params.DisableOverrideDurationId ();
+  m_currentParams.DisableOverrideDurationId ();
   if (m_currentHdr.GetAddr1 ().IsGroup ())
     {
-      params.DisableRts ();
-      params.DisableAck ();
-      params.DisableNextData ();
-      Low ()->StartTransmission (m_currentPacket,
-                                 &m_currentHdr,
-                                 params,
-                                 m_transmissionListener);
+      m_currentParams.DisableRts ();
+      m_currentParams.DisableAck ();
+      m_currentParams.DisableNextData ();
+      GetLow ()->StartTransmission (m_currentPacket, &m_currentHdr, m_currentParams, this);
       NS_LOG_DEBUG ("tx broadcast");
     }
   else
     {
-      params.EnableAck ();
+      m_currentParams.EnableAck ();
 
       if (NeedFragmentation ())
         {
@@ -488,21 +407,19 @@ DcaTxop::NotifyAccessGranted (void)
           if (IsLastFragment ())
             {
               NS_LOG_DEBUG ("fragmenting last fragment size=" << fragment->GetSize ());
-              params.DisableNextData ();
+              m_currentParams.DisableNextData ();
             }
           else
             {
               NS_LOG_DEBUG ("fragmenting size=" << fragment->GetSize ());
-              params.EnableNextData (GetNextFragmentSize ());
+              m_currentParams.EnableNextData (GetNextFragmentSize ());
             }
-          Low ()->StartTransmission (fragment, &hdr, params,
-                                     m_transmissionListener);
+          GetLow ()->StartTransmission (fragment, &hdr, m_currentParams, this);
         }
       else
         {
-          params.DisableNextData ();
-          Low ()->StartTransmission (m_currentPacket, &m_currentHdr,
-                                     params, m_transmissionListener);
+          m_currentParams.DisableNextData ();
+          GetLow ()->StartTransmission (m_currentPacket, &m_currentHdr, m_currentParams, this);
         }
     }
 }
@@ -518,7 +435,7 @@ void
 DcaTxop::NotifyCollision (void)
 {
   NS_LOG_FUNCTION (this);
-  m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
+  m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
   RestartAccessIfNeeded ();
 }
 
@@ -536,7 +453,7 @@ DcaTxop::NotifySleep (void)
   NS_LOG_FUNCTION (this);
   if (m_currentPacket != 0)
     {
-      m_queue->PushFront (m_currentPacket, m_currentHdr);
+      m_queue->PushFront (Create<WifiMacQueueItem> (m_currentPacket, m_currentHdr));
       m_currentPacket = 0;
     }
 }
@@ -548,19 +465,12 @@ DcaTxop::NotifyWakeUp (void)
   RestartAccessIfNeeded ();
 }
 
-void
-DcaTxop::GotCts (double snr, WifiMode txMode)
-{
-  NS_LOG_FUNCTION (this << snr << txMode);
-  NS_LOG_DEBUG ("got cts");
-}
-
 void
 DcaTxop::MissedCts (void)
 {
   NS_LOG_FUNCTION (this);
   NS_LOG_DEBUG ("missed cts");
-  if (!NeedRtsRetransmission ())
+  if (!NeedRtsRetransmission (m_currentPacket, m_currentHdr))
     {
       NS_LOG_DEBUG ("Cts Fail");
       m_stationManager->ReportFinalRtsFailed (m_currentHdr.GetAddr1 (), &m_currentHdr);
@@ -576,14 +486,14 @@ DcaTxop::MissedCts (void)
     {
       m_dcf->UpdateFailedCw ();
     }
-  m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
+  m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
   RestartAccessIfNeeded ();
 }
 
 void
-DcaTxop::GotAck (double snr, WifiMode txMode)
+DcaTxop::GotAck (void)
 {
-  NS_LOG_FUNCTION (this << snr << txMode);
+  NS_LOG_FUNCTION (this);
   if (!NeedFragmentation ()
       || IsLastFragment ())
     {
@@ -598,7 +508,7 @@ DcaTxop::GotAck (double snr, WifiMode txMode)
        */
       m_currentPacket = 0;
       m_dcf->ResetCw ();
-      m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
+      m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
       RestartAccessIfNeeded ();
     }
   else
@@ -612,7 +522,7 @@ DcaTxop::MissedAck (void)
 {
   NS_LOG_FUNCTION (this);
   NS_LOG_DEBUG ("missed ack");
-  if (!NeedDataRetransmission ())
+  if (!NeedDataRetransmission (m_currentPacket, m_currentHdr))
     {
       NS_LOG_DEBUG ("Ack Fail");
       m_stationManager->ReportFinalDataFailed (m_currentHdr.GetAddr1 (), &m_currentHdr);
@@ -630,7 +540,7 @@ DcaTxop::MissedAck (void)
       m_currentHdr.SetRetry ();
       m_dcf->UpdateFailedCw ();
     }
-  m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
+  m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
   RestartAccessIfNeeded ();
 }
 
@@ -643,19 +553,18 @@ DcaTxop::StartNextFragment (void)
   NextFragment ();
   WifiMacHeader hdr;
   Ptr<Packet> fragment = GetFragmentPacket (&hdr);
-  MacLowTransmissionParameters params;
-  params.EnableAck ();
-  params.DisableRts ();
-  params.DisableOverrideDurationId ();
+  m_currentParams.EnableAck ();
+  m_currentParams.DisableRts ();
+  m_currentParams.DisableOverrideDurationId ();
   if (IsLastFragment ())
     {
-      params.DisableNextData ();
+      m_currentParams.DisableNextData ();
     }
   else
     {
-      params.EnableNextData (GetNextFragmentSize ());
+      m_currentParams.EnableNextData (GetNextFragmentSize ());
     }
-  Low ()->StartTransmission (fragment, &hdr, params, m_transmissionListener);
+  GetLow ()->StartTransmission (fragment, &hdr, m_currentParams, this);
 }
 
 void
@@ -663,31 +572,6 @@ DcaTxop::Cancel (void)
 {
   NS_LOG_FUNCTION (this);
   NS_LOG_DEBUG ("transmission cancelled");
-  /**
-   * This happens in only one case: in an AP, you have two DcaTxop:
-   *   - one is used exclusively for beacons and has a high priority.
-   *   - the other is used for everything else and has a normal
-   *     priority.
-   *
-   * If the normal queue tries to send a unicast data frame, but
-   * if the tx fails (ack timeout), it starts a backoff. If the beacon
-   * queue gets a tx oportunity during this backoff, it will trigger
-   * a call to this Cancel function.
-   *
-   * Since we are already doing a backoff, we will get access to
-   * the medium when we can, we have nothing to do here. We just
-   * ignore the cancel event and wait until we are given again a
-   * tx oportunity.
-   *
-   * Note that this is really non-trivial because each of these
-   * frames is assigned a sequence number from the same sequence
-   * counter (because this is a non-802.11e device) so, the scheme
-   * described here fails to ensure in-order delivery of frames
-   * at the receiving side. This, however, does not matter in
-   * this case because we assume that the receiving side does not
-   * update its <seq,ad> tupple for packets whose destination
-   * address is a broadcast address.
-   */
 }
 
 void
@@ -697,8 +581,38 @@ DcaTxop::EndTxNoAck (void)
   NS_LOG_DEBUG ("a transmission that did not require an ACK just finished");
   m_currentPacket = 0;
   m_dcf->ResetCw ();
-  m_dcf->StartBackoffNow (m_rng->GetNext (0, m_dcf->GetCw ()));
+  m_dcf->StartBackoffNow (m_rng->GetInteger (0, m_dcf->GetCw ()));
   StartAccessIfNeeded ();
 }
 
+bool
+DcaTxop::IsEdca ()
+{
+  return false;
+}
+
+void
+DcaTxop::StartNextPacket (void)
+{
+  NS_LOG_WARN ("StartNext should not be called for non QoS!");
+}
+
+void
+DcaTxop::GotBlockAck (const CtrlBAckResponseHeader *blockAck, Mac48Address recipient, double rxSnr, WifiMode txMode, double dataSnr)
+{
+  NS_LOG_WARN ("GotBlockAck should not be called for non QoS!");
+}
+
+void
+DcaTxop::MissedBlockAck (uint8_t nMpdus)
+{
+  NS_LOG_WARN ("MissedBlockAck should not be called for non QoS!");
+}
+
+bool
+DcaTxop::HasTxop (void) const
+{
+  return false;
+}
+
 } //namespace ns3
