@@ -23,19 +23,18 @@
 #ifndef MGT_HEADERS_H
 #define MGT_HEADERS_H
 
-#include <stdint.h>
-
-#include "ns3/header.h"
 #include "status-code.h"
 #include "capability-information.h"
 #include "supported-rates.h"
 #include "ssid.h"
 #include "dsss-parameter-set.h"
 #include "ht-capabilities.h"
-#include "ht-operations.h"
+#include "ht-operation.h"
 #include "vht-capabilities.h"
+#include "vht-operation.h"
 #include "erp-information.h"
 #include "edca-parameter-set.h"
+#include "he-capabilities.h"
 
 namespace ns3 {
 
@@ -85,6 +84,12 @@ public:
    * \param vhtcapabilities VHT capabilities
    */
   void SetVhtCapabilities (VhtCapabilities vhtcapabilities);
+  /**
+   * Set the HE capabilities.
+   *
+   * \param hecapabilities HE capabilities
+   */
+  void SetHeCapabilities (HeCapabilities hecapabilities);
   /**
    * Return the Capability information.
    *
@@ -103,6 +108,12 @@ public:
    * \return VHT capabilities
    */
   VhtCapabilities GetVhtCapabilities (void) const;
+  /**
+   * Return the HE capabilities.
+   *
+   * \return HE capabilities
+   */
+  HeCapabilities GetHeCapabilities (void) const;
   /**
    * Return the Service Set Identifier (SSID).
    *
@@ -127,11 +138,11 @@ public:
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
 
 private:
@@ -140,7 +151,8 @@ private:
   CapabilityInformation m_capability; //!< Capability information
   HtCapabilities m_htCapability;      //!< HT capabilities
   VhtCapabilities m_vhtCapability;    //!< VHT capabilities
-  uint16_t m_listenInterval;
+  HeCapabilities m_heCapability;      //!< HE capabilities
+  uint16_t m_listenInterval;          //!< listen interval
 };
 
 
@@ -172,12 +184,6 @@ public:
    * \return Capability information
    */
   CapabilityInformation GetCapabilities (void) const;
-  /**
-   * Set the Capability information.
-   *
-   * \param capabilities Capability information
-   */
-  void SetCapabilities (CapabilityInformation capabilities);
   /**
    * Return the HT capabilities.
    *
@@ -185,11 +191,29 @@ public:
    */
   HtCapabilities GetHtCapabilities (void) const;
   /**
-   * Return the HT operations.
+   * Return the HT operation.
    *
-   * \return HT operations
+   * \return HT operation
    */
-  HtOperations GetHtOperations (void) const;
+  HtOperation GetHtOperation (void) const;
+  /**
+   * Return the VHT capabilities.
+   *
+   * \return VHT capabilities
+   */
+  VhtCapabilities GetVhtCapabilities (void) const;
+  /**
+   * Return the VHT operation.
+   *
+   * \return VHT operation
+   */
+  VhtOperation GetVhtOperation (void) const;
+  /**
+   * Return the HE capabilities.
+   *
+   * \return HE capabilities
+   */
+  HeCapabilities GetHeCapabilities (void) const;
   /**
    * Return the ERP information.
    *
@@ -202,18 +226,24 @@ public:
    * \return the EDCA Parameter Set
    */
   EdcaParameterSet GetEdcaParameterSet (void) const;
+  /**
+   * Set the Capability information.
+   *
+   * \param capabilities Capability information
+   */
+  void SetCapabilities (CapabilityInformation capabilities);
+  /**
+   * Set the VHT operation.
+   *
+   * \param vhtoperation VHT operation
+   */
+  void SetVhtOperation (VhtOperation vhtoperation);
   /**
    * Set the VHT capabilities.
    *
    * \param vhtcapabilities VHT capabilities
    */
   void SetVhtCapabilities (VhtCapabilities vhtcapabilities);
-  /**
-   * Return the VHT capabilities.
-   *
-   * \return VHT capabilities
-   */
-  VhtCapabilities GetVhtCapabilities (void) const;
   /**
    * Set the HT capabilities.
    *
@@ -221,11 +251,11 @@ public:
    */
   void SetHtCapabilities (HtCapabilities htcapabilities);
   /**
-   * Set the HT operations.
+   * Set the HT operation.
    *
-   * \param htoperations HT operations
+   * \param htoperation HT operation
    */
-  void SetHtOperations (HtOperations htoperations);
+  void SetHtOperation (HtOperation htoperation);
   /**
    * Set the supported rates.
    *
@@ -250,29 +280,37 @@ public:
    * \param edcaParameterSet the EDCA Parameter Set
    */
   void SetEdcaParameterSet (EdcaParameterSet edcaParameterSet);
+  /**
+   * Set the HE capabilities.
+   *
+   * \param hecapabilities HE capabilities
+   */
+  void SetHeCapabilities (HeCapabilities hecapabilities);
 
   /**
    * Register this type.
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
 
 private:
   SupportedRates m_rates; //!< List of supported rates
   CapabilityInformation m_capability; //!< Capability information
   StatusCode m_code; //!< Status code
-  uint16_t m_aid;
+  uint16_t m_aid; //!< aid
   HtCapabilities m_htCapability; //!< HT capabilities
-  HtOperations m_htOperations; //!< HT operations
+  HtOperation m_htOperation; //!< HT operation
   VhtCapabilities m_vhtCapability; //!< VHT capabilities
+  VhtOperation m_vhtOperation; //!< VHT operation
   ErpInformation m_erpInformation; //!< ERP information
   EdcaParameterSet m_edcaParameterSet; //!< EDCA Parameter Set
+  HeCapabilities m_heCapability; //!< HE capabilities
 };
 
 
@@ -297,6 +335,24 @@ public:
    * \param rates the supported rates
    */
   void SetSupportedRates (SupportedRates rates);
+  /**
+   * Set the HT capabilities.
+   *
+   * \param htcapabilities HT capabilities
+   */
+  void SetHtCapabilities (HtCapabilities htcapabilities);
+  /**
+   * Set the VHT capabilities.
+   *
+   * \param vhtcapabilities VHT capabilities
+   */
+  void SetVhtCapabilities (VhtCapabilities vhtcapabilities);
+  /**
+   * Set the HE capabilities.
+   *
+   * \param hecapabilities HE capabilities
+   */
+  void SetHeCapabilities (HeCapabilities hecapabilities);
   /**
    * Return the Service Set Identifier (SSID).
    *
@@ -309,12 +365,6 @@ public:
    * \return the supported rates
    */
   SupportedRates GetSupportedRates (void) const;
-  /**
-   * Set the HT capabilities.
-   *
-   * \param htcapabilities HT capabilities
-   */
-  void SetHtCapabilities (HtCapabilities htcapabilities);
   /**
    * Return the HT capabilities.
    *
@@ -322,28 +372,28 @@ public:
    */
   HtCapabilities GetHtCapabilities (void) const;
   /**
-   * Set the VHT capabilities.
+   * Return the VHT capabilities.
    *
-   * \param vhtcapabilities VHT capabilities
+   * \return VHT capabilities
    */
-  void SetVhtCapabilities (VhtCapabilities vhtcapabilities);
-  /**
-  * Return the VHT capabilities.
-  *
-  * \return VHT capabilities
-  */
   VhtCapabilities GetVhtCapabilities (void) const;
+  /**
+   * Return the HE capabilities.
+   *
+   * \return HE capabilities
+   */
+  HeCapabilities GetHeCapabilities (void) const;
 
   /**
    * Register this type.
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
 
 private:
@@ -351,6 +401,7 @@ private:
   SupportedRates m_rates;          //!< List of supported rates
   HtCapabilities m_htCapability;   //!< HT capabilities
   VhtCapabilities m_vhtCapability; //!< VHT capabilities
+  HeCapabilities m_heCapability; //!< HE capabilities
 };
 
 
@@ -401,17 +452,29 @@ public:
    */
   HtCapabilities GetHtCapabilities (void) const;
   /**
-   * Return the HT operations.
+   * Return the HT operation.
    *
-   * \return HT operations
+   * \return HT operation
    */
-  HtOperations GetHtOperations (void) const;
+  HtOperation GetHtOperation (void) const;
   /**
    * Return the VHT capabilities.
    *
    * \return VHT capabilities
    */
   VhtCapabilities GetVhtCapabilities (void) const;
+  /**
+   * Return the VHT operation.
+   *
+   * \return VHT operation
+   */
+  VhtOperation GetVhtOperation (void) const;
+  /**
+   * Return the HE capabilities.
+   *
+   * \return HE capabilities
+   */
+  HeCapabilities GetHeCapabilities (void) const;
   /**
    * Return the ERP information.
    *
@@ -437,17 +500,29 @@ public:
    */
   void SetHtCapabilities (HtCapabilities htcapabilities);
   /**
-   * Set the HT operations.
+   * Set the HT operation.
    *
-   * \param htoperations HT operations
+   * \param htoperation HT operation
    */
-  void SetHtOperations (HtOperations htoperations);
+  void SetHtOperation (HtOperation htoperation);
   /**
    * Set the VHT capabilities.
    *
    * \param vhtcapabilities VHT capabilities
    */
   void SetVhtCapabilities (VhtCapabilities vhtcapabilities);
+  /**
+   * Set the VHT operation.
+   *
+   * \param vhtoperation VHT operation
+   */
+  void SetVhtOperation (VhtOperation vhtoperation);
+  /**
+   * Set the HE capabilities.
+   *
+   * \param hecapabilities HE capabilities
+   */
+  void SetHeCapabilities (HeCapabilities hecapabilities);
   /**
    * Set the Service Set Identifier (SSID).
    *
@@ -496,11 +571,11 @@ public:
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
 
 private:
@@ -511,8 +586,10 @@ private:
   CapabilityInformation m_capability;  //!< Capability information
   DsssParameterSet m_dsssParameterSet; //!< DSSS Parameter Set
   HtCapabilities m_htCapability;       //!< HT capabilities
-  HtOperations m_htOperations;         //!< HT operations
+  HtOperation m_htOperation;           //!< HT operation
   VhtCapabilities m_vhtCapability;     //!< VHT capabilities
+  VhtOperation m_vhtOperation;         //!< VHT operation
+  HeCapabilities m_heCapability;     //!< HE capabilities
   ErpInformation m_erpInformation;     //!< ERP information
   EdcaParameterSet m_edcaParameterSet; //!< EDCA Parameter Set
 };
@@ -556,6 +633,7 @@ public:
    * Category values - see 802.11-2012 Table 8-38
    */
 
+  ///CategoryValue enumeration
   enum CategoryValue //table 8-38 staring from IEEE 802.11, Part11, (Year 2012)
   {
     BLOCK_ACK = 3,
@@ -567,6 +645,7 @@ public:
     VENDOR_SPECIFIC_ACTION = 127,
   };
 
+  ///SelfProtectedActionValue enumeration
   enum SelfProtectedActionValue //Category: 15 (Self Protected)
   {
     PEER_LINK_OPEN = 1,         //Mesh Peering Open
@@ -576,12 +655,14 @@ public:
     GROUP_KEY_ACK = 5,          //Mesh Group Key Acknowledge
   };
 
+  ///MultihopActionValue enumeration
   enum MultihopActionValue
   {
     PROXY_UPDATE = 0,                   //not used so far
     PROXY_UPDATE_CONFIRMATION = 1,      //not used so far
   };
 
+  ///MeshActionValue enumeration
   enum MeshActionValue
   {
     LINK_METRIC_REPORT = 0,               //Action Value:0 in Category 13: Mesh
@@ -614,18 +695,18 @@ public:
    */
   typedef union
   {
-    enum MeshActionValue meshAction;
-    enum MultihopActionValue multihopAction;
-    enum SelfProtectedActionValue selfProtectedAction;
-    enum BlockAckActionValue blockAck;
-  } ActionValue;
+    MeshActionValue meshAction; ///< mesh action
+    MultihopActionValue multihopAction; ///< multi hop action
+    SelfProtectedActionValue selfProtectedAction; ///< self protected action
+    BlockAckActionValue blockAck; ///< block ack
+  } ActionValue; ///< the action value
   /**
    * Set action for this Action header.
    *
    * \param type category
    * \param action action
    */
-  void   SetAction (enum CategoryValue type,ActionValue action);
+  void SetAction (CategoryValue type, ActionValue action);
 
   /**
    * Return the category value.
@@ -645,15 +726,25 @@ public:
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId () const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize () const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId () const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize () const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
 
 private:
+  /**
+   * Category value to string function
+   * \param value the category value
+   * \returns the categoty value string
+   */
   std::string CategoryValueToString (CategoryValue value) const;
+  /**
+   * Self protected action value to string function
+   * \param value the protected action value
+   * \returns the self protected action value string
+   */
   std::string SelfProtectedActionValueToString (SelfProtectedActionValue value) const;
   uint8_t m_category; //!< Category of the action
   uint8_t m_actionValue; //!< Action value
@@ -674,11 +765,11 @@ public:
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
   /**
    * Enable delayed Block ACK.
@@ -782,7 +873,7 @@ private:
    */
   void SetStartingSequenceControl (uint16_t seqControl);
 
-  uint8_t m_dialogToken;   /* Not used for now */
+  uint8_t m_dialogToken;   //!< Not used for now
   uint8_t m_amsduSupport;  //!< Flag if A-MSDU is supported
   uint8_t m_policy;        //!< Block ACK policy
   uint8_t m_tid;           //!< Traffic ID
@@ -806,11 +897,11 @@ public:
    * \return The TypeId.
    */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
   /**
    * Enable delayed Block ACK.
@@ -903,7 +994,7 @@ private:
    */
   void SetParameterSet (uint16_t params);
 
-  uint8_t m_dialogToken;   /* Not used for now */
+  uint8_t m_dialogToken;   //!< Not used for now
   StatusCode m_code;       //!< Status code
   uint8_t m_amsduSupport;  //!< Flag if A-MSDU is supported
   uint8_t m_policy;        //!< Block ACK policy
@@ -928,11 +1019,11 @@ public:
    */
   static TypeId GetTypeId (void);
   // Inherited
-  virtual TypeId GetInstanceTypeId (void) const;
-  virtual void Print (std::ostream &os) const;
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (Buffer::Iterator start) const;
-  virtual uint32_t Deserialize (Buffer::Iterator start);
+  TypeId GetInstanceTypeId (void) const;
+  void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (Buffer::Iterator start) const;
+  uint32_t Deserialize (Buffer::Iterator start);
 
   /**
    * Check if the initiator bit in the DELBA is setted.
@@ -962,6 +1053,7 @@ public:
    */
   void SetByRecipient (void);
 
+
 private:
   /**
    * Return the raw parameter set.
@@ -976,9 +1068,9 @@ private:
    */
   void SetParameterSet (uint16_t params);
 
-  uint16_t m_initiator;
+  uint16_t m_initiator; //!< initiator
   uint16_t m_tid; //!< Traffic ID
-  uint16_t m_reasonCode; /* Not used for now. Always set to 1: "Unspecified reason" */
+  uint16_t m_reasonCode; //!< Not used for now. Always set to 1: "Unspecified reason"
 };
 
 } //namespace ns3
