@@ -15,16 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Author: Duy Nguyen <duy@soe.ucsc.edu>
- *         Matías Richart <mrichart@fing.edu.uy>
+ * Authors: Duy Nguyen <duy@soe.ucsc.edu>
+ *          Matías Richart <mrichart@fing.edu.uy>
  */
 
 #ifndef MINSTREL_WIFI_MANAGER_H
 #define MINSTREL_WIFI_MANAGER_H
 
+#include "ns3/traced-value.h"
 #include "wifi-remote-station-manager.h"
-#include "wifi-mode.h"
-#include "ns3/nstime.h"
 #include "ns3/random-variable-stream.h"
 #include <fstream>
 
@@ -58,15 +57,15 @@ struct RateInfo
   uint64_t successHist;         //!< Aggregate of all transmission successes.
   uint64_t attemptHist;         //!< Aggregate of all transmission attempts.
 
-  uint8_t numSamplesSkipped;
-  int sampleLimit;
+  uint8_t numSamplesSkipped; //!< number of samples skipped
+  int sampleLimit; //!< sample limit
 };
 
 /**
  * Data structure for a Minstrel Rate table
  * A vector of a struct RateInfo
  */
-typedef std::vector<struct RateInfo> MinstrelRate;
+typedef std::vector<RateInfo> MinstrelRate;
 /**
  * Data structure for a Sample Rate table
  * A vector of a vector uint32_t
@@ -89,14 +88,14 @@ struct MinstrelWifiRemoteStation : public WifiRemoteStation
    * then we wrap back to the row 1 col 1.
    * note: there are many other ways to do this.
    */
-  uint32_t m_col, m_index;
+  uint32_t m_col, m_index;       ///< index
   uint32_t m_maxTpRate;          ///< the current throughput rate
   uint32_t m_maxTpRate2;         ///< second highest throughput rate
   uint32_t m_maxProbRate;        ///< rate with highest prob of success
   uint32_t m_nModes;             ///< number of modes supported
-  int m_totalPacketsCount;             ///< total number of packets as of now
-  int m_samplePacketsCount;             ///< how many packets we have sample so far
-  int m_numSamplesDeferred;
+  int m_totalPacketsCount;       ///< total number of packets as of now
+  int m_samplePacketsCount;      ///< how many packets we have sample so far
+  int m_numSamplesDeferred;      ///< number samles deferred
   bool m_isSampling;             ///< a flag to indicate we are currently sampling
   uint32_t m_sampleRate;         ///< current sample rate
   bool  m_sampleDeferred;        ///< a flag to indicate sample rate is on the second stage
@@ -107,7 +106,7 @@ struct MinstrelWifiRemoteStation : public WifiRemoteStation
   bool m_initialized;            ///< for initializing tables
   MinstrelRate m_minstrelTable;  ///< minstrel table
   SampleRate m_sampleTable;      ///< sample table
-  std::ofstream m_statsFile;
+  std::ofstream m_statsFile;     ///< stats file
 };
 
 
@@ -117,54 +116,58 @@ struct MinstrelWifiRemoteStation : public WifiRemoteStation
  *
  * Minstrel is a rate control algorithm implemented in MadWifi and Linux.
  * The basic principle is to probe the environment and adapt the rate
- * based on statistics collected on the probability of successful 
+ * based on statistics collected on the probability of successful
  * transmission.  The algorithm adapts the rate to the highest rate
  * that it considers successful, and spends a fraction of its time
  * doing 'look around' by trying other rates.
  *
- * Minstrel is appropriate for non-HT/VHT configurations; for HT/VHT
- * (i.e. 802.11n/ac), users should use MinstrelHtWifiManager instead.
+ * Minstrel is appropriate for non-HT/VHT/HE configurations; for HT/VHT/HE
+ * (i.e. 802.11n/ac/ax), users should use MinstrelHtWifiManager instead.
  * Minstrel will error exit if the user tries to configure it with a
- * Wi-Fi MAC that has VhtSupported or HtSupported set.
+ * Wi-Fi MAC that has VhtSupported, HtSupported or HeSupported set.
  *
  * Some notes on this implementation follow.  The implementation has
  * been adapted to bring it closer to the Linux implementation.
- * For each rate, a new parameter samplesSkipped is added. This parameter 
- * is intended to solve an issue regarding the sampling of low rates when 
- * a high rate is working well, which leads to outdated statistics. 
+ * For each rate, a new parameter samplesSkipped is added. This parameter
+ * is intended to solve an issue regarding the sampling of low rates when
+ * a high rate is working well, which leads to outdated statistics.
  * This change makes throughput a bit lower in simple, stable scenarios,
- * but may help in dynamic scenarios to react faster and more accurately 
+ * but may help in dynamic scenarios to react faster and more accurately
  * to changes.
- * 
+ *
  * Related to the previous, the logic for deciding when to sample random
- * rates is as follows.  When a sample rate is deffered to the second MRR 
- * chain stage, a new parameter (numSamplesDeferred) is increased. This 
- * paramters is used (jointly with sampleCount) to compare current 
+ * rates is as follows.  When a sample rate is deffered to the second MRR
+ * chain stage, a new parameter (numSamplesDeferred) is increased. This
+ * paramters is used (jointly with sampleCount) to compare current
  * sample count with the lookaround rate.
- * 
+ *
  * Also related with sampling, another parameter sampleLimit is added.
- * This parameter limits the number of times a very low or very high 
- * probability rate is sampled, avoiding to try a poorly working sample 
+ * This parameter limits the number of times a very low or very high
+ * probability rate is sampled, avoiding to try a poorly working sample
  * rate too often.
  *
  * When updating the EWMA probability of a rate for the first time, it does
- * not apply EWMA but instead assigns the entire probability. 
- * Since the EWMA probability is initialized to zero, this generates 
+ * not apply EWMA but instead assigns the entire probability.
+ * Since the EWMA probability is initialized to zero, this generates
  * a more accurate EWMA.
  */
 class MinstrelWifiManager : public WifiRemoteStationManager
 {
-
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   MinstrelWifiManager ();
   virtual ~MinstrelWifiManager ();
 
   // Inherited from WifiRemoteStationManager
-  virtual void SetupPhy (Ptr<WifiPhy> phy);
-  virtual void SetupMac (Ptr<WifiMac> mac);
-  virtual void SetHtSupported (bool enable);
-  virtual void SetVhtSupported (bool enable);
+  void SetupPhy (const Ptr<WifiPhy> phy);
+  void SetupMac (const Ptr<WifiMac> mac);
+  void SetHtSupported (bool enable);
+  void SetVhtSupported (bool enable);
+  void SetHeSupported (bool enable);
 
   /**
    * Assign a fixed random variable stream number to the random variables
@@ -177,51 +180,107 @@ public:
    */
   int64_t AssignStreams (int64_t stream);
 
+  /**
+   * updating the rate
+   *
+   * \param station the station object
+   */
   void UpdateRate (MinstrelWifiRemoteStation *station);
 
-  //updating the Minstrel Table every 1/10 seconds
+  /**
+   * updating the Minstrel Table every 1/10 seconds
+   *
+   * \param station the station object
+   */
   void UpdateStats (MinstrelWifiRemoteStation *station);
 
-  //find a rate to use from Minstrel Table
+  /**
+   * find a rate to use from Minstrel Table
+   *
+   * \param station the station object
+   * \returns the rate
+   */
   uint32_t FindRate (MinstrelWifiRemoteStation *station);
 
+  /**
+   * Get data transmit vector
+   *
+   * \param station the station object
+   * \returns WifiTxVector
+   */
   WifiTxVector GetDataTxVector (MinstrelWifiRemoteStation *station);
 
+  /**
+   * Get RTS transmit vector
+   *
+   * \param station the station object
+   * \returns WifiTxVector
+   */
   WifiTxVector GetRtsTxVector (MinstrelWifiRemoteStation *station);
 
+  /**
+   * Count retries
+   *
+   * \param station the station object
+   * \returns the number if retries
+   */
   uint32_t CountRetries (MinstrelWifiRemoteStation *station);
 
+  /**
+   * Update packet counters
+   *
+   * \param station the station object
+   */
   void UpdatePacketCounters (MinstrelWifiRemoteStation *station);
 
-  //update the number of retries and reset accordingly
+  /**
+   * update the number of retries and reset accordingly
+   *
+   * \param station the station object
+   */
   void UpdateRetry (MinstrelWifiRemoteStation *station);
 
-  void CheckInit (MinstrelWifiRemoteStation *station);  ///< check for initializations
+  /**
+   * check for initializations
+   *
+   * \param station the station object
+   */
+  void CheckInit (MinstrelWifiRemoteStation *station);
 
-  //initialize Sample Table
+  /**
+   * initialize Sample Table
+   *
+   * \param station the station object
+   */
   void InitSampleTable (MinstrelWifiRemoteStation *station);
 
 private:
   //overriden from base class
-  virtual WifiRemoteStation * DoCreateStation (void) const;
-  virtual void DoReportRxOk (WifiRemoteStation *station,
-                             double rxSnr, WifiMode txMode);
-  virtual void DoReportRtsFailed (WifiRemoteStation *station);
-  virtual void DoReportDataFailed (WifiRemoteStation *station);
-  virtual void DoReportRtsOk (WifiRemoteStation *station,
-                              double ctsSnr, WifiMode ctsMode, double rtsSnr);
-  virtual void DoReportDataOk (WifiRemoteStation *station,
-                               double ackSnr, WifiMode ackMode, double dataSnr);
-  virtual void DoReportFinalRtsFailed (WifiRemoteStation *station);
-  virtual void DoReportFinalDataFailed (WifiRemoteStation *station);
-  virtual WifiTxVector DoGetDataTxVector (WifiRemoteStation *station);
-  virtual WifiTxVector DoGetRtsTxVector (WifiRemoteStation *station);
+  WifiRemoteStation * DoCreateStation (void) const;
+  void DoReportRxOk (WifiRemoteStation *station,
+                     double rxSnr, WifiMode txMode);
+  void DoReportRtsFailed (WifiRemoteStation *station);
+  void DoReportDataFailed (WifiRemoteStation *station);
+  void DoReportRtsOk (WifiRemoteStation *station,
+                      double ctsSnr, WifiMode ctsMode, double rtsSnr);
+  void DoReportDataOk (WifiRemoteStation *station,
+                       double ackSnr, WifiMode ackMode, double dataSnr);
+  void DoReportFinalRtsFailed (WifiRemoteStation *station);
+  void DoReportFinalDataFailed (WifiRemoteStation *station);
+  WifiTxVector DoGetDataTxVector (WifiRemoteStation *station);
+  WifiTxVector DoGetRtsTxVector (WifiRemoteStation *station);
 
-  virtual bool DoNeedDataRetransmission (WifiRemoteStation *st, Ptr<const Packet> packet, bool normally);
+  bool DoNeedDataRetransmission (WifiRemoteStation *st,
+                                 Ptr<const Packet> packet, bool normally);
 
-  virtual bool IsLowLatency (void) const;
+  bool IsLowLatency (void) const;
 
-  //for estimating the TxTime of a packet with a given mode
+  /**
+   * for estimating the TxTime of a packet with a given mode
+   *
+   * \param mode Wi-Fi mode
+   * \returns the transmission time
+   */
   Time GetCalcTxTime (WifiMode mode) const;
   /**
    * Add transmission time for the given mode to an internal list.
@@ -231,10 +290,19 @@ private:
    */
   void AddCalcTxTime (WifiMode mode, Time t);
 
-  //initialize Minstrel Table
+  /**
+   * initialize Minstrel Table
+   *
+   * \param station the station object
+   */
   void RateInit (MinstrelWifiRemoteStation *station);
 
-  //getting the next sample from Sample Table
+  /**
+   * getting the next sample from Sample Table
+   *
+   * \param station the station object
+   * \returns the next sample
+   */
   uint32_t GetNextSample (MinstrelWifiRemoteStation *station);
 
   /**
@@ -252,16 +320,28 @@ private:
    *  - ACK timeouts
    *  - DATA transmission
    *  - backoffs according to CW
+   *
+   * \param dataTransmissionTime the data transmission time
+   * \param shortRetries short retries
+   * \param longRetries long retries
+   * \returns the unicast packet time
    */
   Time CalculateTimeUnicastPacket (Time dataTransmissionTime, uint32_t shortRetries, uint32_t longRetries);
 
-  //printing Sample Table
+  /**
+   * printing Sample Table
+   *
+   * \param station the station object
+   */
   void PrintSampleTable (MinstrelWifiRemoteStation *station);
 
-  //printing Minstrel Table
+  /**
+   * printing Minstrel Table
+   *
+   * \param station the station object
+   */
   void PrintTable (MinstrelWifiRemoteStation *station);
 
-
   /**
    * typedef for a vector of a pair of Time, WifiMode.
    * (Essentially a list for WifiMode and its corresponding transmission time
@@ -277,8 +357,10 @@ private:
   uint32_t m_pktLen;        ///< packet length used for calculate mode TxTime
   bool m_printStats;        ///< If statistics table should be printed.
 
-  //Provides uniform random variables.
+  /// Provides uniform random variables.
   Ptr<UniformRandomVariable> m_uniformRandomVariable;
+
+  TracedValue<uint64_t> m_currentRate; //!< Trace rate changes
 };
 
 } //namespace ns3
