@@ -21,11 +21,9 @@
 #ifndef MSDU_AGGREGATOR_H
 #define MSDU_AGGREGATOR_H
 
-#include "ns3/ptr.h"
 #include "ns3/packet.h"
 #include "ns3/object.h"
 #include "amsdu-subframe-header.h"
-#include <list>
 
 namespace ns3 {
 
@@ -38,21 +36,51 @@ class WifiMacHeader;
 class MsduAggregator : public Object
 {
 public:
+  /// DeaggregatedMsdus typedef
   typedef std::list<std::pair<Ptr<Packet>, AmsduSubframeHeader> > DeaggregatedMsdus;
+  /// DeaggregatedMsdusCI typedef
   typedef std::list<std::pair<Ptr<Packet>, AmsduSubframeHeader> >::const_iterator DeaggregatedMsdusCI;
 
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
 
+  /**
+   * Sets the maximum A-MSDU size in bytes.
+   * Value 0 means that MSDU aggregation is disabled.
+   *
+   * \param maxSize the maximum A-MSDU size in bytes.
+   */
   virtual void SetMaxAmsduSize (uint32_t maxSize) = 0;
+  /**
+   * Returns the maximum A-MSDU size in bytes.
+   * Value 0 means that MSDU aggregation is disabled.
+   *
+   * \return the maximum A-MSDU size in bytes.
+   */
   virtual uint32_t GetMaxAmsduSize (void) const = 0;
 
-  /* Adds <i>packet</i> to <i>aggregatedPacket</i>. In concrete aggregator's implementation is
+  /**
+   * Adds <i>packet</i> to <i>aggregatedPacket</i>. In concrete aggregator's implementation is
    * specified how and if <i>packet</i> can be added to <i>aggregatedPacket</i>. If <i>packet</i>
    * can be added returns true, false otherwise.
+   *
+   * \param packet the packet.
+   * \param aggregatedPacket the aggregated packet.
+   * \param src the source address.
+   * \param dest the destination address
+   * \return true if successful.
    */
   virtual bool Aggregate (Ptr<const Packet> packet, Ptr<Packet> aggregatedPacket,
-                          Mac48Address src, Mac48Address dest) = 0;
+                          Mac48Address src, Mac48Address dest) const = 0;
 
+  /**
+   *
+   * \param aggregatedPacket the aggregated packet.
+   * \returns DeaggregatedMsdus.
+   */
   static DeaggregatedMsdus Deaggregate (Ptr<Packet> aggregatedPacket);
 };
 
