@@ -19,7 +19,6 @@
  *          Sébastien Deronne <sebastien.deronne@gmail.com>
  */
 
-#include <cmath>
 #include "nist-error-rate-model.h"
 #include "wifi-phy.h"
 #include "ns3/log.h"
@@ -84,6 +83,7 @@ NistErrorRateModel::Get64QamBer (double snr) const
   NS_LOG_INFO ("64-Qam" << " snr=" << snr << " ber=" << ber);
   return ber;
 }
+
 double
 NistErrorRateModel::Get256QamBer (double snr) const
 {
@@ -94,6 +94,16 @@ NistErrorRateModel::Get256QamBer (double snr) const
   return ber;
 }
 
+double
+NistErrorRateModel::Get1024QamBer (double snr) const
+{
+  NS_LOG_FUNCTION (this << snr);
+  double z = std::sqrt (snr / (341.0 * 2.0));
+  double ber = 31.0 / 160.0 * 0.5 * erfc (z);
+  NS_LOG_INFO ("1024-Qam" << " snr=" << snr << " ber=" << ber);
+  return ber;
+}
+
 double
 NistErrorRateModel::GetFecBpskBer (double snr, uint32_t nbits,
                                    uint32_t bValue) const
@@ -246,6 +256,22 @@ NistErrorRateModel::GetFec256QamBer (double snr, uint32_t nbits,
   return pms;
 }
 
+double
+NistErrorRateModel::GetFec1024QamBer (double snr, uint32_t nbits,
+                                      uint32_t bValue) const
+{
+  NS_LOG_FUNCTION (this << snr << nbits << bValue);
+  double ber = Get1024QamBer (snr);
+  if (ber == 0.0)
+    {
+      return 1.0;
+    }
+  double pe = CalculatePe (ber, bValue);
+  pe = std::min (pe, 1.0);
+  double pms = std::pow (1 - pe, static_cast<double> (nbits));
+  return pms;
+}
+
 double
 NistErrorRateModel::GetChunkSuccessRate (WifiMode mode, WifiTxVector txVector, double snr, uint32_t nbits) const
 {
@@ -253,7 +279,8 @@ NistErrorRateModel::GetChunkSuccessRate (WifiMode mode, WifiTxVector txVector, d
   if (mode.GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM
       || mode.GetModulationClass () == WIFI_MOD_CLASS_OFDM
       || mode.GetModulationClass () == WIFI_MOD_CLASS_HT
-      || mode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+      || mode.GetModulationClass () == WIFI_MOD_CLASS_VHT
+      || mode.GetModulationClass () == WIFI_MOD_CLASS_HE)
     {
       if (mode.GetConstellationSize () == 2)
         {
@@ -338,10 +365,27 @@ NistErrorRateModel::GetChunkSuccessRate (WifiMode mode, WifiTxVector txVector, d
                                       );
             }
         }
+      else if (mode.GetConstellationSize () == 1024)
+        {
+          if (mode.GetCodeRate () == WIFI_CODE_RATE_5_6)
+            {
+              return GetFec1024QamBer (snr,
+                                       nbits,
+                                       5    // b value
+                                       );
+            }
+          else
+            {
+              return GetFec1024QamBer (snr,
+                                       nbits,
+                                       3    // b value
+                                       );
+            }
+        }
     }
   else if (mode.GetModulationClass () == WIFI_MOD_CLASS_DSSS || mode.GetModulationClass () == WIFI_MOD_CLASS_HR_DSSS)
     {
-      switch (mode.GetDataRate (20, 0, 1))
+      switch (mode.GetDataRate (20))
         {
         case 1000000:
           return DsssErrorRateModel::GetDsssDbpskSuccessRate (snr, nbits);
