@@ -21,8 +21,6 @@
 #ifndef NIST_ERROR_RATE_MODEL_H
 #define NIST_ERROR_RATE_MODEL_H
 
-#include <stdint.h>
-#include "wifi-mode.h"
 #include "error-rate-model.h"
 #include "dsss-error-rate-model.h"
 
@@ -39,11 +37,15 @@ namespace ns3 {
 class NistErrorRateModel : public ErrorRateModel
 {
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
 
   NistErrorRateModel ();
 
-  virtual double GetChunkSuccessRate (WifiMode mode, WifiTxVector txVector, double snr, uint32_t nbits) const;
+  double GetChunkSuccessRate (WifiMode mode, WifiTxVector txVector, double snr, uint32_t nbits) const;
 
 
 private:
@@ -95,6 +97,13 @@ private:
    * \return BER of QAM256 at the given SNR
    */
   double Get256QamBer (double snr) const;
+  /**
+   * Return BER of QAM1024 at the given SNR.
+   *
+   * \param snr snr ratio (not dB)
+   * \return BER of QAM1024 at the given SNR
+   */
+  double Get1024QamBer (double snr) const;
   /**
    * Return BER of BPSK at the given SNR after applying FEC.
    *
@@ -149,6 +158,16 @@ private:
    */
   double GetFec256QamBer (double snr, uint32_t nbits,
                           uint32_t bValue) const;
+  /**
+   * Return BER of QAM1024 at the given SNR after applying FEC.
+   *
+   * \param snr snr ratio (not dB)
+   * \param nbits the number of bits in the chunk
+   * \param bValue
+   * \return BER of QAM1024 at the given SNR after applying FEC
+   */
+  double GetFec1024QamBer (double snr, uint32_t nbits,
+                           uint32_t bValue) const;
 };
 
 } //namespace ns3
