@@ -27,7 +27,7 @@
 
 namespace ns3 {
 
-NS_LOG_COMPONENT_DEFINE ("OnoeWifiRemoteStation");
+NS_LOG_COMPONENT_DEFINE ("OnoeWifiManager");
 
 /**
  * \brief hold per-remote-station state for ONOE Wifi manager.
@@ -37,14 +37,14 @@ NS_LOG_COMPONENT_DEFINE ("OnoeWifiRemoteStation");
  */
 struct OnoeWifiRemoteStation : public WifiRemoteStation
 {
-  Time m_nextModeUpdate;
-  uint32_t m_shortRetry;
-  uint32_t m_longRetry;
-  uint32_t m_tx_ok;
-  uint32_t m_tx_err;
-  uint32_t m_tx_retr;
-  uint32_t m_tx_upper;
-  uint32_t m_txrate;
+  Time m_nextModeUpdate; ///< next mode update
+  uint32_t m_shortRetry; ///< short retry
+  uint32_t m_longRetry; ///< long retry
+  uint32_t m_tx_ok; ///< transmit ok
+  uint32_t m_tx_err; ///< transmit error
+  uint32_t m_tx_retr; ///< transmit retr
+  uint32_t m_tx_upper; ///< transmit upper
+  uint32_t m_txrate; ///< transmit rate
 };
 
 NS_OBJECT_ENSURE_REGISTERED (OnoeWifiManager);
@@ -69,11 +69,21 @@ OnoeWifiManager::GetTypeId (void)
                    UintegerValue (10),
                    MakeUintegerAccessor (&OnoeWifiManager::m_addCreditThreshold),
                    MakeUintegerChecker<uint32_t> ())
+    .AddTraceSource ("Rate",
+                     "Traced value for rate changes (b/s)",
+                     MakeTraceSourceAccessor (&OnoeWifiManager::m_currentRate),
+                     "ns3::TracedValueCallback::Uint64")
   ;
   return tid;
 }
 
 OnoeWifiManager::OnoeWifiManager ()
+  : WifiRemoteStationManager (),
+    m_currentRate (0)
+{
+}
+
+OnoeWifiManager::~OnoeWifiManager ()
 {
 }
 
@@ -283,7 +293,13 @@ OnoeWifiManager::DoGetDataTxVector (WifiRemoteStation *st)
       //avoid to use legacy rate adaptation algorithms for IEEE 802.11n/ac
       channelWidth = 20;
     }
-  return WifiTxVector (GetSupported (station, rateIndex), GetDefaultTxPowerLevel (), GetLongRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+  WifiMode mode = GetSupported (station, rateIndex);
+  if (m_currentRate != mode.GetDataRate (channelWidth))
+    {
+      NS_LOG_DEBUG ("New datarate: " << mode.GetDataRate (channelWidth));
+      m_currentRate = mode.GetDataRate (channelWidth);
+    }
+  return WifiTxVector (mode, GetDefaultTxPowerLevel (), GetLongRetryCount (station), GetPreambleForTransmission (mode, GetAddress (st)), 800, 1, 1, 0, channelWidth, GetAggregation (station), false);
 }
 
 WifiTxVector
@@ -298,14 +314,16 @@ OnoeWifiManager::DoGetRtsTxVector (WifiRemoteStation *st)
     }
   UpdateMode (station);
   WifiTxVector rtsTxVector;
+  WifiMode mode;
   if (GetUseNonErpProtection () == false)
     {
-      rtsTxVector = WifiTxVector (GetSupported (station, 0), GetDefaultTxPowerLevel (), GetShortRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+      mode = GetSupported (station, 0);
     }
   else
     {
-      rtsTxVector = WifiTxVector (GetNonErpSupported (station, 0), GetDefaultTxPowerLevel (), GetShortRetryCount (station), false, 1, 0, channelWidth, GetAggregation (station), false);
+      mode = GetNonErpSupported (station, 0);
     }
+  rtsTxVector = WifiTxVector (mode, GetDefaultTxPowerLevel (), GetShortRetryCount (station), GetPreambleForTransmission (mode, GetAddress (st)), 800, 1, 1, 0, channelWidth, GetAggregation (station), false);
   return rtsTxVector;
 }
 
@@ -335,4 +353,14 @@ OnoeWifiManager::SetVhtSupported (bool enable)
     }
 }
 
+void
+OnoeWifiManager::SetHeSupported (bool enable)
+{
+  //HE is not supported by this algorithm.
+  if (enable)
+    {
+      NS_FATAL_ERROR ("WifiRemoteStationManager selected does not support HE rates");
+    }
+}
+
 } //namespace ns3
