@@ -32,11 +32,18 @@ namespace ns3 {
  */
 class OriginatorBlockAckAgreement : public BlockAckAgreement
 {
+  /// allow BlockAckManager class access
   friend class BlockAckManager;
 
 
 public:
   OriginatorBlockAckAgreement ();
+  /**
+   * Constructor
+   *
+   * \param recipient MAC address
+   * \param tid Traffic ID
+   */
   OriginatorBlockAckAgreement (Mac48Address recipient, uint8_t tid);
   ~OriginatorBlockAckAgreement ();
   /*                                      receive ADDBAResponse
@@ -88,6 +95,7 @@ public:
   *    In both cases for station addressed by BlockAckAgreement::m_peer and for
   *    TID BlockAckAgreement::m_tid block ack mechanism won't be used.
   */
+  /// State enumeration
   enum State
   {
     PENDING,
@@ -95,7 +103,12 @@ public:
     INACTIVE,
     UNSUCCESSFUL
   };
-  void SetState (enum State state);
+  /**
+   * Set the current state.
+   *
+   * \param state to set
+   */
+  void SetState (State state);
   /**
    * Check if the current state of this agreement is PENDING.
    *
@@ -138,13 +151,14 @@ public:
    * false otherwise
    */
   bool IsBlockAckRequestNeeded (void) const;
+  /// Complete exchange function
   void CompleteExchange (void);
 
 
 private:
-  enum State m_state;
-  uint16_t m_sentMpdus;
-  bool m_needBlockAckReq;
+  State m_state; ///< state
+  uint16_t m_sentMpdus; ///< sent MPDUs
+  bool m_needBlockAckReq; ///< flag whether it needs a Block ACK request
 };
 
 } //namespace ns3
