@@ -16,11 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Author: Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
- * Author: Mirko Banchi <mk.banchi@gmail.com>
+ * Authors: Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
+ *          Mirko Banchi <mk.banchi@gmail.com>
  */
 
-#include "ns3/assert.h"
 #include "ns3/address-utils.h"
 #include "wifi-mac-header.h"
 
@@ -28,6 +27,7 @@ namespace ns3 {
 
 NS_OBJECT_ENSURE_REGISTERED (WifiMacHeader);
 
+/// type enumeration
 enum
 {
   TYPE_MGT = 0,
@@ -35,6 +35,7 @@ enum
   TYPE_DATA = 2
 };
 
+/// subtype enumeration
 enum
 {
   //Reserved: 0 - 6
@@ -177,7 +178,7 @@ WifiMacHeader::SetMultihopAction (void)
 }
 
 void
-WifiMacHeader::SetType (enum WifiMacType type)
+WifiMacHeader::SetType (WifiMacType type)
 {
   switch (type)
     {
@@ -396,7 +397,7 @@ void WifiMacHeader::SetQosNoEosp ()
   m_qosEosp = 0;
 }
 
-void WifiMacHeader::SetQosAckPolicy (enum QosAckPolicy policy)
+void WifiMacHeader::SetQosAckPolicy (QosAckPolicy policy)
 {
   switch (policy)
     {
@@ -487,7 +488,7 @@ WifiMacHeader::GetAddr4 (void) const
   return m_addr4;
 }
 
-enum WifiMacType
+WifiMacType
 WifiMacHeader::GetType (void) const
 {
   switch (m_ctrlType)
@@ -497,43 +498,30 @@ WifiMacHeader::GetType (void) const
         {
         case 0:
           return WIFI_MAC_MGT_ASSOCIATION_REQUEST;
-          break;
         case 1:
           return WIFI_MAC_MGT_ASSOCIATION_RESPONSE;
-          break;
         case 2:
           return WIFI_MAC_MGT_REASSOCIATION_REQUEST;
-          break;
         case 3:
           return WIFI_MAC_MGT_REASSOCIATION_RESPONSE;
-          break;
         case 4:
           return WIFI_MAC_MGT_PROBE_REQUEST;
-          break;
         case 5:
           return WIFI_MAC_MGT_PROBE_RESPONSE;
-          break;
         case 8:
           return WIFI_MAC_MGT_BEACON;
-          break;
         case 10:
           return WIFI_MAC_MGT_DISASSOCIATION;
-          break;
         case 11:
           return WIFI_MAC_MGT_AUTHENTICATION;
-          break;
         case 12:
           return WIFI_MAC_MGT_DEAUTHENTICATION;
-          break;
         case 13:
           return WIFI_MAC_MGT_ACTION;
-          break;
         case 14:
           return WIFI_MAC_MGT_ACTION_NO_ACK;
-          break;
         case 15:
           return WIFI_MAC_MGT_MULTIHOP_ACTION;
-          break;
         }
       break;
     case TYPE_CTL:
@@ -541,19 +529,14 @@ WifiMacHeader::GetType (void) const
         {
         case SUBTYPE_CTL_BACKREQ:
           return WIFI_MAC_CTL_BACKREQ;
-          break;
         case SUBTYPE_CTL_BACKRESP:
           return WIFI_MAC_CTL_BACKRESP;
-          break;
         case SUBTYPE_CTL_RTS:
           return WIFI_MAC_CTL_RTS;
-          break;
         case SUBTYPE_CTL_CTS:
           return WIFI_MAC_CTL_CTS;
-          break;
         case SUBTYPE_CTL_ACK:
           return WIFI_MAC_CTL_ACK;
-          break;
         }
       break;
     case TYPE_DATA:
@@ -561,55 +544,40 @@ WifiMacHeader::GetType (void) const
         {
         case 0:
           return WIFI_MAC_DATA;
-          break;
         case 1:
           return WIFI_MAC_DATA_CFACK;
-          break;
         case 2:
           return WIFI_MAC_DATA_CFPOLL;
-          break;
         case 3:
           return WIFI_MAC_DATA_CFACK_CFPOLL;
-          break;
         case 4:
           return WIFI_MAC_DATA_NULL;
-          break;
         case 5:
           return WIFI_MAC_DATA_NULL_CFACK;
-          break;
         case 6:
           return WIFI_MAC_DATA_NULL_CFPOLL;
-          break;
         case 7:
           return WIFI_MAC_DATA_NULL_CFACK_CFPOLL;
-          break;
         case 8:
           return WIFI_MAC_QOSDATA;
-          break;
         case 9:
           return WIFI_MAC_QOSDATA_CFACK;
-          break;
         case 10:
           return WIFI_MAC_QOSDATA_CFPOLL;
-          break;
         case 11:
           return WIFI_MAC_QOSDATA_CFACK_CFPOLL;
-          break;
         case 12:
           return WIFI_MAC_QOSDATA_NULL;
-          break;
         case 14:
           return WIFI_MAC_QOSDATA_NULL_CFPOLL;
-          break;
         case 15:
           return WIFI_MAC_QOSDATA_NULL_CFACK_CFPOLL;
-          break;
         }
       break;
     }
   // NOTREACHED
   NS_ASSERT (false);
-  return (enum WifiMacType) -1;
+  return (WifiMacType) - 1;
 }
 
 bool
@@ -663,10 +631,8 @@ WifiMacHeader::IsCfpoll (void) const
     case WIFI_MAC_QOSDATA_NULL_CFPOLL:
     case WIFI_MAC_QOSDATA_NULL_CFACK_CFPOLL:
       return true;
-      break;
     default:
       return false;
-      break;
     }
 }
 
@@ -856,27 +822,23 @@ WifiMacHeader::GetQosTid (void) const
   return m_qosTid;
 }
 
-enum WifiMacHeader::QosAckPolicy
+WifiMacHeader::QosAckPolicy
 WifiMacHeader::GetQosAckPolicy (void) const
 {
   switch (m_qosAckPolicy)
     {
     case 0:
       return NORMAL_ACK;
-      break;
     case 1:
       return NO_ACK;
-      break;
     case 2:
       return NO_EXPLICIT_ACK;
-      break;
     case 3:
       return BLOCK_ACK;
-      break;
     }
   // NOTREACHED
   NS_ASSERT (false);
-  return (enum QosAckPolicy) -1;
+  return (QosAckPolicy) - 1;
 }
 
 uint8_t
