@@ -45,12 +45,16 @@ enum WifiPhyStandard
    *  ACM MOBICOM, 2001.
    */
   WIFI_PHY_STANDARD_holland,
-  /** HT OFDM PHY for the 2.4 GHz band (clause 20) */
+  /** HT PHY for the 2.4 GHz band (clause 20) */
   WIFI_PHY_STANDARD_80211n_2_4GHZ,
-  /** HT OFDM PHY for the 5 GHz band (clause 20) */
+  /** HT PHY for the 5 GHz band (clause 20) */
   WIFI_PHY_STANDARD_80211n_5GHZ,
-  /** VHT OFDM PHY (clause 22) */
+  /** VHT PHY (clause 22) */
   WIFI_PHY_STANDARD_80211ac,
+  /** HE PHY for the 2.4 GHz band (clause 26) */
+  WIFI_PHY_STANDARD_80211ax_2_4GHZ,
+  /** HE PHY for the 5 GHz band (clause 26) */
+  WIFI_PHY_STANDARD_80211ax_5GHZ,
   /** Unspecified */
   WIFI_PHY_STANDARD_UNSPECIFIED
 };
