@@ -17,13 +17,12 @@
  *
  * Author: Nicola Baldo <nbaldo@cttc.es>
  */
+
 #ifndef WIFI_PHY_TAG_H
 #define WIFI_PHY_TAG_H
 
 #include <ns3/tag.h>
-#include <ns3/wifi-tx-vector.h>
-#include <ns3/wifi-preamble.h>
-#include <ns3/wifi-phy.h>
+#include "wifi-phy.h"
 
 namespace ns3 {
 
@@ -36,8 +35,12 @@ namespace ns3 {
 class WifiPhyTag : public Tag
 {
 public:
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
-  virtual TypeId GetInstanceTypeId (void) const;
+  TypeId GetInstanceTypeId (void) const;
 
   /**
    * Constructor
@@ -46,36 +49,30 @@ public:
   /**
    * Constructor
    * \param txVector the WifiTxVector
-   * \param preamble the WifiPreamble
-   * \param mpduType the mpduType
+   * \param mpdutype the mpduType
    */
-  WifiPhyTag (WifiTxVector txVector, WifiPreamble preamble, enum mpduType mpdutype);
+  WifiPhyTag (WifiTxVector txVector, MpduType mpdutype);
   /**
    * Getter for WifiTxVector parameter
    * \return the WifiTxVector
    */
   WifiTxVector GetWifiTxVector (void) const;
-  /**
-   * Getter for WifiPreamble parameter
-   * \return preamble the WifiPreamble
-   */
-  WifiPreamble GetWifiPreamble (void) const;
   /**
    * Getter for mpduType parameter
    * \return mpduType the mpduType
    */
-  enum mpduType GetMpduType (void) const;
+  MpduType GetMpduType (void) const;
 
   // From class Tag
-  virtual uint32_t GetSerializedSize (void) const;
-  virtual void Serialize (TagBuffer i) const;
-  virtual void Deserialize (TagBuffer i);
-  virtual void Print (std::ostream &os) const;
+  uint32_t GetSerializedSize (void) const;
+  void Serialize (TagBuffer i) const;
+  void Deserialize (TagBuffer i);
+  void Print (std::ostream &os) const;
+
 
 private:
-  WifiTxVector m_wifiTxVector;
-  int32_t m_wifiPreamble;
-  enum mpduType m_mpduType;
+  WifiTxVector m_wifiTxVector; ///< wifi transmit vector
+  MpduType m_mpduType; ///< MPDU type
 };
 
 } // namespace ns3
