@@ -18,19 +18,14 @@
  * Author: Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
  */
 
-#include <iostream>
 #include "wifi-remote-station-manager.h"
 #include "ns3/simulator.h"
-#include "ns3/assert.h"
 #include "ns3/log.h"
-#include "ns3/tag.h"
 #include "ns3/boolean.h"
-#include "ns3/double.h"
-#include "ns3/uinteger.h"
 #include "ns3/enum.h"
-#include "ns3/wifi-phy.h"
-#include "ns3/wifi-mac.h"
-#include "ns3/trace-source-accessor.h"
+#include "wifi-mac.h"
+#include "wifi-phy.h"
+#include "wifi-utils.h"
 #include "wifi-mac-header.h"
 #include "wifi-mac-trailer.h"
 
@@ -42,24 +37,37 @@ namespace ns3 {
 
 NS_LOG_COMPONENT_DEFINE ("WifiRemoteStationManager");
 
+/**
+ * HighLatencyDataTxVectorTag class
+ */
 class HighLatencyDataTxVectorTag : public Tag
 {
 public:
   HighLatencyDataTxVectorTag ();
+  /**
+   * Constructor
+   *
+   * \param dataTxVector TXVECTOR for data frames
+   */
   HighLatencyDataTxVectorTag (WifiTxVector dataTxVector);
   /**
    * \returns the transmission mode to use to send this packet
    */
   WifiTxVector GetDataTxVector (void) const;
 
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   virtual TypeId GetInstanceTypeId (void) const;
   virtual uint32_t GetSerializedSize (void) const;
   virtual void Serialize (TagBuffer i) const;
   virtual void Deserialize (TagBuffer i);
   virtual void Print (std::ostream &os) const;
+
 private:
-  WifiTxVector m_dataTxVector;
+  WifiTxVector m_dataTxVector; ///< TXVECTOR for data frames
 };
 
 HighLatencyDataTxVectorTag::HighLatencyDataTxVectorTag ()
@@ -118,10 +126,18 @@ HighLatencyDataTxVectorTag::Print (std::ostream &os) const
   os << "Data=" << m_dataTxVector;
 }
 
+/**
+ * HighLatencyRtsTxVectorTag class
+ */
 class HighLatencyRtsTxVectorTag : public Tag
 {
 public:
   HighLatencyRtsTxVectorTag ();
+  /**
+   * Constructor
+   *
+   * \param rtsTxVector TXVECTOR for RTS frames
+   */
   HighLatencyRtsTxVectorTag (WifiTxVector rtsTxVector);
   /**
    * \returns the transmission mode to use to send the RTS prior to the
@@ -129,14 +145,19 @@ public:
    */
   WifiTxVector GetRtsTxVector (void) const;
 
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   virtual TypeId GetInstanceTypeId (void) const;
   virtual uint32_t GetSerializedSize (void) const;
   virtual void Serialize (TagBuffer i) const;
   virtual void Deserialize (TagBuffer i);
   virtual void Print (std::ostream &os) const;
+
 private:
-  WifiTxVector m_rtsTxVector;
+  WifiTxVector m_rtsTxVector; ///< TXVECTOR for data frames
 };
 
 HighLatencyRtsTxVectorTag::HighLatencyRtsTxVectorTag ()
@@ -195,24 +216,37 @@ HighLatencyRtsTxVectorTag::Print (std::ostream &os) const
   os << "Rts=" << m_rtsTxVector;
 }
 
+/**
+ * HighLatencyCtsToSelfTxVectorTag class
+ */
 class HighLatencyCtsToSelfTxVectorTag : public Tag
 {
 public:
   HighLatencyCtsToSelfTxVectorTag ();
+  /**
+   * Constructor
+   *
+   * \param ctsToSelfTxVector TXVECTOR for CTS-to-self frames
+   */
   HighLatencyCtsToSelfTxVectorTag (WifiTxVector ctsToSelfTxVector);
   /**
    * \returns the transmission mode to use for the CTS-to-self.
    */
   WifiTxVector GetCtsToSelfTxVector (void) const;
 
+  /**
+   * \brief Get the type ID.
+   * \return the object TypeId
+   */
   static TypeId GetTypeId (void);
   virtual TypeId GetInstanceTypeId (void) const;
   virtual uint32_t GetSerializedSize (void) const;
   virtual void Serialize (TagBuffer i) const;
   virtual void Deserialize (TagBuffer i);
   virtual void Print (std::ostream &os) const;
+
 private:
-  WifiTxVector m_ctsToSelfTxVector;
+  WifiTxVector m_ctsToSelfTxVector; ///< TXVECTOR for CTS-to-self frames
 };
 
 HighLatencyCtsToSelfTxVectorTag::HighLatencyCtsToSelfTxVectorTag ()
@@ -330,11 +364,18 @@ WifiRemoteStationManager::GetTypeId (void)
                    UintegerValue (0),
                    MakeUintegerAccessor (&WifiRemoteStationManager::m_defaultTxPowerLevel),
                    MakeUintegerChecker<uint8_t> ())
-    .AddAttribute ("ProtectionMode",
+    .AddAttribute ("ErpProtectionMode",
                    "Protection mode used when non-ERP STAs are connected to an ERP AP: Rts-Cts or Cts-To-Self",
                    EnumValue (WifiRemoteStationManager::CTS_TO_SELF),
-                   MakeEnumAccessor (&WifiRemoteStationManager::SetProtectionMode,
-                                     &WifiRemoteStationManager::GetProtectionMode),
+                   MakeEnumAccessor (&WifiRemoteStationManager::SetErpProtectionMode,
+                                     &WifiRemoteStationManager::GetErpProtectionMode),
+                   MakeEnumChecker (WifiRemoteStationManager::RTS_CTS, "Rts-Cts",
+                                    WifiRemoteStationManager::CTS_TO_SELF, "Cts-To-Self"))
+    .AddAttribute ("HtProtectionMode",
+                   "Protection mode used when non-HT STAs are connected to a HT AP: Rts-Cts or Cts-To-Self",
+                   EnumValue (WifiRemoteStationManager::CTS_TO_SELF),
+                   MakeEnumAccessor (&WifiRemoteStationManager::SetHtProtectionMode,
+                                     &WifiRemoteStationManager::GetHtProtectionMode),
                    MakeEnumChecker (WifiRemoteStationManager::RTS_CTS, "Rts-Cts",
                                     WifiRemoteStationManager::CTS_TO_SELF, "Cts-To-Self"))
     .AddTraceSource ("MacTxRtsFailed",
@@ -358,11 +399,16 @@ WifiRemoteStationManager::GetTypeId (void)
 }
 
 WifiRemoteStationManager::WifiRemoteStationManager ()
-  : m_htSupported (false),
+  : m_qosSupported (false),
+    m_htSupported (false),
     m_vhtSupported (false),
+    m_heSupported (false),
     m_useNonErpProtection (false),
+    m_useNonHtProtection (false),
+    m_useGreenfieldProtection (false),
     m_shortPreambleEnabled (false),
-    m_shortSlotTimeEnabled (false)
+    m_shortSlotTimeEnabled (false),
+    m_rifsPermitted (false)
 {
 }
 
@@ -386,7 +432,7 @@ WifiRemoteStationManager::DoDispose (void)
 }
 
 void
-WifiRemoteStationManager::SetupPhy (Ptr<WifiPhy> phy)
+WifiRemoteStationManager::SetupPhy (const Ptr<WifiPhy> phy)
 {
   //We need to track our PHY because it is the object that knows the
   //full set of transmit rates that are supported. We need to know
@@ -395,7 +441,7 @@ WifiRemoteStationManager::SetupPhy (Ptr<WifiPhy> phy)
   //acknowledgements.
   m_wifiPhy = phy;
   m_defaultTxMode = phy->GetMode (0);
-  if (HasHtSupported () || HasVhtSupported ())
+  if (HasHtSupported () || HasVhtSupported () || HasHeSupported ())
     {
       m_defaultTxMcs = phy->GetMcs (0);
     }
@@ -403,7 +449,7 @@ WifiRemoteStationManager::SetupPhy (Ptr<WifiPhy> phy)
 }
 
 void
-WifiRemoteStationManager::SetupMac (Ptr<WifiMac> mac)
+WifiRemoteStationManager::SetupMac (const Ptr<WifiMac> mac)
 {
   //We need to track our MAC because it is the object that knows the
   //full set of interframe spaces.
@@ -411,6 +457,12 @@ WifiRemoteStationManager::SetupMac (Ptr<WifiMac> mac)
   Reset ();
 }
 
+void
+WifiRemoteStationManager::SetQosSupported (bool enable)
+{
+  m_qosSupported = enable;
+}
+
 void
 WifiRemoteStationManager::SetHtSupported (bool enable)
 {
@@ -442,23 +494,40 @@ WifiRemoteStationManager::SetFragmentationThreshold (uint32_t threshold)
 }
 
 void
-WifiRemoteStationManager::SetProtectionMode (WifiRemoteStationManager::ProtectionMode mode)
+WifiRemoteStationManager::SetErpProtectionMode (WifiRemoteStationManager::ProtectionMode mode)
 {
-  m_protectionMode = mode;
+  NS_LOG_FUNCTION (this << mode);
+  m_erpProtectionMode = mode;
+}
+
+void
+WifiRemoteStationManager::SetHtProtectionMode (WifiRemoteStationManager::ProtectionMode mode)
+{
+  NS_LOG_FUNCTION (this << mode);
+  m_htProtectionMode = mode;
 }
 
 void
 WifiRemoteStationManager::SetShortPreambleEnabled (bool enable)
 {
+  NS_LOG_FUNCTION (this << enable);
   m_shortPreambleEnabled = enable;
 }
 
 void
 WifiRemoteStationManager::SetShortSlotTimeEnabled (bool enable)
 {
+  NS_LOG_FUNCTION (this << enable);
   m_shortSlotTimeEnabled = enable;
 }
 
+void
+WifiRemoteStationManager::SetRifsPermitted (bool allow)
+{
+  NS_LOG_FUNCTION (this << allow);
+  m_rifsPermitted = allow;
+}
+
 bool
 WifiRemoteStationManager::GetShortSlotTimeEnabled (void) const
 {
@@ -471,10 +540,28 @@ WifiRemoteStationManager::GetShortPreambleEnabled (void) const
   return m_shortPreambleEnabled;
 }
 
-WifiRemoteStationManager::ProtectionMode
-WifiRemoteStationManager::GetProtectionMode (void) const
+bool
+WifiRemoteStationManager::GetRifsPermitted (void) const
 {
-  return m_protectionMode;
+  return m_rifsPermitted;
+}
+
+WifiRemoteStationManager::ProtectionMode
+WifiRemoteStationManager::GetErpProtectionMode (void) const
+{
+  return m_erpProtectionMode;
+}
+
+WifiRemoteStationManager::ProtectionMode
+WifiRemoteStationManager::GetHtProtectionMode (void) const
+{
+  return m_htProtectionMode;
+}
+
+bool
+WifiRemoteStationManager::HasQosSupported (void) const
+{
+  return m_qosSupported;
 }
 
 bool
@@ -495,6 +582,18 @@ WifiRemoteStationManager::HasVhtSupported (void) const
   return m_vhtSupported;
 }
 
+void
+WifiRemoteStationManager::SetHeSupported (bool enable)
+{
+  m_heSupported = enable;
+}
+
+bool
+WifiRemoteStationManager::HasHeSupported (void) const
+{
+  return m_heSupported;
+}
+
 uint32_t
 WifiRemoteStationManager::GetMaxSsrc (void) const
 {
@@ -596,6 +695,15 @@ WifiRemoteStationManager::AddAllSupportedMcs (Mac48Address address)
     }
 }
 
+void
+WifiRemoteStationManager::RemoveAllSupportedMcs (Mac48Address address)
+{
+  NS_LOG_FUNCTION (this << address);
+  NS_ASSERT (!address.IsGroup ());
+  WifiRemoteStationState *state = LookupState (address);
+  state->m_operationalMcsSet.clear ();
+}
+
 void
 WifiRemoteStationManager::AddSupportedMcs (Mac48Address address, WifiMode mcs)
 {
@@ -625,6 +733,12 @@ WifiRemoteStationManager::GetShortSlotTimeSupported (Mac48Address address) const
   return LookupState (address)->m_shortSlotTime;
 }
 
+bool
+WifiRemoteStationManager::GetQosSupported (Mac48Address address) const
+{
+  return LookupState (address)->m_qosSupported;
+}
+
 bool
 WifiRemoteStationManager::IsBrandNew (Mac48Address address) const
 {
@@ -717,11 +831,13 @@ WifiRemoteStationManager::GetDataTxVector (Mac48Address address, const WifiMacHe
   NS_LOG_FUNCTION (this << address << *header << packet);
   if (address.IsGroup ())
     {
+      WifiMode mode = GetNonUnicastMode ();
       WifiTxVector v;
-      v.SetMode (GetNonUnicastMode ());
+      v.SetMode (mode);
+      v.SetPreambleType (GetPreambleForTransmission (mode, address));
       v.SetTxPowerLevel (m_defaultTxPowerLevel);
       v.SetChannelWidth (m_wifiPhy->GetChannelWidth ());
-      v.SetShortGuardInterval (m_wifiPhy->GetGuardInterval ());
+      v.SetGuardInterval (ConvertGuardIntervalToNanoSeconds (mode, m_wifiPhy->GetShortGuardInterval (), m_wifiPhy->GetGuardInterval ()));
       v.SetNss (1);
       v.SetNess (0);
       v.SetStbc (false);
@@ -763,13 +879,34 @@ WifiRemoteStationManager::GetCtsToSelfTxVector (const WifiMacHeader *header,
 WifiTxVector
 WifiRemoteStationManager::DoGetCtsToSelfTxVector (void)
 {
-  return WifiTxVector (GetDefaultMode (),
+  WifiMode defaultMode = GetDefaultMode ();
+  WifiPreamble defaultPreamble;
+  if (defaultMode.GetModulationClass () == WIFI_MOD_CLASS_HE)
+    {
+      defaultPreamble = WIFI_PREAMBLE_HE_SU;
+    }
+  else if (defaultMode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+    {
+      defaultPreamble = WIFI_PREAMBLE_VHT;
+    }
+  else if (defaultMode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+    {
+      defaultPreamble = WIFI_PREAMBLE_HT_MF;
+    }
+  else
+    {
+      defaultPreamble = WIFI_PREAMBLE_LONG;
+    }
+
+  return WifiTxVector (defaultMode,
                        GetDefaultTxPowerLevel (),
                        0,
+                       defaultPreamble,
+                       ConvertGuardIntervalToNanoSeconds (defaultMode, m_wifiPhy->GetShortGuardInterval (), m_wifiPhy->GetGuardInterval ()),
+                       GetNumberOfAntennas (),
+                       GetMaxNumberOfTransmitStreams (),
+                       0,
                        m_wifiPhy->GetChannelWidth (),
-                       m_wifiPhy->GetGuardInterval (),
-                       GetNumberOfTransmitAntennas (),
-                       GetNumberOfTransmitAntennas (),
                        false,
                        false);
 }
@@ -879,11 +1016,16 @@ WifiRemoteStationManager::ReportRxOk (Mac48Address address, const WifiMacHeader
 
 void
 WifiRemoteStationManager::ReportAmpduTxStatus (Mac48Address address, uint8_t tid,
-                                               uint32_t nSuccessfulMpdus, uint32_t nFailedMpdus, double rxSnr, double dataSnr)
+                                               uint8_t nSuccessfulMpdus, uint8_t nFailedMpdus,
+                                               double rxSnr, double dataSnr)
 {
-  NS_LOG_FUNCTION (this << address << (uint16_t)tid << nSuccessfulMpdus << nFailedMpdus << rxSnr << dataSnr);
+  NS_LOG_FUNCTION (this << address << (uint16_t)tid << (uint16_t)nSuccessfulMpdus << (uint16_t)nFailedMpdus << rxSnr << dataSnr);
   NS_ASSERT (!address.IsGroup ());
   WifiRemoteStation *station = Lookup (address, tid);
+  for (uint32_t i = 0; i < nFailedMpdus; i++)
+    {
+      m_macTxDataFailed (address);
+    }
   DoReportAmpduTxStatus (station, nSuccessfulMpdus, nFailedMpdus, rxSnr, dataSnr);
 }
 
@@ -893,18 +1035,28 @@ WifiRemoteStationManager::NeedRts (Mac48Address address, const WifiMacHeader *he
 {
   WifiMode mode = txVector.GetMode ();
   NS_LOG_FUNCTION (this << address << *header << packet << mode);
-  if (m_protectionMode == RTS_CTS
+  if (address.IsGroup ())
+    {
+      return false;
+    }
+  if (m_erpProtectionMode == RTS_CTS
       && ((mode.GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
-      || (mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
-      || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT))
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_HE))
       && m_useNonErpProtection)
     {
       NS_LOG_DEBUG ("WifiRemoteStationManager::NeedRTS returning true to protect non-ERP stations");
       return true;
     }
-  if (address.IsGroup ())
+  else if (m_htProtectionMode == RTS_CTS
+           && ((mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+               || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT))
+           && m_useNonHtProtection
+           && !(m_erpProtectionMode != RTS_CTS && m_useNonErpProtection))
     {
-      return false;
+      NS_LOG_DEBUG ("WifiRemoteStationManager::NeedRTS returning true to protect non-HT stations");
+      return true;
     }
   bool normally = (packet->GetSize () + header->GetSize () + WIFI_MAC_FCS_LENGTH) > GetRtsCtsThreshold ();
   return DoNeedRts (Lookup (address, header), packet, normally);
@@ -915,18 +1067,28 @@ WifiRemoteStationManager::NeedCtsToSelf (WifiTxVector txVector)
 {
   WifiMode mode = txVector.GetMode ();
   NS_LOG_FUNCTION (this << mode);
-  if (m_protectionMode == CTS_TO_SELF
+  if (m_erpProtectionMode == CTS_TO_SELF
       && ((mode.GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
-      || (mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
-      || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT))
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+          || (mode.GetModulationClass () == WIFI_MOD_CLASS_HE))
       && m_useNonErpProtection)
     {
       NS_LOG_DEBUG ("WifiRemoteStationManager::NeedCtsToSelf returning true to protect non-ERP stations");
       return true;
     }
+  else if (m_htProtectionMode == CTS_TO_SELF
+           && ((mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+               || (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT))
+           && m_useNonHtProtection
+           && !(m_erpProtectionMode != CTS_TO_SELF && m_useNonErpProtection))
+    {
+      NS_LOG_DEBUG ("WifiRemoteStationManager::NeedCtsToSelf returning true to protect non-HT stations");
+      return true;
+    }
   else if (!m_useNonErpProtection)
     {
-    //search for the BSS Basic Rate set, if the used mode is in the basic set then there is no need for Cts To Self
+      //search for the BSS Basic Rate set, if the used mode is in the basic set then there is no need for Cts To Self
       for (WifiModeListIterator i = m_bssBasicRateSet.begin (); i != m_bssBasicRateSet.end (); i++)
         {
           if (mode == *i)
@@ -956,6 +1118,7 @@ WifiRemoteStationManager::NeedCtsToSelf (WifiTxVector txVector)
 void
 WifiRemoteStationManager::SetUseNonErpProtection (bool enable)
 {
+  NS_LOG_FUNCTION (this << enable);
   m_useNonErpProtection = enable;
 }
 
@@ -965,6 +1128,32 @@ WifiRemoteStationManager::GetUseNonErpProtection (void) const
   return m_useNonErpProtection;
 }
 
+void
+WifiRemoteStationManager::SetUseNonHtProtection (bool enable)
+{
+  NS_LOG_FUNCTION (this << enable);
+  m_useNonHtProtection = enable;
+}
+
+bool
+WifiRemoteStationManager::GetUseNonHtProtection (void) const
+{
+  return m_useNonHtProtection;
+}
+
+void
+WifiRemoteStationManager::SetUseGreenfieldProtection (bool enable)
+{
+  NS_LOG_FUNCTION (this << enable);
+  m_useGreenfieldProtection = enable;
+}
+
+bool
+WifiRemoteStationManager::GetUseGreenfieldProtection (void) const
+{
+  return m_useGreenfieldProtection;
+}
+
 bool
 WifiRemoteStationManager::NeedRtsRetransmission (Mac48Address address, const WifiMacHeader *header,
                                                  Ptr<const Packet> packet)
@@ -1114,7 +1303,7 @@ WifiRemoteStationManager::IsLastFragment (Mac48Address address, const WifiMacHea
 }
 
 bool
-WifiRemoteStationManager::IsAllowedControlAnswerModulationClass (enum WifiModulationClass modClassReq, enum WifiModulationClass modClassAnswer) const
+WifiRemoteStationManager::IsAllowedControlAnswerModulationClass (WifiModulationClass modClassReq, WifiModulationClass modClassAnswer) const
 {
   switch (modClassReq)
     {
@@ -1128,6 +1317,7 @@ WifiRemoteStationManager::IsAllowedControlAnswerModulationClass (enum WifiModula
       return (modClassAnswer == WIFI_MOD_CLASS_OFDM);
     case WIFI_MOD_CLASS_HT:
     case WIFI_MOD_CLASS_VHT:
+    case WIFI_MOD_CLASS_HE:
       return true;
     default:
       NS_FATAL_ERROR ("Modulation class not defined");
@@ -1169,7 +1359,7 @@ WifiRemoteStationManager::GetControlAnswerMode (Mac48Address address, WifiMode r
           found = true;
         }
     }
-  if (HasHtSupported () || HasVhtSupported ())
+  if (HasHtSupported () || HasVhtSupported () || HasHeSupported ())
     {
       if (!found)
         {
@@ -1238,14 +1428,14 @@ WifiRemoteStationManager::GetControlAnswerMode (Mac48Address address, WifiMode r
           found = true;
         }
     }
-  if (HasHtSupported () || HasVhtSupported ())
+  if (HasHtSupported () || HasVhtSupported () || HasHeSupported ())
     {
       for (uint32_t idx = 0; idx < m_wifiPhy->GetNMcs (); idx++)
         {
           WifiMode thismode = m_wifiPhy->GetMcs (idx);
           if (thismode.IsMandatory ()
               && (!found || thismode.IsHigherDataRate (mode))
-              && (!thismode.IsHigherCodeRate(reqMode))
+              && (!thismode.IsHigherCodeRate (reqMode))
               && (thismode.GetModulationClass () == reqMode.GetModulationClass ()))
             {
               mode = thismode;
@@ -1281,14 +1471,16 @@ WifiTxVector
 WifiRemoteStationManager::GetCtsTxVector (Mac48Address address, WifiMode rtsMode)
 {
   NS_ASSERT (!address.IsGroup ());
+  WifiMode ctsMode = GetControlAnswerMode (address, rtsMode);
   WifiTxVector v;
-  v.SetMode (GetControlAnswerMode (address, rtsMode));
-  v.SetTxPowerLevel (DoGetCtsTxPowerLevel (address, v.GetMode ()));
-  v.SetChannelWidth (DoGetCtsTxChannelWidth (address, v.GetMode ()));
-  v.SetShortGuardInterval (DoGetCtsTxGuardInterval (address, v.GetMode ()));
-  v.SetNss (DoGetCtsTxNss (address, v.GetMode ()));
-  v.SetNess (DoGetCtsTxNess (address, v.GetMode ()));
-  v.SetStbc (DoGetCtsTxStbc (address, v.GetMode ()));
+  v.SetMode (ctsMode);
+  v.SetPreambleType (GetPreambleForTransmission (ctsMode, address));
+  v.SetTxPowerLevel (DoGetCtsTxPowerLevel (address, ctsMode));
+  v.SetChannelWidth (DoGetCtsTxChannelWidth (address, ctsMode));
+  v.SetGuardInterval (DoGetCtsTxGuardInterval (address, ctsMode));
+  v.SetNss (DoGetCtsTxNss (address, ctsMode));
+  v.SetNess (DoGetCtsTxNess (address, ctsMode));
+  v.SetStbc (DoGetCtsTxStbc (address, ctsMode));
   return v;
 }
 
@@ -1296,14 +1488,16 @@ WifiTxVector
 WifiRemoteStationManager::GetAckTxVector (Mac48Address address, WifiMode dataMode)
 {
   NS_ASSERT (!address.IsGroup ());
+  WifiMode ackMode = GetControlAnswerMode (address, dataMode);
   WifiTxVector v;
-  v.SetMode (GetControlAnswerMode (address, dataMode));
-  v.SetTxPowerLevel (DoGetAckTxPowerLevel (address, v.GetMode ()));
-  v.SetChannelWidth (DoGetAckTxChannelWidth (address, v.GetMode ()));
-  v.SetShortGuardInterval (DoGetAckTxGuardInterval (address, v.GetMode ()));
-  v.SetNss (DoGetAckTxNss (address, v.GetMode ()));
-  v.SetNess (DoGetAckTxNess (address, v.GetMode ()));
-  v.SetStbc (DoGetAckTxStbc (address, v.GetMode ()));
+  v.SetMode (ackMode);
+  v.SetPreambleType (GetPreambleForTransmission (ackMode, address));
+  v.SetTxPowerLevel (DoGetAckTxPowerLevel (address, ackMode));
+  v.SetChannelWidth (DoGetAckTxChannelWidth (address, ackMode));
+  v.SetGuardInterval (DoGetAckTxGuardInterval (address, ackMode));
+  v.SetNss (DoGetAckTxNss (address, ackMode));
+  v.SetNess (DoGetAckTxNess (address, ackMode));
+  v.SetStbc (DoGetAckTxStbc (address, ackMode));
   return v;
 }
 
@@ -1311,14 +1505,16 @@ WifiTxVector
 WifiRemoteStationManager::GetBlockAckTxVector (Mac48Address address, WifiMode blockAckReqMode)
 {
   NS_ASSERT (!address.IsGroup ());
+  WifiMode blockAckMode = GetControlAnswerMode (address, blockAckReqMode);
   WifiTxVector v;
-  v.SetMode (GetControlAnswerMode (address, blockAckReqMode));
-  v.SetTxPowerLevel (DoGetBlockAckTxPowerLevel (address, v.GetMode ()));
-  v.SetChannelWidth (DoGetBlockAckTxChannelWidth (address, v.GetMode ()));
-  v.SetShortGuardInterval (DoGetBlockAckTxGuardInterval (address, v.GetMode ()));
-  v.SetNss (DoGetBlockAckTxNss (address, v.GetMode ()));
-  v.SetNess (DoGetBlockAckTxNess (address, v.GetMode ()));
-  v.SetStbc (DoGetBlockAckTxStbc (address, v.GetMode ()));
+  v.SetMode (blockAckMode);
+  v.SetPreambleType (GetPreambleForTransmission (blockAckMode, address));
+  v.SetTxPowerLevel (DoGetBlockAckTxPowerLevel (address, blockAckMode));
+  v.SetChannelWidth (DoGetBlockAckTxChannelWidth (address, blockAckMode));
+  v.SetGuardInterval (DoGetBlockAckTxGuardInterval (address, blockAckMode));
+  v.SetNss (DoGetBlockAckTxNss (address, blockAckMode));
+  v.SetNess (DoGetBlockAckTxNess (address, blockAckMode));
+  v.SetStbc (DoGetBlockAckTxStbc (address, blockAckMode));
   return v;
 }
 
@@ -1328,16 +1524,16 @@ WifiRemoteStationManager::DoGetCtsTxPowerLevel (Mac48Address address, WifiMode c
   return m_defaultTxPowerLevel;
 }
 
-uint32_t
+uint8_t
 WifiRemoteStationManager::DoGetCtsTxChannelWidth (Mac48Address address, WifiMode ctsMode)
 {
   return m_wifiPhy->GetChannelWidth ();
 }
 
-bool
+uint16_t
 WifiRemoteStationManager::DoGetCtsTxGuardInterval (Mac48Address address, WifiMode ctsMode)
 {
-  return m_wifiPhy->GetGuardInterval ();
+  return ConvertGuardIntervalToNanoSeconds (ctsMode, m_wifiPhy->GetShortGuardInterval (), m_wifiPhy->GetGuardInterval ());
 }
 
 uint8_t
@@ -1364,16 +1560,16 @@ WifiRemoteStationManager::DoGetAckTxPowerLevel (Mac48Address address, WifiMode a
   return m_defaultTxPowerLevel;
 }
 
-uint32_t
+uint8_t
 WifiRemoteStationManager::DoGetAckTxChannelWidth (Mac48Address address, WifiMode ctsMode)
 {
   return m_wifiPhy->GetChannelWidth ();
 }
 
-bool
+uint16_t
 WifiRemoteStationManager::DoGetAckTxGuardInterval (Mac48Address address, WifiMode ackMode)
 {
-  return m_wifiPhy->GetGuardInterval ();
+  return ConvertGuardIntervalToNanoSeconds (ackMode, m_wifiPhy->GetShortGuardInterval (), m_wifiPhy->GetGuardInterval ());
 }
 
 uint8_t
@@ -1400,16 +1596,16 @@ WifiRemoteStationManager::DoGetBlockAckTxPowerLevel (Mac48Address address, WifiM
   return m_defaultTxPowerLevel;
 }
 
-uint32_t
+uint8_t
 WifiRemoteStationManager::DoGetBlockAckTxChannelWidth (Mac48Address address, WifiMode ctsMode)
 {
   return m_wifiPhy->GetChannelWidth ();
 }
 
-bool
+uint16_t
 WifiRemoteStationManager::DoGetBlockAckTxGuardInterval (Mac48Address address, WifiMode blockAckMode)
 {
-  return m_wifiPhy->GetGuardInterval ();
+  return ConvertGuardIntervalToNanoSeconds (blockAckMode, m_wifiPhy->GetShortGuardInterval (), m_wifiPhy->GetGuardInterval ());
 }
 
 uint8_t
@@ -1461,14 +1657,17 @@ WifiRemoteStationManager::LookupState (Mac48Address address) const
   state->m_operationalRateSet.push_back (GetDefaultMode ());
   state->m_operationalMcsSet.push_back (GetDefaultMcs ());
   state->m_channelWidth = m_wifiPhy->GetChannelWidth ();
-  state->m_shortGuardInterval = m_wifiPhy->GetGuardInterval ();
+  state->m_shortGuardInterval = m_wifiPhy->GetShortGuardInterval ();
+  state->m_guardInterval = m_wifiPhy->GetGuardInterval ().GetNanoSeconds ();
   state->m_greenfield = m_wifiPhy->GetGreenfield ();
-  state->m_rx = 1;
+  state->m_streams = 1;
   state->m_ness = 0;
   state->m_aggregation = false;
   state->m_stbc = false;
+  state->m_qosSupported = false;
   state->m_htSupported = false;
   state->m_vhtSupported = false;
+  state->m_heSupported = false;
   const_cast<WifiRemoteStationManager *> (this)->m_states.push_back (state);
   NS_LOG_DEBUG ("WifiRemoteStationManager::LookupState returning new state");
   return state;
@@ -1512,6 +1711,15 @@ WifiRemoteStationManager::Lookup (Mac48Address address, uint8_t tid) const
   return station;
 }
 
+void
+WifiRemoteStationManager::SetQosSupport (Mac48Address from, bool qosSupported)
+{
+  NS_LOG_FUNCTION (this << from << qosSupported);
+  WifiRemoteStationState *state;
+  state = LookupState (from);
+  state->m_qosSupported = qosSupported;
+}
+
 void
 WifiRemoteStationManager::AddStationHtCapabilities (Mac48Address from, HtCapabilities htCapabilities)
 {
@@ -1529,8 +1737,9 @@ WifiRemoteStationManager::AddStationHtCapabilities (Mac48Address from, HtCapabil
       state->m_channelWidth = 20;
     }
   state->m_htSupported = true;
+  SetQosSupport (from, true);
   state->m_greenfield = htCapabilities.GetGreenfield ();
-  state->m_rx = htCapabilities.GetRxHighestSupportedAntennas ();
+  state->m_streams = htCapabilities.GetRxHighestSupportedAntennas ();
 }
 
 void
@@ -1557,6 +1766,29 @@ WifiRemoteStationManager::AddStationVhtCapabilities (Mac48Address from, VhtCapab
   state->m_vhtSupported = true;
 }
 
+void
+WifiRemoteStationManager::AddStationHeCapabilities (Mac48Address from, HeCapabilities heCapabilities)
+{
+  //Used by all stations to record HE capabilities of remote stations
+  NS_LOG_FUNCTION (this << from << heCapabilities);
+  WifiRemoteStationState *state;
+  state = LookupState (from);
+  if (heCapabilities.GetHeLtfAndGiForHePpdus () >= 2)
+    {
+      state->m_guardInterval = 800;
+    }
+  else if (heCapabilities.GetHeLtfAndGiForHePpdus () == 1)
+    {
+      state->m_guardInterval = 1600;
+    }
+  else
+    {
+      state->m_guardInterval = 3200;
+    }
+  state->m_heSupported = true;
+  SetQosSupport (from, true);
+}
+
 bool
 WifiRemoteStationManager::GetGreenfieldSupported (Mac48Address address) const
 {
@@ -1595,7 +1827,7 @@ void
 WifiRemoteStationManager::AddBasicMode (WifiMode mode)
 {
   NS_LOG_FUNCTION (this << mode);
-  if (mode.GetModulationClass () == WIFI_MOD_CLASS_HT || mode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+  if (mode.GetModulationClass () == WIFI_MOD_CLASS_HT || mode.GetModulationClass () == WIFI_MOD_CLASS_VHT || mode.GetModulationClass () == WIFI_MOD_CLASS_HE)
     {
       NS_FATAL_ERROR ("It is not allowed to add a (V)HT rate in the BSSBasicRateSet!");
     }
@@ -1627,13 +1859,13 @@ WifiRemoteStationManager::GetNNonErpBasicModes (void) const
 {
   uint32_t size = 0;
   for (WifiModeListIterator i = m_bssBasicRateSet.begin (); i != m_bssBasicRateSet.end (); i++)
-  {
-    if (i->GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
-      {
-        continue;
-      }
-    size++;
-  }
+    {
+      if (i->GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
+        {
+          continue;
+        }
+      size++;
+    }
   return size;
 }
 
@@ -1644,28 +1876,28 @@ WifiRemoteStationManager::GetNonErpBasicMode (uint32_t i) const
   uint32_t index = 0;
   bool found = false;
   for (WifiModeListIterator j = m_bssBasicRateSet.begin (); j != m_bssBasicRateSet.end (); )
-  {
-    if (i == index)
-      {
-        found = true;
-      }
-    if (j->GetModulationClass () != WIFI_MOD_CLASS_ERP_OFDM)
-      {
-        if (found)
-          {
-            break;
-          }
-      }
-    index++;
-    j++;
-  }
+    {
+      if (i == index)
+        {
+          found = true;
+        }
+      if (j->GetModulationClass () != WIFI_MOD_CLASS_ERP_OFDM)
+        {
+          if (found)
+            {
+              break;
+            }
+        }
+      index++;
+      j++;
+    }
   return m_bssBasicRateSet[index];
 }
 
 void
 WifiRemoteStationManager::AddBasicMcs (WifiMode mcs)
 {
-  NS_LOG_FUNCTION (this << (uint32_t)mcs.GetMcsValue ());
+  NS_LOG_FUNCTION (this << (uint16_t)mcs.GetMcsValue ());
   for (uint32_t i = 0; i < GetNBasicMcs (); i++)
     {
       if (GetBasicMcs (i) == mcs)
@@ -1731,7 +1963,7 @@ WifiRemoteStationManager::DoNeedFragmentation (WifiRemoteStation *station,
 }
 
 void
-WifiRemoteStationManager::DoReportAmpduTxStatus (WifiRemoteStation *station, uint32_t nSuccessfulMpdus, uint32_t nFailedMpdus, double rxSnr, double dataSnr)
+WifiRemoteStationManager::DoReportAmpduTxStatus (WifiRemoteStation *station, uint8_t nSuccessfulMpdus, uint8_t nFailedMpdus, double rxSnr, double dataSnr)
 {
   NS_LOG_DEBUG ("DoReportAmpduTxStatus received but the manager does not handle A-MPDUs!");
 }
@@ -1760,25 +1992,31 @@ WifiRemoteStationManager::GetNonErpSupported (const WifiRemoteStation *station,
   uint32_t index = 0;
   bool found = false;
   for (WifiModeListIterator j = station->m_state->m_operationalRateSet.begin (); j != station->m_state->m_operationalRateSet.end (); )
-  {
-    if (i == index)
-      {
-        found = true;
-      }
-    if (j->GetModulationClass () != WIFI_MOD_CLASS_ERP_OFDM)
-      {
-        if (found)
-          {
-            break;
-          }
-      }
-    index++;
-    j++;
-  }
+    {
+      if (i == index)
+        {
+          found = true;
+        }
+      if (j->GetModulationClass () != WIFI_MOD_CLASS_ERP_OFDM)
+        {
+          if (found)
+            {
+              break;
+            }
+        }
+      index++;
+      j++;
+    }
   return station->m_state->m_operationalRateSet[index];
 }
 
-uint32_t
+Mac48Address
+WifiRemoteStationManager::GetAddress (const WifiRemoteStation *station) const
+{
+  return station->m_state->m_address;
+}
+
+uint8_t
 WifiRemoteStationManager::GetChannelWidth (const WifiRemoteStation *station) const
 {
   return station->m_state->m_channelWidth;
@@ -1790,6 +2028,12 @@ WifiRemoteStationManager::GetShortGuardInterval (const WifiRemoteStation *statio
   return station->m_state->m_shortGuardInterval;
 }
 
+uint16_t
+WifiRemoteStationManager::GetGuardInterval (const WifiRemoteStation *station) const
+{
+  return station->m_state->m_guardInterval;
+}
+
 bool
 WifiRemoteStationManager::GetGreenfield (const WifiRemoteStation *station) const
 {
@@ -1809,9 +2053,9 @@ WifiRemoteStationManager::GetStbc (const WifiRemoteStation *station) const
 }
 
 uint8_t
-WifiRemoteStationManager::GetNumberOfSupportedRxAntennas (const WifiRemoteStation *station) const
+WifiRemoteStationManager::GetNumberOfSupportedStreams (const WifiRemoteStation *station) const
 {
-  return station->m_state->m_rx;
+  return station->m_state->m_streams;
 }
 
 uint32_t
@@ -1850,6 +2094,12 @@ WifiRemoteStationManager::GetNSupported (const WifiRemoteStation *station) const
   return station->m_state->m_operationalRateSet.size ();
 }
 
+bool
+WifiRemoteStationManager::GetQosSupported (const WifiRemoteStation *station) const
+{
+  return station->m_state->m_qosSupported;
+}
+
 bool
 WifiRemoteStationManager::GetHtSupported (const WifiRemoteStation *station) const
 {
@@ -1862,6 +2112,12 @@ WifiRemoteStationManager::GetVhtSupported (const WifiRemoteStation *station) con
   return station->m_state->m_vhtSupported;
 }
 
+bool
+WifiRemoteStationManager::GetHeSupported (const WifiRemoteStation *station) const
+{
+  return station->m_state->m_heSupported;
+}
+
 uint32_t
 WifiRemoteStationManager::GetNMcsSupported (const WifiRemoteStation *station) const
 {
@@ -1873,26 +2129,83 @@ WifiRemoteStationManager::GetNNonErpSupported (const WifiRemoteStation *station)
 {
   uint32_t size = 0;
   for (WifiModeListIterator i = station->m_state->m_operationalRateSet.begin (); i != station->m_state->m_operationalRateSet.end (); i++)
-  {
-    if (i->GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
-      {
-        continue;
-      }
-    size++;
-  }
+    {
+      if (i->GetModulationClass () == WIFI_MOD_CLASS_ERP_OFDM)
+        {
+          continue;
+        }
+      size++;
+    }
   return size;
 }
 
+uint8_t
+WifiRemoteStationManager::GetChannelWidthSupported (Mac48Address address) const
+{
+  return LookupState (address)->m_channelWidth;
+}
+
+bool
+WifiRemoteStationManager::GetVhtSupported (Mac48Address address) const
+{
+  return LookupState (address)->m_vhtSupported;
+}
+
 void
 WifiRemoteStationManager::SetDefaultTxPowerLevel (uint8_t txPower)
 {
   m_defaultTxPowerLevel = txPower;
 }
 
-uint32_t
-WifiRemoteStationManager::GetNumberOfTransmitAntennas (void)
+uint8_t
+WifiRemoteStationManager::GetNumberOfAntennas (void)
 {
-  return m_wifiPhy->GetNumberOfTransmitAntennas ();
+  return m_wifiPhy->GetNumberOfAntennas ();
+}
+
+uint8_t
+WifiRemoteStationManager::GetMaxNumberOfTransmitStreams (void)
+{
+  return m_wifiPhy->GetMaxSupportedTxSpatialStreams ();
+}
+
+WifiPreamble
+WifiRemoteStationManager::GetPreambleForTransmission (WifiMode mode, Mac48Address dest)
+{
+  NS_LOG_FUNCTION (this << mode << dest);
+  WifiPreamble preamble;
+  if (mode.GetModulationClass () == WIFI_MOD_CLASS_HE)
+    {
+      preamble = WIFI_PREAMBLE_HE_SU;
+    }
+  else if (mode.GetModulationClass () == WIFI_MOD_CLASS_VHT)
+    {
+      preamble = WIFI_PREAMBLE_VHT;
+    }
+  else if (mode.GetModulationClass () == WIFI_MOD_CLASS_HT && m_wifiPhy->GetGreenfield () && GetGreenfieldSupported (dest) && !GetUseGreenfieldProtection ())
+    {
+      //If protection for greenfield is used we go for HT_MF preamble which is the default protection for GF format defined in the standard.
+      preamble = WIFI_PREAMBLE_HT_GF;
+    }
+  else if (mode.GetModulationClass () == WIFI_MOD_CLASS_HT)
+    {
+      preamble = WIFI_PREAMBLE_HT_MF;
+    }
+  else if (GetShortPreambleEnabled ())
+    {
+      preamble = WIFI_PREAMBLE_SHORT;
+    }
+  else
+    {
+      preamble = WIFI_PREAMBLE_LONG;
+    }
+  NS_LOG_DEBUG ("selected preamble=" << preamble);
+  return preamble;
+}
+
+WifiRemoteStation::~WifiRemoteStation ()
+{
+  NS_LOG_FUNCTION (this);
 }
 
 WifiRemoteStationInfo::WifiRemoteStationInfo ()
@@ -1931,9 +2244,4 @@ WifiRemoteStationInfo::GetFrameErrorRate () const
   return m_failAvg;
 }
 
-WifiRemoteStation::~WifiRemoteStation ()
-{
-  NS_LOG_FUNCTION (this);
-}
-
 } //namespace ns3
