@@ -19,15 +19,16 @@
  *          Ghada Badawy <gbadawy@gmail.com>
  */
 
-#include "ns3/wifi-tx-vector.h"
-#include "ns3/fatal-error.h"
+#include "wifi-tx-vector.h"
 
 namespace ns3 {
 
 WifiTxVector::WifiTxVector ()
   : m_retries (0),
+    m_preamble (WIFI_PREAMBLE_NONE),
     m_channelWidth (20),
-    m_shortGuardInterval (false),
+    m_guardInterval (800),
+    m_nTx (1),
     m_nss (1),
     m_ness (0),
     m_aggregation (false),
@@ -37,14 +38,24 @@ WifiTxVector::WifiTxVector ()
 {
 }
 
-WifiTxVector::WifiTxVector (WifiMode mode, uint8_t powerLevel, uint8_t retries,
-                            bool shortGuardInterval, uint8_t nss, uint8_t ness,
-                            uint32_t channelWidth, bool aggregation, bool stbc)
+WifiTxVector::WifiTxVector (WifiMode mode,
+                            uint8_t powerLevel,
+                            uint8_t retries,
+                            WifiPreamble preamble,
+                            uint16_t guardInterval,
+                            uint8_t nTx,
+                            uint8_t nss,
+                            uint8_t ness,
+                            uint8_t channelWidth,
+                            bool aggregation,
+                            bool stbc)
   : m_mode (mode),
     m_txPowerLevel (powerLevel),
     m_retries (retries),
+    m_preamble (preamble),
     m_channelWidth (channelWidth),
-    m_shortGuardInterval (shortGuardInterval),
+    m_guardInterval (guardInterval),
+    m_nTx (nTx),
     m_nss (nss),
     m_ness (ness),
     m_aggregation (aggregation),
@@ -80,16 +91,28 @@ WifiTxVector::GetRetries (void) const
   return m_retries;
 }
 
-uint32_t
+WifiPreamble
+WifiTxVector::GetPreambleType (void) const
+{
+  return m_preamble;
+}
+
+uint8_t
 WifiTxVector::GetChannelWidth (void) const
 {
   return m_channelWidth;
 }
 
-bool
-WifiTxVector::IsShortGuardInterval (void) const
+uint16_t
+WifiTxVector::GetGuardInterval (void) const
 {
-  return m_shortGuardInterval;
+  return m_guardInterval;
+}
+
+uint8_t
+WifiTxVector::GetNTx (void) const
+{
+  return m_nTx;
 }
 
 uint8_t
@@ -137,15 +160,27 @@ WifiTxVector::SetRetries (uint8_t retries)
 }
 
 void
-WifiTxVector::SetChannelWidth (uint32_t channelWidth)
+WifiTxVector::SetPreambleType (WifiPreamble preamble)
+{
+  m_preamble = preamble;
+}
+
+void
+WifiTxVector::SetChannelWidth (uint8_t channelWidth)
 {
   m_channelWidth = channelWidth;
 }
 
 void
-WifiTxVector::SetShortGuardInterval (bool guardinterval)
+WifiTxVector::SetGuardInterval (uint16_t guardInterval)
 {
-  m_shortGuardInterval = guardinterval;
+  m_guardInterval = guardInterval;
+}
+
+void
+WifiTxVector::SetNTx (uint8_t nTx)
+{
+  m_nTx = nTx;
 }
 
 void
@@ -175,12 +210,14 @@ WifiTxVector::SetStbc (bool stbc)
 std::ostream & operator << ( std::ostream &os, const WifiTxVector &v)
 {
   os << "mode: " << v.GetMode () <<
-    " txpwrlvl: " << (uint32_t)v.GetTxPowerLevel () <<
-    " retries: " << (uint32_t)v.GetRetries () <<
-    " channel width: " << v.GetChannelWidth () <<
-    " Short GI: " << v.IsShortGuardInterval () <<
-    " Nss: " << (uint32_t)v.GetNss () <<
-    " Ness: " << (uint32_t)v.GetNess () <<
+    " txpwrlvl: " << (uint16_t)v.GetTxPowerLevel () <<
+    " retries: " << (uint16_t)v.GetRetries () <<
+    " preamble: " << v.GetPreambleType () <<
+    " channel width: " << (uint16_t)v.GetChannelWidth () <<
+    " GI: " << v.GetGuardInterval () <<
+    " NTx: " << (uint16_t)v.GetNTx () <<
+    " Nss: " << (uint16_t)v.GetNss () <<
+    " Ness: " << (uint16_t)v.GetNess () <<
     " MPDU aggregation: " << v.IsAggregation () <<
     " STBC: " << v.IsStbc ();
   return os;
