@@ -17,39 +17,67 @@
  */
 
 #include "ns3/test.h"
-#include "ns3/packet.h"
-#include "ns3/tag.h"
-#include "ns3/packet-burst.h"
 #include "ns3/spectrum-wifi-helper.h"
 #include "ns3/wifi-spectrum-value-helper.h"
 #include "ns3/spectrum-wifi-phy.h"
-#include "ns3/interference-helper.h"
 #include "ns3/nist-error-rate-model.h"
 #include "ns3/wifi-mac-header.h"
 #include "ns3/wifi-mac-trailer.h"
 #include "ns3/wifi-phy-tag.h"
-#include "ns3/wifi-phy-standard.h"
 #include "ns3/wifi-spectrum-signal-parameters.h"
 
 using namespace ns3;
 
-static const uint16_t CHANNEL_NUMBER = 36;
+static const uint8_t CHANNEL_NUMBER = 36;
 static const uint32_t FREQUENCY = 5180; // MHz
-static const uint32_t CHANNEL_WIDTH = 20; // MHz
+static const uint8_t CHANNEL_WIDTH = 20; // MHz
+static const uint8_t GUARD_WIDTH = 10; // MHz
 
+/**
+ * \ingroup wifi-test
+ * \ingroup tests
+ *
+ * \brief Spectrum Wifi Phy Basic Test
+ */
 class SpectrumWifiPhyBasicTest : public TestCase
 {
 public:
   SpectrumWifiPhyBasicTest ();
+  /**
+   * Constructor
+   *
+   * \param name reference name
+   */
   SpectrumWifiPhyBasicTest (std::string name);
   virtual ~SpectrumWifiPhyBasicTest ();
 protected:
   virtual void DoSetup (void);
-  Ptr<SpectrumWifiPhy> m_phy;
+  Ptr<SpectrumWifiPhy> m_phy; ///< Phy
+  /**
+   * Make signal function
+   * \param txPowerWatts the transmit power in watts
+   * \returns Ptr<SpectrumSignalParameters>
+   */
   Ptr<SpectrumSignalParameters> MakeSignal (double txPowerWatts);
+  /**
+   * Send signal function
+   * \param txPowerWatts the transmit power in watts
+   */
   void SendSignal (double txPowerWatts);
-  void SpectrumWifiPhyReceiver (bool rxSucceeded);
-  uint32_t m_count;
+  /**
+   * Spectrum wifi receive success function
+   * \param p the packet
+   * \param snr the SNR
+   * \param txVector the transmit vector
+   */
+  void SpectrumWifiPhyRxSuccess (Ptr<Packet> p, double snr, WifiTxVector txVector);
+  /**
+   * Spectrum wifi receive failure function
+   * \param p the packet
+   * \param snr the SNR
+   */
+  void SpectrumWifiPhyRxFailure (Ptr<Packet> p, double snr);
+  uint32_t m_count; ///< count
 private:
   virtual void DoRun (void);
 };
@@ -67,14 +95,11 @@ SpectrumWifiPhyBasicTest::SpectrumWifiPhyBasicTest (std::string name)
 }
 
 // Make a Wi-Fi signal to inject directly to the StartRx() method
-Ptr<SpectrumSignalParameters> 
+Ptr<SpectrumSignalParameters>
 SpectrumWifiPhyBasicTest::MakeSignal (double txPowerWatts)
 {
-  WifiPreamble preamble;
-  preamble = WIFI_PREAMBLE_LONG;
-  WifiMode mode = WifiPhy::GetOfdmRate6Mbps ();
-  WifiTxVector txVector = WifiTxVector (mode, 0, 0, false, 1, 0, 20000000, false, false);
-  enum mpduType mpdutype = NORMAL_MPDU;
+  WifiTxVector txVector = WifiTxVector (WifiPhy::GetOfdmRate6Mbps (), 0, 0, WIFI_PREAMBLE_LONG, false, 1, 1, 0, 20, false, false);
+  MpduType mpdutype = NORMAL_MPDU;
 
   Ptr<Packet> pkt = Create<Packet> (1000);
   WifiMacHeader hdr;
@@ -83,14 +108,14 @@ SpectrumWifiPhyBasicTest::MakeSignal (double txPowerWatts)
   hdr.SetType (WIFI_MAC_QOSDATA);
   hdr.SetQosTid (0);
   uint32_t size = pkt->GetSize () + hdr.GetSize () + trailer.GetSerializedSize ();
-  Time txDuration = m_phy->CalculateTxDuration (size, txVector, preamble, m_phy->GetFrequency(), mpdutype, 0);
+  Time txDuration = m_phy->CalculateTxDuration (size, txVector, m_phy->GetFrequency (), mpdutype, 0);
   hdr.SetDuration (txDuration);
 
   pkt->AddHeader (hdr);
   pkt->AddTrailer (trailer);
-  WifiPhyTag tag (txVector, preamble, mpdutype);
+  WifiPhyTag tag (txVector, mpdutype);
   pkt->AddPacketTag (tag);
-  Ptr<SpectrumValue> txPowerSpectrum = WifiSpectrumValueHelper::CreateOfdmTxPowerSpectralDensity (FREQUENCY, CHANNEL_WIDTH, txPowerWatts);
+  Ptr<SpectrumValue> txPowerSpectrum = WifiSpectrumValueHelper::CreateOfdmTxPowerSpectralDensity (FREQUENCY, CHANNEL_WIDTH, txPowerWatts, GUARD_WIDTH);
   Ptr<WifiSpectrumSignalParameters> txParams = Create<WifiSpectrumSignalParameters> ();
   txParams->psd = txPowerSpectrum;
   txParams->txPhy = 0;
@@ -107,7 +132,13 @@ SpectrumWifiPhyBasicTest::SendSignal (double txPowerWatts)
 }
 
 void
-SpectrumWifiPhyBasicTest::SpectrumWifiPhyReceiver (bool rxSucceeded)
+SpectrumWifiPhyBasicTest::SpectrumWifiPhyRxSuccess (Ptr<Packet> p, double snr, WifiTxVector txVector)
+{
+  m_count++;
+}
+
+void
+SpectrumWifiPhyBasicTest::SpectrumWifiPhyRxFailure (Ptr<Packet> p, double snr)
 {
   m_count++;
 }
@@ -127,7 +158,8 @@ SpectrumWifiPhyBasicTest::DoSetup (void)
   m_phy->SetErrorRateModel (error);
   m_phy->SetChannelNumber (CHANNEL_NUMBER);
   m_phy->SetFrequency (FREQUENCY);
-  m_phy->SetPacketReceivedCallback (MakeCallback (&SpectrumWifiPhyBasicTest::SpectrumWifiPhyReceiver, this));
+  m_phy->SetReceiveOkCallback (MakeCallback (&SpectrumWifiPhyBasicTest::SpectrumWifiPhyRxSuccess, this));
+  m_phy->SetReceiveErrorCallback (MakeCallback (&SpectrumWifiPhyBasicTest::SpectrumWifiPhyRxFailure, this));
   //Bug 2460: CcaMode1Threshold default should be set to -62 dBm when using Spectrum
   m_phy->SetCcaMode1Threshold (-62.0);
 }
@@ -138,18 +170,24 @@ SpectrumWifiPhyBasicTest::DoRun (void)
 {
   double txPowerWatts = 0.010;
   // Send packets spaced 1 second apart; all should be received
-  Simulator::Schedule (Seconds (1), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts); 
-  Simulator::Schedule (Seconds (2), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts); 
-  Simulator::Schedule (Seconds (3), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts); 
+  Simulator::Schedule (Seconds (1), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts);
+  Simulator::Schedule (Seconds (2), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts);
+  Simulator::Schedule (Seconds (3), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts);
   // Send packets spaced 1 microsecond second apart; only one should be received
-  Simulator::Schedule (MicroSeconds (4000000), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts); 
-  Simulator::Schedule (MicroSeconds (4000001), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts); 
+  Simulator::Schedule (MicroSeconds (4000000), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts);
+  Simulator::Schedule (MicroSeconds (4000001), &SpectrumWifiPhyBasicTest::SendSignal, this, txPowerWatts);
   Simulator::Run ();
   Simulator::Destroy ();
 
   NS_TEST_ASSERT_MSG_EQ (m_count, 4, "Didn't receive right number of packets");
 }
 
+/**
+ * \ingroup wifi-test
+ * \ingroup tests
+ *
+ * \brief Test Phy Listener
+ */
 class TestPhyListener : public ns3::WifiPhyListener
 {
 public:
@@ -157,11 +195,11 @@ public:
    * Create a test PhyListener
    *
    */
-  TestPhyListener (void) :
-    m_notifyRxStart (0),
-    m_notifyRxEndOk (0),
-    m_notifyRxEndError (0),
-    m_notifyMaybeCcaBusyStart (0)
+  TestPhyListener (void)
+    : m_notifyRxStart (0),
+      m_notifyRxEndOk (0),
+      m_notifyRxEndError (0),
+      m_notifyMaybeCcaBusyStart (0)
   {
   }
   virtual ~TestPhyListener ()
@@ -195,14 +233,19 @@ public:
   virtual void NotifyWakeup (void)
   {
   }
-  uint32_t m_notifyRxStart;
-  uint32_t m_notifyRxEndOk;
-  uint32_t m_notifyRxEndError;
-  uint32_t m_notifyMaybeCcaBusyStart;
+  uint32_t m_notifyRxStart; ///< notify receive start
+  uint32_t m_notifyRxEndOk; ///< notify receive end OK
+  uint32_t m_notifyRxEndError; ///< notify receive end error
+  uint32_t m_notifyMaybeCcaBusyStart; ///< notify maybe CCA busy start
 private:
 };
 
-
+/**
+ * \ingroup wifi-test
+ * \ingroup tests
+ *
+ * \brief Spectrum Wifi Phy Listener Test
+ */
 class SpectrumWifiPhyListenerTest : public SpectrumWifiPhyBasicTest
 {
 public:
@@ -211,7 +254,7 @@ public:
 private:
   virtual void DoSetup (void);
   virtual void DoRun (void);
-  TestPhyListener* m_listener;
+  TestPhyListener* m_listener; ///< listener
 };
 
 SpectrumWifiPhyListenerTest::SpectrumWifiPhyListenerTest ()
@@ -235,7 +278,7 @@ void
 SpectrumWifiPhyListenerTest::DoRun (void)
 {
   double txPowerWatts = 0.010;
-  Simulator::Schedule (Seconds (1), &SpectrumWifiPhyListenerTest::SendSignal, this, txPowerWatts); 
+  Simulator::Schedule (Seconds (1), &SpectrumWifiPhyListenerTest::SendSignal, this, txPowerWatts);
   Simulator::Run ();
 
   NS_TEST_ASSERT_MSG_EQ (m_count, 1, "Didn't receive right number of packets");
@@ -247,6 +290,12 @@ SpectrumWifiPhyListenerTest::DoRun (void)
   delete m_listener;
 }
 
+/**
+ * \ingroup wifi-test
+ * \ingroup tests
+ *
+ * \brief Spectrum Wifi Phy Test Suite
+ */
 class SpectrumWifiPhyTestSuite : public TestSuite
 {
 public:
@@ -260,4 +309,4 @@ SpectrumWifiPhyTestSuite::SpectrumWifiPhyTestSuite ()
   AddTestCase (new SpectrumWifiPhyListenerTest, TestCase::QUICK);
 }
 
-static SpectrumWifiPhyTestSuite spectrumWifiPhyTestSuite;
+static SpectrumWifiPhyTestSuite spectrumWifiPhyTestSuite; ///< the test suite
