@@ -46,6 +46,7 @@ except ImportError:
 #
 interesting_config_items = [
     "NS3_ENABLED_MODULES",
+    "NS3_ENABLED_CONTRIBUTED_MODULES",
     "NS3_MODULE_PATH",
     "NSC_ENABLED",
     "ENABLE_REAL_TIME",
@@ -753,7 +754,7 @@ def run_job_synchronously(shell_command, directory, valgrind, is_python, build_p
             path_cmd = os.path.join (NS3_BUILDDIR, shell_command)
 
     if valgrind:
-        cmd = "valgrind --suppressions=%s --leak-check=full --show-reachable=yes --error-exitcode=2 %s" % (suppressions_path, 
+        cmd = "valgrind --suppressions=%s --leak-check=full --show-reachable=yes --error-exitcode=2 --errors-for-leak-kinds=all %s" % (suppressions_path,
             path_cmd)
     else:
         cmd = path_cmd
@@ -780,18 +781,6 @@ def run_job_synchronously(shell_command, directory, valgrind, is_python, build_p
         print(stderr_results)
         retval = 1
 
-    #
-    # valgrind sometimes has its own idea about what kind of memory management
-    # errors are important.  We want to detect *any* leaks, so the way to do 
-    # that is to look for the presence of a valgrind leak summary section.
-    #
-    # If another error has occurred (like a test suite has failed), we don't 
-    # want to trump that error, so only do the valgrind output scan if the 
-    # test has otherwise passed (return code was zero).
-    #
-    if valgrind and retval == 0 and "== LEAK SUMMARY:" in stderr_results:
-        retval = 2
-    
     if options.verbose:
         print("Return code = ", retval)
         print("stderr = ", stderr_results)
@@ -1153,6 +1142,26 @@ def run_tests():
             example_tests,
             example_names_original,
             python_tests)
+            
+    for module in NS3_ENABLED_CONTRIBUTED_MODULES:
+        # Remove the "ns3-" from the module name.
+        module = module[len("ns3-"):]
+
+        # Set the directories and paths for this example. 
+        module_directory     = os.path.join("contrib", module)
+        example_directory    = os.path.join(module_directory, "examples")
+        examples_to_run_path = os.path.join(module_directory, "test", "examples-to-run.py")
+        cpp_executable_dir   = os.path.join(NS3_BUILDDIR, example_directory)
+        python_script_dir    = os.path.join(example_directory)
+
+        # Parse this module's file.
+        parse_examples_to_run_file(
+            examples_to_run_path,
+            cpp_executable_dir,
+            python_script_dir,
+            example_tests,
+            example_names_original,
+            python_tests)
 
     #
     # If lots of logging is enabled, we can crash Python when it tries to 
