@@ -39,6 +39,7 @@
 #include "ns3/string.h"
 #include "ns3/node-container.h"
 #include "ns3/simple-channel.h"
+#include "ns3/system-path.h"
 
 using namespace ns3;
 
@@ -60,6 +61,7 @@ namespace
   std::string commentStart;        ///< start of code comment
   std::string commentStop;         ///< end of code comment
   std::string copyDoc;             ///< copy (or refer) to docs elsewhere
+  std::string file;                ///< file 
   std::string flagSpanStart;       ///< start of Attribute flag value
   std::string flagSpanStop;        ///< end of Attribute flag value
   std::string functionStart;       ///< start of a method/function
@@ -71,18 +73,19 @@ namespace
   std::string listLineStop;        ///< end unordered list item
   std::string listStart;           ///< start unordered list
   std::string listStop;            ///< end unordered list
+  std::string note;                ///< start a note section
   std::string page;                ///< start a separate page
   std::string reference;           ///< reference tag
   std::string returns;             ///< the return value
   std::string sectionStart;        ///< start of a section or group
   std::string seeAlso;             ///< Reference to other docs
   std::string subSectionStart;     ///< start a new subsection
-  std::string templateArgument;    ///< template argument
-  std::string templArgExplicit;    ///< template argument required
   std::string templArgDeduced;     ///< template argument deduced from function
+  std::string templArgExplicit;    ///< template argument required
+  std::string templateArgument;    ///< template argument
   std::string variable;            ///< variable or class member
 
-} // anonymous namespace
+}  // unnamed namespace
 
 
 /**
@@ -110,6 +113,7 @@ SetMarkup (bool outputText)
       commentStart                 = "===============================================================\n";
       commentStop                  = "";
       copyDoc                      = "  See: ";
+      file                         = "File: ";
       flagSpanStart                = "";
       flagSpanStop                 = "";
       functionStart                = "";
@@ -117,19 +121,20 @@ SetMarkup (bool outputText)
       headingStart                 = "";
       headingStop                  = "";
       indentHtmlOnly               = "";
-      page                         = "Page ";
-      listStart                    = "";
-      listStop                     = "";
       listLineStart                = "    * ";
       listLineStop                 = "";
+      listStart                    = "";
+      listStop                     = "";
+      note                         = "Note: ";
+      page                         = "Page ";
       reference                    = " ";
       returns                      = "  Returns: ";
       sectionStart                 = "Section ";
       seeAlso                      = "  See: ";
       subSectionStart              = "Subsection ";
-      templateArgument             = "Template Arg: ";
       templArgDeduced              = "[deduced]  ";
       templArgExplicit             = "[explicit] ";
+      templateArgument             = "Template Arg: ";
       variable                     = "Variable: ";
     }
   else
@@ -148,6 +153,7 @@ SetMarkup (bool outputText)
       commentStart                 = "/*!\n";
       commentStop                  = "*/\n";
       copyDoc                      = "\\copydoc ";
+      file                         = "\\file ";
       flagSpanStart                = "<span class=\"mlabel\">";
       flagSpanStop                 = "</span>";
       functionStart                = "\\fn ";
@@ -155,19 +161,20 @@ SetMarkup (bool outputText)
       headingStart                 = "<h3>";
       headingStop                  = "</h3>";
       indentHtmlOnly               = "  ";
-      page                         = "\\page ";
-      listStart                    = "<ul>";
-      listStop                     = "</ul>";
       listLineStart                = "<li>";
       listLineStop                 = "</li>";
+      listStart                    = "<ul>";
+      listStop                     = "</ul>";
+      note                         = "\\note ";
+      page                         = "\\page ";
       reference                    = " \\ref ";
       returns                      = "\\returns ";
       sectionStart                 = "\\ingroup ";
       seeAlso                      = "\\see ";
       subSectionStart              = "\\addtogroup ";
-      templateArgument             = "\\tparam ";
       templArgDeduced              = "\\deduced ";
       templArgExplicit             = "\\explicit ";
+      templateArgument             = "\\tparam ";
       variable                     = "\\var ";
     }
 }  // SetMarkup ()
@@ -674,7 +681,7 @@ PrintAttributeValueSection (std::ostream & os,
 /**
  * Print the AttributeValue documentation for a class.
  *
- * This will print documentation for the \p <name>Value class and methods.
+ * This will print documentation for the \p AttributeValue class and methods.
  *
  * \param [in,out] os The output stream.
  * \param [in] name The token to use in defining the accessor name.
@@ -873,11 +880,20 @@ PrintAttributeImplementations (std::ostream & os)
 
   const AttributeDescriptor attributes [] =
     {
+      // Name             Type             see Base  header-file
       // Users of ATTRIBUTE_HELPER_HEADER
       //
       { "Address",        "Address",        true,  "address.h"          },
       { "Box",            "Box",            true,  "box.h"              },
       { "DataRate",       "DataRate",       true,  "data-rate.h"        },
+      { "DsssParameterSet",
+                          "DsssParameterSet",
+                                            true,  "dsss-parameter-set.h"},
+      { "EdcaParameterSet",
+                          "EdcaParameterSet",
+                                            true,  "edca-parameter-set.h"},
+      { "ErpInformation", "ErpInformation", true,  "erp-information.h"  },
+      { "HeCapabilities", "HeCapabilities", true,  "he-capabilities.h"  },
       { "HtCapabilities", "HtCapabilities", true,  "ht-capabilities.h"  },
       { "IeMeshId",       "IeMeshId",       true,  "ie-dot11s-id.h"     },
       { "Ipv4Address",    "Ipv4Address",    true,  "ipv4-address.h"     },
@@ -895,9 +911,11 @@ PrintAttributeImplementations (std::ostream & os)
       { "Ssid",           "Ssid",           true,  "ssid.h"             },
       { "TypeId",         "TypeId",         true,  "type-id.h"          },
       { "UanModesList",   "UanModesList",   true,  "uan-tx-mode.h"      },
-      { "ValueClassTest", "ValueClassTest", false, "" /* outside ns3 */ },
+      // { "ValueClassTest", "ValueClassTest", false, "" /* outside ns3 */ },
       { "Vector2D",       "Vector2D",       true,  "vector.h"           },
       { "Vector3D",       "Vector3D",       true,  "vector.h"           },
+      { "VhtCapabilities","VhtCapabilities",true,  "vht-capabilities.h" },
+      { "VhtOperation",   "VhtOperation",   true,  "vht-operation.h"    },
       { "Waypoint",       "Waypoint",       true,  "waypoint.h"         },
       { "WifiMode",       "WifiMode",       true,  "wifi-mode.h"        },
       
@@ -1248,8 +1266,11 @@ StaticInformation::DoGather (TypeId tid)
 }  // StaticInformation::DoGather ()
 
 
-StaticInformation
-GetTypicalAggregations ()
+/// Register aggregation relationships that are not automatically
+/// detected by this introspection program.  Statements added here
+/// result in more configuration paths being added to the doxygen.
+/// \return instance of StaticInformation with the registered information
+StaticInformation GetTypicalAggregations ()
 {
   NS_LOG_FUNCTION_NOARGS ();
   // The below statements register typical aggregation relationships
@@ -1297,13 +1318,18 @@ GetTypicalAggregations ()
 }  // GetTypicalAggregations ()
 
 
-// Map from TypeId name to tid
+/// Map from TypeId name to tid
 typedef std::map< std::string, int32_t> NameMap;
-typedef NameMap::const_iterator         NameMapIterator;
+typedef NameMap::const_iterator         NameMapIterator; ///< NameMap iterator
 
 
-// Create a map from the class names to their index in the vector of
-// TypeId's so that the names will end up in alphabetical order.
+/**
+ * Create a map from the class names to their index in the vector of
+ * TypeId's so that the names will end up in alphabetical order.
+ *
+ * \param info type names withut type ids
+ * \returns NameMap
+ */
 NameMap
 GetNameMap (const StaticInformation & info)
 {
@@ -1344,6 +1370,12 @@ GetNameMap (const StaticInformation & info)
 }  // GetNameMap ()
 
 
+/**
+ * Print config paths
+ * \param os the output stream
+ * \param info the information
+ * \param tid the type ID
+ */
 void
 PrintConfigPaths (std::ostream & os, const StaticInformation & info,
 		  const TypeId tid)
@@ -1412,8 +1444,22 @@ int main (int argc, char *argv[])
       std::cout << "/* -*- Mode:C++; c-file-style:\"gnu\"; "
 	           "indent-tabs-mode:nil; -*- */\n"
 		<< std::endl;
+      std::cout << "#include \"ns3/log.h\""
+                << std::endl;
     }
-    
+
+  // Doxygen file header
+  std::cout << std::endl;
+  std::cout << commentStart
+            << file << "\n"
+            << sectionStart << "utils\n"
+            << "Doxygen docs generated from the TypeId database.\n"
+            << note << "This file is automatically generated by "
+            << codeWord << "print-introspected-doxygen.cc. Do not edit this file! "
+            << "Edit that file instead.\n"
+            << commentStop
+            << std::endl;
+  
   // Get typical aggregation relationships.
   StaticInformation info = GetTypicalAggregations ();
   
